@@ -1256,12 +1256,10 @@ channel_widget_button_events (GtkWidget *widget,
 	{
 	  if (exclusive || old_state != visible)
 	    {
-	      int x1,y1,x2,y2;
-              GDisplay * gdisp = gdisplay_get_ID (channel_widget->gimage->ID);
-
-	      gdisplay_transform_coords (gdisp, 0, 0, &x1, &y1, FALSE);
-	      gdisplay_transform_coords (gdisp, width, height, &x2, &y2, FALSE);
-	      gdisplay_expose_area (gdisp, x1, y1, x2-x1, y2-y1);
+	      gimage_invalidate_preview (channel_widget->gimage);
+	      gdisplays_update_area (channel_widget->gimage->ID, 0, 0,
+				     channel_widget->gimage->width,
+				     channel_widget->gimage->height);
 	      gdisplays_flush ();
 	    }
 	}
