@@ -27,6 +27,9 @@
    */
 
 /*
+ * 04/03/30 - added function for colourspace conversion 
+ *  [Stefan Klein, kleins at web dot de] 
+ *
  * 98/04/13 - avoid a division by zero when converting an empty gray-scale
  *  image (who would like to do such a thing anyway??)  [Sven ] 
  *
@@ -61,6 +64,7 @@
 #include "appenv.h"
 #include "actionarea.h"
 #include "canvas.h"
+#include "cms.h"
 #include "convert.h"
 #include "drawable.h"
 #include "floating_sel.h"
@@ -75,6 +79,7 @@
 #include "layer_pvt.h"
 #include "../lib/wire/enums.h"
 #include "paint_funcs_area.h"
+#include "rc.h"
 
 static Argument * convert_rgb_invoker               (Argument *);
 static Argument * convert_grayscale_invoker         (Argument *);
@@ -128,9 +133,6 @@ convert_image (GImage *gimage,
   if (floating_layer)
     floating_sel_relax (floating_layer, TRUE);
   
-  /*  Push the image size to the stack  */
-  undo_push_gimage_mod (gimage);
-
   /*  Set the new gimage tag */
   gimage->tag = tag_set_format (gimage->tag, new_format);
 
@@ -191,7 +193,7 @@ convert_image (GImage *gimage,
   layer_invalidate_previews (gimage->ID);
   gimage_invalidate_preview (gimage);
   gdisplays_update_title (gimage->ID);
-  gdisplays_update_full (gimage->ID);
+  gdisplays_update_gimage (gimage->ID);
 }
 
 void
@@ -289,10 +291,54 @@ convert_image_precision (void *gimage_ptr,
   undo_push_group_end (gimage);
 
   /*  shrink wrap and update all views  */
+
   layer_invalidate_previews (gimage->ID);
   gimage_invalidate_preview (gimage);
   gdisplays_update_title (gimage->ID);
-  gdisplays_update_full (gimage->ID);
+  gdisplays_update_gimage (gimage->ID);
+}
+
+
+/* converts the image data to the colorspace given by new_profile
+   only works if gimage has a profile
+   lcms_intent is littlecms intent parameter */
+void
+convert_image_colorspace (void *gimage_ptr, CMSProfile *new_profile,
+                          int lcms_intent)
+{   GImage *image = (GImage *) gimage_ptr;
+
+    /*  get the profiles */
+    GSList *profiles = NULL;
+    CMSProfile *image_profile = gimage_get_cms_profile(image);
+    if (image_profile == NULL)
+    {   g_warning("convert_image_colourspace: No cms profile assigned to gimage. Cannot convert.");
+        return;
+    } 
+    profiles = g_slist_append(profiles, (gpointer)image_profile);
+    profiles = g_slist_append(profiles, (gpointer)new_profile);
+
+    /* assuming all layers have homogenous alpha */
+    DWORD lcms_format_in  = cms_get_lcms_format(
+                              drawable_tag(GIMP_DRAWABLE(image->active_layer)),
+                              gimage_get_cms_profile(image)  );
+    DWORD lcms_format_out = cms_get_lcms_format(
+                              drawable_tag(GIMP_DRAWABLE(image->active_layer)),
+                              new_profile  );
+    if  (!(int)lcms_format_in || !(int)lcms_format_out)
+    {   g_warning("%s:%d %s(): wrong lcms_format %d|%d",
+                  __FILE__,__LINE__,__func__,  lcms_format_in, lcms_format_out);
+        return;
+    } 
+
+    CMSTransform *transform = cms_get_transform(profiles, lcms_format_in, lcms_format_out, lcms_intent, cms_default_flags);
+    g_slist_free(profiles);
+    gimage_transform_colors(image, transform, TRUE);
+    cms_return_transform(transform);    
+
+    gimage_set_cms_profile(image, new_profile);
+
+    /* refresh the profile name in the title bar */
+    gdisplays_update_title (image->ID);
 }
 
 #define MAXNUMCOLORS 256
