@@ -18,6 +18,8 @@
 #ifndef __CONVERT_H__
 #define __CONVERT_H__
 
+#include "cms.h"
+
 #include "procedural_db.h"
 #include "tag.h"
 
@@ -27,6 +29,12 @@ void  convert_to_grayscale  (void *);
 void  convert_to_indexed    (void *);
 void  convert_image_precision (void *gimage, Precision  new_precision);
 
+/* converts the image data to the colorspace given by new_profile
+   only works if gimage has a profile
+   lcms_intent is littlecms intent parameter */
+void convert_image_colorspace (void *gimage_ptr, CMSProfile *new_profile,
+			       int lcms_intent);
+
 /*  Procedure definition and marshalling function  */
 extern ProcRecord convert_rgb_proc;
 extern ProcRecord convert_grayscale_proc;
