@@ -25,6 +25,7 @@
 #include "../canvas.h"
 #include "../drawable.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../general.h"
 #include "../gimage_mask.h"
 #include "../gdisplay.h"
@@ -126,6 +127,9 @@ static void brightness_contrast_float (PixelArea *, PixelArea *, void *);
 static void brightness_contrast_init_transfers_float16(void *);
 static void brightness_contrast_float16 (PixelArea *, PixelArea *, void *);
 
+static void brightness_contrast_init_transfers_bfp (void *);
+static void brightness_contrast_bfp (PixelArea *, PixelArea *, void *);
+
 static void
 brightness_contrast_funcs (Tag drawable_tag)
 {
@@ -147,6 +151,10 @@ brightness_contrast_funcs (Tag drawable_tag)
     brightness_contrast = brightness_contrast_float16;
     brightness_contrast_init_transfers = brightness_contrast_init_transfers_float16;
     break;
+  case PRECISION_BFP:
+    brightness_contrast = brightness_contrast_bfp;
+    brightness_contrast_init_transfers = brightness_contrast_init_transfers_bfp;
+    break;
   default:
     brightness_contrast = NULL;
     brightness_contrast_init_transfers = NULL;
@@ -306,6 +314,36 @@ brightness_contrast_init_transfers_float16 (void * user_data)
 {
 }
 
+static void 
+brightness_contrast_init_transfers_bfp (void * user_data)
+{
+  gint i;
+  BrightnessContrastDialog *bcd = (BrightnessContrastDialog *) user_data;
+  guint16 *brightness_data = (guint16*) pixelrow_data (&brightness_lut);
+  guint16 *contrast_data = (guint16*) pixelrow_data (&contrast_lut);
+  Tag lut_tag = tag_new (PRECISION_BFP, FORMAT_GRAY, ALPHA_NO);
+  gdouble brightness = bcd->brightness;
+  gdouble contrast = bcd->contrast;
+ 
+  /* allocate memory for the transfers if we dont have it yet */
+  if (!brightness_data && !contrast_data)
+  {
+    brightness_data = (guint16 *)g_malloc ( sizeof (guint16) * 65535);
+    contrast_data = (guint16 *)g_malloc ( sizeof (guint16) * 65535 );
+  }
+  
+  /* note: 32768.0 is ONE_BFP in float */
+  for (i = 0; i < 65536; i++)
+    brightness_data[i] = (guint16) (32768.0 * brightness_func ( (gdouble)i/ONE_BFP,
+				      brightness*2.0));
+  for (i = 0; i < 65536; i++)
+    contrast_data[i] = (guint16) (32768.0 * contrast_func ( (double)i/ONE_BFP,
+				      contrast*2.0));
+  
+  pixelrow_init (&brightness_lut, lut_tag, (guchar*)brightness_data, 65536); 
+  pixelrow_init (&contrast_lut, lut_tag, (guchar*)contrast_data, 65536); 
+}
+
 static void 
 brightness_contrast_free_transfers (void)
 {
@@ -515,6 +553,54 @@ brightness_contrast_float16 (PixelArea *src_area,
     }
 }
 
+static void
+brightness_contrast_bfp (PixelArea *src_area,
+		        PixelArea *dest_area,
+		        void        *user_data)
+{
+  Tag src_tag = pixelarea_tag (src_area);
+  Tag dest_tag = pixelarea_tag (dest_area);
+  gint src_num_channels = tag_num_channels (src_tag);
+  gint dest_num_channels = tag_num_channels (dest_tag);
+  guchar *src, *dest;
+  guint16 *s, *d;
+  int has_alpha;
+  int alpha;
+  int w, h, b;
+  BrightnessContrastDialog *bcd = (BrightnessContrastDialog *) user_data;
+  gdouble brightness = (gfloat)bcd->brightness;
+  gdouble contrast = (gfloat)bcd->contrast;
+
+  src = (guchar *)pixelarea_data (src_area);
+  dest = (guchar *)pixelarea_data (dest_area);
+  
+  has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
+  alpha = has_alpha ? src_num_channels - 1 : src_num_channels;
+  h = pixelarea_height (src_area);
+  while (h--)
+    {
+      s = (guint16*)src;
+      d = (guint16*)dest;
+      w = pixelarea_width (src_area);
+      while (w--)
+	{
+          /* note: 32768. is ONE_BFP in float */
+	  for (b = 0; b < alpha; b++)
+	    d[b] = CLAMP(32768. * contrast_func(brightness_func(s[b]/32768., 
+	           2.*brightness), 2.*contrast), 0, 65535);
+
+	  if (has_alpha)
+	    d[alpha] = s[alpha];
+
+	  s += src_num_channels;
+	  d += dest_num_channels; 
+	}
+
+      src += pixelarea_rowstride (src_area);
+      dest += pixelarea_rowstride (dest_area);
+    }
+}
+
 static void
 brightness_contrast_button_press (Tool           *tool,
 				  GdkEventButton *bevent,
