@@ -69,6 +69,9 @@ gimp_brush_generated_compute_float(GimpBrushGenerated *brush,
 static void 
 gimp_brush_generated_compute_float16(GimpBrushGenerated *brush, 	
 				gint width, gint height, gfloat *lookup);
+static void 
+gimp_brush_generated_compute_bfp(GimpBrushGenerated *brush, 	
+				gint width, gint height, gfloat *lookup);
 
 static GimpBrushGeneratedComputeFunc gimp_brush_generated_compute_funcs (Tag t);
 
@@ -401,6 +404,8 @@ gimp_brush_generated_compute_funcs (
       return gimp_brush_generated_compute_float;
     case PRECISION_FLOAT16:
       return gimp_brush_generated_compute_float16;
+    case PRECISION_BFP:
+      return gimp_brush_generated_compute_bfp;
     case PRECISION_NONE:
     default:
       g_warning ("gimp_brush_generated_compute_func: bad precision");
@@ -611,6 +616,56 @@ gimp_brush_generated_compute_u8(
   canvas_portion_unref (gbrush->mask, 0, 0);
 }
 				
+static void
+gimp_brush_generated_compute_bfp( 
+				GimpBrushGenerated *brush,
+				gint width,
+				gint height,
+			  	gfloat *lookup	 
+				)
+{
+  GimpBrush *gbrush = GIMP_BRUSH(brush);
+  guint16 *centerp;
+  gdouble a, d, c, s, tx, ty;
+  gint x,y;
+  gint canvas_w, canvas_h;
+  guint16 *data;
+  gint w = width;
+  gint h = height;
+  Tag tag = tag_new ( default_precision, FORMAT_GRAY, ALPHA_NO ); 
+
+  /* make the canvas */
+  gbrush->mask = canvas_new( tag, width*2 + 1, height*2 + 1, STORAGE_FLAT);
+  canvas_portion_refrw (gbrush->mask,0,0);
+  data = (guint16*)canvas_portion_data (gbrush->mask, 0, 0);  
+  canvas_w = canvas_width (gbrush->mask); 
+  canvas_h = canvas_height (gbrush->mask); 
+  centerp = &data[height*canvas_w + width];
+	
+  s = sin(brush->angle*M_PI/180.0);
+  c = cos(brush->angle*M_PI/180.0);
+
+  /* compute one half and mirror it */
+  for (y = 0; y <= h; y++)
+  {
+    for (x = -w; x <= w; x++)
+    {
+      tx = c*x - s*y;
+      ty = c*y + s*x;
+      ty *= brush->aspect_ratio;
+      d = sqrt(tx*tx + ty*ty);
+      if (d < brush->radius+1)
+	a = lookup[(int)rint(d*OVERSAMPLING)];
+      else
+	a = 0.0;
+      a = MIN(a * 65535.0,65535);
+      centerp[   y*canvas_w + x] = a;
+      centerp[-1*y*canvas_w - x] = a;
+    }
+  }
+  canvas_portion_unref (gbrush->mask, 0, 0);
+}
+
 
 float
 gimp_brush_generated_set_radius (GimpBrushGenerated* brush, float radius)
