@@ -23,6 +23,7 @@
 #include "../color_picker.h"
 #include "../drawable.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../gdisplay.h"
 #include "../info_dialog.h"
 #include "../palette.h"
@@ -55,6 +56,10 @@ static Argument *color_picker_invoker (Argument *);
 static guchar         _color [TAG_MAX_BYTES];
 static PixelRow       color;
 
+static Format prev_format;  	/* For changing picker display when
+				 * we change the format of the image
+				 * Added 8Jul03 Tom Huffmn*/
+
 static CanvasDrawable * active_drawable;
 static int            update_type;
 static InfoDialog *   color_picker_info = NULL;
@@ -143,8 +148,9 @@ color_picker_button_press (Tool           *tool,
   /*  If this is the first invocation of the tool, or a different gdisplay,
    *  create (or recreate) the info dialog...
    */
-  if (tool->state == INACTIVE || gdisp_ptr != tool->gdisp_ptr ||
-      active_drawable != gimage_active_drawable (gdisp->gimage))
+  if (tool->state == INACTIVE || gdisp_ptr != tool->gdisp_ptr   ||
+      active_drawable != gimage_active_drawable (gdisp->gimage) ||
+      prev_format != gimage_format(gdisp->gimage))
     {
       /*  if the dialog exists, free it  */
       if (color_picker_info)
@@ -156,7 +162,7 @@ color_picker_button_press (Tool           *tool,
       active_drawable = gimage_active_drawable (gdisp->gimage);
 
       /*  if the gdisplay is for a color image, the dialog must have RGB  */
-      switch (tag_format (drawable_tag (active_drawable)))
+      switch (prev_format=tag_format (drawable_tag (active_drawable)))
 	{
 	case FORMAT_RGB:
 	  info_dialog_add_field (color_picker_info, "Red", red_buf);
@@ -535,6 +541,53 @@ color_picker_info_update_float16  (
 }
 
 
+static void 
+color_picker_info_update_bfp  (
+                               void
+                               )
+{
+  guint16 * src = (guint16*) pixelrow_data (&color);
+  Tag sample_tag = pixelrow_tag (&color);
+  
+  switch (tag_format (sample_tag))
+    {
+    case FORMAT_RGB:
+      sprintf (red_buf, "%7.6f", BFP_TO_FLOAT (src [0]));
+      sprintf (green_buf, "%7.6f", BFP_TO_FLOAT (src [1]));
+      sprintf (blue_buf, "%7.6f", BFP_TO_FLOAT (src [2]));
+      if (tag_alpha (sample_tag) == ALPHA_YES)
+        sprintf (alpha_buf, "%7.6f", BFP_TO_FLOAT (src [3]));
+      else
+        sprintf (alpha_buf, "N/A");
+      sprintf (hex_buf, "N/A");
+      break;
+
+    case FORMAT_INDEXED:
+      sprintf (index_buf, "N/A");
+      sprintf (alpha_buf, "N/A");
+      sprintf (red_buf, "N/A");
+      sprintf (green_buf, "N/A");
+      sprintf (blue_buf, "N/A");
+      sprintf (hex_buf, "N/A");
+      break;
+
+    case FORMAT_GRAY:
+      sprintf (gray_buf, "%7.6f", BFP_TO_FLOAT (src [0]));
+      if (tag_alpha (sample_tag) == ALPHA_YES)
+        sprintf (alpha_buf, "%7.6f", BFP_TO_FLOAT (src [1]));
+      else
+        sprintf (alpha_buf, "N/A");
+      sprintf (hex_buf, "N/A");
+      break;
+          
+    case FORMAT_NONE:
+    default:
+      g_warning ("bad format");
+      break;
+    }
+}
+
+
 static void 
 color_picker_info_update  (
                            Tool * tool,
@@ -570,6 +623,10 @@ color_picker_info_update  (
 	case PRECISION_FLOAT16:
 	  color_picker_info_update_float16 ();
           break;
+
+	case PRECISION_BFP:
+	  color_picker_info_update_bfp ();
+          break;
 	
 	case PRECISION_NONE:
         default:
@@ -605,7 +662,6 @@ tools_new_color_picker ()
   tool->cursor_update_func = color_picker_cursor_update;
   tool->control_func = color_picker_control;
   tool->preserve = TRUE;
-
   return tool;
 }
 
