@@ -83,6 +83,10 @@ struct Curves
 
 typedef struct CurvesDialog CurvesDialog;
 
+#define CURVES 6
+#define H_SATURATION 5
+#define USE_HISTOGRAM
+
 struct CurvesDialog
 {
   GtkWidget *    shell;
@@ -91,12 +95,16 @@ struct CurvesDialog
   GtkWidget *    yrange;
   GtkWidget *    graph;
   GdkPixmap *    pixmap;
+  #ifdef USE_HISTOGRAM
+  Histogram *    histogram;
+  #endif
 
   CanvasDrawable * drawable;
   ImageMap       image_map;
   int            color;
   int            channel;
   gint           preview;
+  int            is_Lab;
 
   int            grab_point;
   int            last;
@@ -105,12 +113,12 @@ struct CurvesDialog
   int            curve_type;
 
   /* points and curves for ui 0-255 based */
-  int            points[5][17][2];
-  unsigned char  curve[5][256];
+  int            points[CURVES][17][2];
+  unsigned char  curve[CURVES][256];
 };
 
 /* curves for image data */
-static PixelRow curve[5];
+static PixelRow curve[CURVES];
 
 typedef double CRMatrix[4][4];
 
@@ -132,6 +140,7 @@ static void            curves_red_callback       (GtkWidget *, gpointer);
 static void            curves_green_callback     (GtkWidget *, gpointer);
 static void            curves_blue_callback      (GtkWidget *, gpointer);
 static void            curves_alpha_callback     (GtkWidget *, gpointer);
+static void            curves_saturation_callback(GtkWidget *, gpointer);
 static void            curves_smooth_callback    (GtkWidget *, gpointer);
 static void            curves_free_callback      (GtkWidget *, gpointer);
 static void            curves_reset_callback     (GtkWidget *, gpointer);
@@ -253,7 +262,7 @@ static void
 curves_free_curves (void *user_data)
 {
   gint i;
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
   { 
      guchar* data =  pixelrow_data (&curve[i]);
      g_free (data);
@@ -273,10 +282,12 @@ curves_u8 (PixelArea *src_area,
   CurvesDialog *cd;
   guchar *src, *dest;
   guint8 *s, *d;
-  guint8 *curve_red, *curve_green, *curve_blue, *curve_value, *curve_alpha = NULL;
+  guint8 *curve_red, *curve_green, *curve_blue, *curve_value, *curve_alpha = NULL, *curve_satur = NULL;
   int has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   int alpha = has_alpha ? src_num_channels - 1 : src_num_channels;
   int w, h;
+  double dist, arc;
+  int tmp1,tmp2;
 
   cd = (CurvesDialog *) user_data;
 
@@ -289,6 +300,8 @@ curves_u8 (PixelArea *src_area,
   curve_value = (guint8*)pixelrow_data (&curve[HISTOGRAM_VALUE]);
   if (has_alpha)
     curve_alpha = (guint8*)pixelrow_data (&curve[HISTOGRAM_ALPHA]);
+  if (cd->is_Lab)
+    curve_satur = (guint8*)pixelrow_data (&curve[H_SATURATION]);
 
   h = pixelarea_height (src_area);
   while (h--)
@@ -316,6 +329,41 @@ curves_u8 (PixelArea *src_area,
 	  if (has_alpha) 
 	    d[alpha] = curve_alpha[s[alpha]];
 
+          /* radial scaling of CIE*a and CIE*b + clamp on the square borders */
+	  if (cd->is_Lab) { /* calcultation relative to the gray axix */
+            arc = atan2( (double)(d[BLUE_PIX] - 128.0)
+                        ,(double)(d[GREEN_PIX] - 128.0));
+            dist = pow( ((double)d[GREEN_PIX] - 128.0)*((double)d[GREEN_PIX] - 128.0)
+                       +((double)d[BLUE_PIX] - 127.0)*((double)d[BLUE_PIX] - 128.0) , 0.5);
+            tmp1= cos(arc) * curve_satur[(int)(dist*1.2+0.5)] / 1.2;
+	    tmp2= sin(arc) * curve_satur[(int)(dist*1.2+0.5)] / 1.2;
+            if (tmp1 <= 127 && tmp1 >= -128) {
+              d[GREEN_PIX] = tmp1 + 128;
+            } else { /* clamp */
+              if (tmp1 > 127) {
+                tmp2 = 128.0 / (double)tmp1 * tmp2;
+                tmp1 = 127;
+              } else {
+                tmp2 = -128.0 / (double)tmp1 * tmp2;
+                tmp1 = -128;
+              }
+              d[GREEN_PIX] = tmp1 + 128;
+            }
+            if (tmp2 <= 127 && tmp2 >= -128) {
+              d[BLUE_PIX] = tmp2 + 128;
+            } else { /* clamp */
+              if (tmp2 > 127) {
+                tmp1 = 128.0 / (double)tmp2 * tmp1;
+                tmp2 = 127;
+              } else {
+                tmp1 = -128.0 / (double)tmp2 * tmp1;
+                tmp2 = -128;
+              }
+              d[GREEN_PIX] = tmp1 + 128;
+              d[BLUE_PIX] = tmp2 + 128;
+            }
+          }
+
 	  s += src_num_channels;
 	  d += dest_num_channels;
 	}
@@ -331,7 +379,7 @@ curves_alloc_curves_u8 (void * user_data)
   gint i;
   Tag tag = tag_new (PRECISION_U8, FORMAT_GRAY, ALPHA_NO);
   
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
   { 
        guint8* data = (guint8*) g_malloc (sizeof(guint8) * 256 );
        pixelrow_init (&curve[i], tag, (guchar*)data, 256);
@@ -362,10 +410,12 @@ curves_u16 (PixelArea *src_area,
   CurvesDialog *cd;
   guchar *src, *dest;
   guint16 *s, *d;
-  guint16 *curve_red, *curve_green, *curve_blue, *curve_value, *curve_alpha = NULL;
+  guint16 *curve_red, *curve_green, *curve_blue, *curve_value, *curve_alpha = NULL, *curve_satur = NULL;
   int has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
   int alpha = has_alpha ? src_num_channels - 1 : src_num_channels;
   int w, h;
+  double dist, arc;
+  int tmp1,tmp2;
 
   cd = (CurvesDialog *) user_data;
 
@@ -378,6 +428,8 @@ curves_u16 (PixelArea *src_area,
   curve_value = (guint16*)pixelrow_data (&curve[HISTOGRAM_VALUE]);
   if (has_alpha)
     curve_alpha = (guint16*)pixelrow_data (&curve[HISTOGRAM_ALPHA]);
+  if (cd->is_Lab)
+    curve_satur = (guint16*)pixelrow_data (&curve[H_SATURATION]);
 
   h = pixelarea_height (src_area);
   while (h--)
@@ -405,6 +457,41 @@ curves_u16 (PixelArea *src_area,
 	  if (has_alpha) 
 	    d[alpha] = curve_alpha[s[alpha]];
 
+          /* radial scaling of CIE*a and CIE*b + clamp on the square borders */
+	  if (cd->is_Lab) { /* calcultation relative to the gray axix */
+            arc = atan2( (double)(d[BLUE_PIX] - 32768.0)
+                        ,(double)(d[GREEN_PIX] - 32768.0));
+            dist = pow( ((double)d[GREEN_PIX] - 32768.0)*((double)d[GREEN_PIX] - 32768.0)
+                       +((double)d[BLUE_PIX] - 32768.0)*((double)d[BLUE_PIX] - 32768.0) , 0.5);
+            tmp1= cos(arc) * curve_satur[(int)(dist*1.4+0.5)] / 1.4;
+	    tmp2= sin(arc) * curve_satur[(int)(dist*1.4+0.5)] / 1.4;
+            if (tmp1 <= 32767 && tmp1 >= -32768) {
+              d[GREEN_PIX] = tmp1 + 32768;
+            } else { /* clamp */
+              if (tmp1 > 32767) {
+                tmp2 = 32768.0 / (double)tmp1 * tmp2;
+                tmp1 = 32767;
+              } else {
+                tmp2 = -32768.0 / (double)tmp1 * tmp2;
+                tmp1 = -32768;
+              }
+              d[GREEN_PIX] = tmp1 + 32768;
+            }
+            if (tmp2 <= 32767 && tmp2 >= -32768) {
+              d[BLUE_PIX] = tmp2 + 32768;
+            } else { /* clamp */
+              if (tmp2 > 32767) {
+                tmp1 = 32768.0 / (double)tmp2 * tmp1;
+                tmp2 = 32767;
+              } else {
+                tmp1 = -32768.0 / (double)tmp2 * tmp1;
+                tmp2 = -32768;
+              }
+              d[GREEN_PIX] = tmp1 + 32768;
+              d[BLUE_PIX] = tmp2 + 32768;
+            }
+          }
+
 	  s += src_num_channels;
 	  d += dest_num_channels;
 	}
@@ -420,7 +507,7 @@ curves_alloc_curves_u16 (void * user_data)
   gint i;
   Tag tag = tag_new (PRECISION_U16, FORMAT_GRAY, ALPHA_NO);
   
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
   { 
        guint16* data = (guint16*) g_malloc (sizeof(guint16) * 65536 );
        pixelrow_init (&curve[i], tag, (guchar*)data, 65536);
@@ -563,6 +650,7 @@ static MenuItem channel_items[] =
   { "Green", 0, 0, curves_green_callback, NULL, NULL, NULL },
   { "Blue", 0, 0, curves_blue_callback, NULL, NULL, NULL },
   { "Alpha", 0, 0, curves_alpha_callback, NULL, NULL, NULL },
+  { "Saturation", 0, 0, curves_saturation_callback, NULL, NULL, NULL },
   { NULL, 0, 0, NULL, NULL, NULL, NULL }
 };
 
@@ -587,6 +675,7 @@ curves_initialize (void *gdisp_ptr)
       return;
     }
 
+  histogram_histogram_funcs (gimage_tag (gdisp->gimage));
   /* Set up the function pointers for this tag */
   curves_funcs( drawable_tag (gimage_active_drawable (gdisp->gimage)));
 
@@ -594,28 +683,36 @@ curves_initialize (void *gdisp_ptr)
     g_message ("Curves for this image type cannot be adjusted.");
     return;
   }
-  
+
   /*  The curves dialog  */
   if (!curves_dialog)
     curves_dialog = curves_new_dialog ();
   else if (!GTK_WIDGET_VISIBLE (curves_dialog->shell))
     gtk_widget_show (curves_dialog->shell);
 
+  /* Look for Lab colorspace */ 
+  curves_dialog->is_Lab = FALSE;
+  if (gdisp->gimage->cms_profile) {
+    CMSProfileInfo *pi = cms_get_profile_info(gdisp->gimage->cms_profile);
+    if (strstr(pi->color_space_name, "Lab") != 0)
+      curves_dialog->is_Lab = TRUE;
+  }
+  
   /* allocate all curves data */
   (*curves_alloc_curves) (curves_dialog);
   
   /* Initialize all the curves for image data */ 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     (*curves_init_curve) (i);
 
   /* Initialize all the curves for the ui  */
   curves_dialog->channel = HISTOGRAM_VALUE;
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     for (j = 0; j < 256; j++)
       curves_dialog->curve[i][j] = j;
 
   curves_dialog->grab_point = -1;
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     {
       for (j = 0; j < 17; j++)
 	{
@@ -632,12 +729,33 @@ curves_initialize (void *gdisp_ptr)
   curves_dialog->color = drawable_color ( (curves_dialog->drawable));
   curves_dialog->image_map = image_map_create (gdisp_ptr, curves_dialog->drawable);
 
+  { if (curves_dialog->is_Lab) {
+      gtk_widget_hide( channel_items[0].widget);
+      gtk_widget_show( channel_items[5].widget);
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[1].widget)->child),
+                          "CIE Lightness");
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[2].widget)->child),
+                          "CIE a");
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[3].widget)->child),
+                          "CIE b");
+    } else {
+      gtk_widget_show( channel_items[0].widget);
+      gtk_widget_hide( channel_items[5].widget);
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[1].widget)->child),
+                          "Red");
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[2].widget)->child),
+                          "Green");
+      gtk_label_set_text (GTK_LABEL(GTK_BIN (channel_items[3].widget)->child),
+                          "Blue");
+    }
+  }
+  
   /* check for alpha channel */
   if (drawable_has_alpha ( (curves_dialog->drawable)))
-    gtk_widget_set_sensitive( channel_items[4].widget, TRUE);
+    gtk_widget_show( channel_items[4].widget);
   else 
-    gtk_widget_set_sensitive( channel_items[4].widget, FALSE);
-  
+    gtk_widget_hide( channel_items[4].widget);
+
   /*  hide or show the channel menu based on image type  */
   if (curves_dialog->color)
     for (i = 0; i < 4; i++) 
@@ -651,6 +769,14 @@ curves_initialize (void *gdisp_ptr)
   gtk_option_menu_set_history ( GTK_OPTION_MENU (curves_dialog->channel_menu), 0);
   */
 
+  #ifdef USE_HISTOGRAM
+  histogram_update (curves_dialog->histogram,
+		    curves_dialog->drawable,
+		    histogram_histogram_info,
+		    (void *) curves_dialog->histogram);
+  histogram_range (curves_dialog->histogram, -1, -1);
+  #endif
+
   if (!GTK_WIDGET_VISIBLE (curves_dialog->shell))
     gtk_widget_show (curves_dialog->shell);
 
@@ -702,11 +828,11 @@ curves_new_dialog ()
   cd->curve_type = SMOOTH;
   cd->pixmap = NULL;
   cd->channel = HISTOGRAM_VALUE;
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     for (j = 0; j < 256; j++)
       cd->curve[i][j] = j;
 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     channel_items [i].user_data = (gpointer) cd;
   for (i = 0; i < 2; i++)
     curve_type_items [i].user_data = (gpointer) cd;
@@ -742,7 +868,7 @@ curves_new_dialog ()
   gtk_option_menu_set_menu (GTK_OPTION_MENU (cd->channel_menu), menu);
 
   /*  The table for the yrange and the graph  */
-  table = gtk_table_new (2, 2, FALSE);
+  table = gtk_table_new (3, 2, FALSE);
   gtk_container_border_width (GTK_CONTAINER (table), 2);
   gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 
@@ -798,6 +924,23 @@ curves_new_dialog ()
   gtk_widget_show (frame);
   gtk_widget_show (table);
 
+  /*  The curves histogram  */
+#ifdef USE_HISTOGRAM
+  frame = gtk_frame_new (NULL);
+  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_IN);
+  gtk_table_attach (GTK_TABLE (table), frame, 1, 2, 2, 3,
+		    GTK_EXPAND, GTK_EXPAND, 0, 0);
+
+  cd->histogram = histogram_create (GRAPH_WIDTH + RADIUS * 2,
+                                    64 + RADIUS * 2, 256,
+				    histogram_histogram_range, (void *) cd);
+  cd->histogram->channel_menu = cd->channel_menu;
+  //cd->histogram->color_option_items = channel_items;
+  gtk_container_add (GTK_CONTAINER (frame), cd->histogram->histogram_widget);
+  gtk_widget_show (cd->histogram->histogram_widget);
+  gtk_widget_show (frame);
+#endif
+
   /*  Horizontal box for preview  */
   hbox = gtk_hbox_new (FALSE, 2);
   gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
@@ -974,7 +1117,7 @@ curves_calculate_image_data_curves (CurvesDialog *cd)
   /* reset all the function pointers to this tag type */
   curves_funcs (drawable_tag (cd->drawable));
 
-  for (k = 0; k < 5; k++)  
+  for (k = 0; k < CURVES; k++)  
   {
     if (cd->curve_type == GFREE) 
       (*curves_calculate_free_curve)(cd->curve[k], &curve[k]); 
@@ -1333,6 +1476,13 @@ curves_preview (CurvesDialog *cd)
   image_map_apply (cd->image_map, curves, (void *) cd);
 
   active_tool->preserve = FALSE;  /* All done */
+
+  #ifdef USE_HISTOGRAM_  // allways one moment before image_map_apply
+  histogram_update (curves_dialog->histogram,
+		    curves_dialog->drawable,
+		    histogram_histogram_info,
+		    (void *) curves_dialog->histogram);
+  #endif
 }
 
 static void
@@ -1348,6 +1498,9 @@ curves_value_callback (GtkWidget *w,
       cd->channel = HISTOGRAM_VALUE;
       curves_update (cd, GRAPH | DRAW);
     }
+  #ifdef USE_HISTOGRAM
+  histogram_channel (cd->histogram, HISTOGRAM_VALUE);
+  #endif
 }
 
 static void
@@ -1363,6 +1516,9 @@ curves_red_callback (GtkWidget *w,
       cd->channel = HISTOGRAM_RED;
       curves_update (cd, GRAPH | DRAW);
     }
+  #ifdef USE_HISTOGRAM
+  histogram_channel (cd->histogram, HISTOGRAM_RED);
+  #endif
 }
 
 static void
@@ -1378,6 +1534,9 @@ curves_green_callback (GtkWidget *w,
       cd->channel = HISTOGRAM_GREEN;
       curves_update (cd, GRAPH | DRAW);
     }
+  #ifdef USE_HISTOGRAM
+  histogram_channel (cd->histogram, HISTOGRAM_GREEN);
+  #endif
 }
 
 static void
@@ -1393,6 +1552,9 @@ curves_blue_callback (GtkWidget *w,
       cd->channel = HISTOGRAM_BLUE;
       curves_update (cd, GRAPH | DRAW);
     }
+  #ifdef USE_HISTOGRAM
+  histogram_channel (cd->histogram, HISTOGRAM_BLUE);
+  #endif
 }
 
 static void
@@ -1408,6 +1570,24 @@ curves_alpha_callback (GtkWidget *w,
       cd->channel = HISTOGRAM_ALPHA;
       curves_update (cd, GRAPH | DRAW);
     }
+  #ifdef USE_HISTOGRAM
+  histogram_channel (cd->histogram, HISTOGRAM_ALPHA);
+  #endif
+}
+
+static void
+curves_saturation_callback (GtkWidget *w,
+		      gpointer   client_data)
+{
+  CurvesDialog *cd;
+
+  cd = (CurvesDialog *) client_data;
+
+  if (cd->channel != H_SATURATION)
+    {
+      cd->channel = H_SATURATION;
+      curves_update (cd, GRAPH | DRAW);
+    }
 }
 
 static void
@@ -1914,11 +2094,11 @@ file_dialog_ok_callback (GtkWidget *widget,
 
   if (load_save)
     {
-      f = fopen (filename, "rt");
+      f = fopen (filename, "r");
 
       if (!f)
 	{
-	  g_message ("Unable to open file %s", filename);
+	  g_message ("Unable to open file: %s", filename);
 	  return;
 	}
 
@@ -1932,11 +2112,11 @@ file_dialog_ok_callback (GtkWidget *widget,
     }
   else
     {
-      f = fopen (filename, "wt");
+      f = fopen (filename, "w");
 
       if (!f)
 	{
-	  g_message ("Unable to open file %s", filename);
+	  g_message ("Unable to open file: %s", filename);
 	  return;
 	}
 
@@ -1961,8 +2141,8 @@ curves_read_from_file (FILE *f)
   gint  i, j;
   gint  fields;
   gchar buf[50];
-  gint  index[5][17];
-  gint  value[5][17];
+  gint  index[CURVES][17];
+  gint  value[CURVES][17];
   
   if (!fgets (buf, 50, f))
     return FALSE;
@@ -1970,7 +2150,7 @@ curves_read_from_file (FILE *f)
   if (strcmp (buf, "# GIMP Curves File\n") != 0)
     return FALSE;
 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     {
       for (j = 0; j < 17; j++)
 	{
@@ -1983,7 +2163,7 @@ curves_read_from_file (FILE *f)
 	}
     }
 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     {
       curves_dialog[i].curve_type = SMOOTH;
       for (j = 0; j < 17; j++)
@@ -2013,7 +2193,7 @@ curves_write_to_file (FILE *f)
   gint   i, j;
   gint32 index;
 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     if (curves_dialog[i].curve_type == GFREE)
       {  
 	/*  pick representative points from the curve and make them control points  */
@@ -2027,7 +2207,7 @@ curves_write_to_file (FILE *f)
   
   fprintf (f, "# GIMP Curves File\n");
 
-  for (i = 0; i < 5; i++)
+  for (i = 0; i < CURVES; i++)
     {
       for (j = 0; j < 17; j++)
 	fprintf (f, "%d %d ", curves_dialog->points[i][j][0], 
@@ -2098,7 +2278,7 @@ ProcRecord curves_spline_proc =
   PDB_INTERNAL,
 
   /*  Input arguments  */
-  5,
+  CURVES,
   curves_spline_args,
 
   /*  Output arguments  */
@@ -2182,11 +2362,11 @@ curves_spline_invoker (Argument *args)
   /*  arrange to modify the curves  */
   if (success)
     {
-      for (i = 0; i < 5; i++)
+      for (i = 0; i < CURVES; i++)
 	for (j = 0; j < 256; j++)
 	  cd.curve[i][j] = j;
 
-      for (i = 0; i < 5; i++)
+      for (i = 0; i < CURVES; i++)
 	for (j = 0; j < 17; j++)
 	  {
 	    cd.points[i][j][0] = -1;
@@ -2210,7 +2390,7 @@ curves_spline_invoker (Argument *args)
       (*curves_alloc_curves) (&cd);
       
       /* Initialize all the curves for image data */ 
-      for (i = 0; i < 5; i++)
+      for (i = 0; i < CURVES; i++)
 	(*curves_init_curve) (i);
 
       /* calculate curves for the image data */
@@ -2273,7 +2453,7 @@ ProcRecord curves_explicit_proc =
   PDB_INTERNAL,
 
   /*  Input arguments  */
-  5,
+  CURVES,
   curves_explicit_args,
 
   /*  Output arguments  */
@@ -2356,7 +2536,7 @@ curves_explicit_invoker (Argument *args)
   /*  arrange to modify the curves  */
   if (success)
     {
-      for (i = 0; i < 5; i++)
+      for (i = 0; i < CURVES; i++)
 	for (j = 0; j < 256; j++)
 	  cd.curve[i][j] = j;
 
@@ -2373,7 +2553,7 @@ curves_explicit_invoker (Argument *args)
       (*curves_alloc_curves) (&cd);
       
       /* Initialize all the curves for image data */ 
-      for (i = 0; i < 5; i++)
+      for (i = 0; i < CURVES; i++)
 	(*curves_init_curve) (i);
 
       /* calculate curves for the image data */
