@@ -127,6 +127,9 @@ static void gamma_expose_float (PixelArea *, PixelArea *, void *);
 static void gamma_expose_init_transfers_float16(void *);
 static void gamma_expose_float16 (PixelArea *, PixelArea *, void *);
 
+static void gamma_expose_init_transfers_bfp (void *);
+static void gamma_expose_bfp (PixelArea *, PixelArea *, void *);
+
 static void
 gamma_expose_funcs (Tag drawable_tag)
 {
@@ -148,6 +151,10 @@ gamma_expose_funcs (Tag drawable_tag)
     gamma_expose = gamma_expose_float16;
     gamma_expose_init_transfers = gamma_expose_init_transfers_float16;
     break;
+  case PRECISION_BFP:
+    gamma_expose = gamma_expose_bfp;
+    gamma_expose_init_transfers = gamma_expose_init_transfers_bfp;
+    break;
   default:
     gamma_expose = NULL;
     gamma_expose_init_transfers = NULL;
@@ -277,6 +284,35 @@ gamma_expose_free_transfers (void)
   pixelrow_init (&expose_lut, tag_null(), NULL, 0);
 }
 
+static void 
+gamma_expose_init_transfers_bfp (void * user_data)
+{
+  gint i;
+  GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
+  guint16 *gamma_data = (guint16*) pixelrow_data (&gamma_lut);
+  guint16 *expose_data = (guint16*) pixelrow_data (&expose_lut);
+  Tag lut_tag = tag_new (PRECISION_BFP, FORMAT_GRAY, ALPHA_NO);
+  gdouble gamma = bcd->gamma;
+  gdouble expose = bcd->expose;
+ 
+  /* allocate memory for the transfers if we dont have it yet */
+  if (!gamma_data && !expose_data)
+  {
+    gamma_data = (guint16 *)g_malloc ( sizeof (guint16) * 65536 );
+    expose_data = (guint16 *)g_malloc ( sizeof (guint16) * 65536 );
+  }
+  
+  for (i = 0; i < 65536; i++)
+    gamma_data[i] = (guint16) (65535.0 * gamma_func ( (gdouble)i/65535.0,
+				      gamma*2.0));
+  for (i = 0; i < 65536; i++)
+    expose_data[i] = (guint16) (65535.0 * expose_func ( (double)i/65535.0,
+				      expose*2.0));
+  
+  pixelrow_init (&gamma_lut, lut_tag, (guchar*)gamma_data, 65536); 
+  pixelrow_init (&expose_lut, lut_tag, (guchar*)expose_data, 65536); 
+}
+
 static void
 gamma_expose_u8 (PixelArea *src_area,
 		        PixelArea *dest_area,
@@ -476,6 +512,54 @@ gamma_expose_float16 (PixelArea *src_area,
     }
 }
 
+static void
+gamma_expose_bfp (PixelArea *src_area,
+		        PixelArea *dest_area,
+		        void        *user_data)
+{
+  Tag src_tag = pixelarea_tag (src_area);
+  Tag dest_tag = pixelarea_tag (dest_area);
+  gint src_num_channels = tag_num_channels (src_tag);
+  gint dest_num_channels = tag_num_channels (dest_tag);
+  guchar *src, *dest;
+  guint16 *s, *d;
+  int has_alpha;
+  int alpha;
+  int w, h, b;
+  GammaExposeDialog *bcd = (GammaExposeDialog *) user_data;
+  gdouble gamma = (gfloat)bcd->gamma;
+  gdouble expose = (gfloat)bcd->expose;
+
+  src = (guchar *)pixelarea_data (src_area);
+  dest = (guchar *)pixelarea_data (dest_area);
+  
+  has_alpha = tag_alpha (src_tag) == ALPHA_YES ? TRUE: FALSE;
+  alpha = has_alpha ? src_num_channels - 1 : src_num_channels;
+  h = pixelarea_height (src_area);
+  while (h--)
+    {
+      s = (guint16*)src;
+      d = (guint16*)dest;
+      w = pixelarea_width (src_area);
+      while (w--)
+	{
+	  // Added by IMAGEWORKS thedoug (01/02)
+	  for (b = 0; b < alpha; b++)
+	    d[b] = CLAMP(65535 * expose_func(gamma_func(s[b]/65535., 
+	           2.*gamma), 2.*expose), 0, 65535);
+
+	  if (has_alpha)
+	    d[alpha] = s[alpha];
+
+	  s += src_num_channels;
+	  d += dest_num_channels; 
+	}
+
+      src += pixelarea_rowstride (src_area);
+      dest += pixelarea_rowstride (dest_area);
+    }
+}
+
 static void
 gamma_expose_button_press (Tool           *tool,
 				  GdkEventButton *bevent,
@@ -677,7 +761,7 @@ gamma_expose_new_dialog ()
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
 		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
 
-  data = gtk_adjustment_new (2.2, -10, 10, .001, .01, .01);
+  data = gtk_adjustment_new (2.2, -10, 10, .001, .01, 0.0);
   bcd->gamma_data = GTK_ADJUSTMENT (data);
   slider = gtk_hscale_new (GTK_ADJUSTMENT (data));
   gtk_widget_set_usize (slider, SLIDER_WIDTH, SLIDER_HEIGHT);
@@ -711,7 +795,7 @@ gamma_expose_new_dialog ()
   gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
 		    GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 2, 2);
 
-  data = gtk_adjustment_new (0, -10, 10, .01, .01, .01);
+  data = gtk_adjustment_new (0, -10, 10, .01, .01, 0.0);
   bcd->expose_data = GTK_ADJUSTMENT (data);
   slider = gtk_hscale_new (GTK_ADJUSTMENT (data));
   gtk_widget_set_usize (slider, SLIDER_WIDTH, SLIDER_HEIGHT);
@@ -938,9 +1022,9 @@ gamma_expose_expose_scale_update (GtkAdjustment *adjustment,
 					   gpointer       data)
 {
   GammaExposeDialog *bcd;
-
+  int i;
   bcd = (GammaExposeDialog *) data;
-
+  i=4;
   if (bcd->expose != adjustment->value)
     {
       bcd->expose = adjustment->value;
