@@ -406,6 +406,7 @@ static void  histogram_histogram_info_u8 (PixelArea*, PixelArea*, HistogramValue
 static void  histogram_histogram_info_u16 (PixelArea*, PixelArea*, HistogramValues, void *);
 static void  histogram_histogram_info_float (PixelArea*, PixelArea*, HistogramValues, void *);
 static void  histogram_histogram_info_float16 (PixelArea*, PixelArea*, HistogramValues, void *);
+static void  histogram_histogram_info_bfp (PixelArea*, PixelArea*, HistogramValues, void *);
 
 #if 0
 typedef void (*HistogramToolHistogramRangeFunc)(int, int, int, HistogramValues, void *);
@@ -414,6 +415,7 @@ static void       histogram_histogram_range_u8 (int, int, int, HistogramValues,
 static void       histogram_histogram_range_u16 (int, int, int, HistogramValues, void *);
 static void       histogram_histogram_range_float (int, int, int, HistogramValues, void *);
 static void       histogram_histogram_range_float16 (int, int, int, HistogramValues, void *);
+static void       histogram_histogram_range_bfp (int, int, int, HistogramValues, void *);
 
 typedef void (*HistogramUpdataFunc)(Histogram *, int, int);
 static HistogramUpdataFunc histogram_dialog_update;
@@ -421,6 +423,7 @@ static void       histogram_dialog_update_u8 (Histogram *, int, int);
 static void       histogram_dialog_update_u16 (Histogram *, int, int);
 static void       histogram_dialog_update_float (Histogram *, int, int);
 static void       histogram_dialog_update_float16 (Histogram *, int, int);
+static void       histogram_dialog_update_bfp (Histogram *, int, int);
 
 /* static Argument * histogram_invoker (Argument *args); */
 
@@ -461,6 +464,11 @@ histogram_histogram_funcs (Tag tag)
      histogram_dialog_update = histogram_dialog_update_float16; 
      histogram_histogram_range = histogram_histogram_range_float16;
      break;
+  case PRECISION_BFP:
+     histogram_histogram_info = histogram_histogram_info_bfp; 
+     histogram_dialog_update = histogram_dialog_update_bfp; 
+     histogram_histogram_range = histogram_histogram_range_bfp;
+     break;
   default:
      histogram_histogram_info = NULL; 
      histogram_dialog_update = NULL; 
@@ -834,6 +842,96 @@ histogram_histogram_info_float16 (PixelArea *src_area,
      }
 }
 
+static void
+histogram_histogram_info_bfp (PixelArea       *src_area,
+			       PixelArea       *mask_area,
+			       HistogramValues  values,
+			       void            *user_data)
+{
+  Histogram *htd = (Histogram *) user_data;
+  gint bins;
+  int                  w, h;
+  guchar *src, *mask = NULL;
+  guint16 *s, *m = NULL;
+  gint    value, red, green, blue;
+  gfloat  scale = 1./65535.;
+  Tag                  src_tag = pixelarea_tag (src_area);
+  gint                 s_num_channels = tag_num_channels (src_tag);
+  gint                 alpha = tag_alpha (src_tag);
+  Tag                  mask_tag = pixelarea_tag (mask_area);
+  gint                 m_num_channels = tag_num_channels (mask_tag);
+  gint 		       value_bin=0, red_bin, green_bin, blue_bin;
+  HistogramPrivate *histogram_p = (HistogramPrivate *) htd->private_part;
+  bins = histogram_p->bins;
+  h = pixelarea_height (src_area);
+  src = (guchar*)pixelarea_data (src_area);
+
+  if (mask_area)
+    mask = (guchar*)pixelarea_data (mask_area);
+
+   while (h--)
+     {
+       w = pixelarea_width (src_area);
+       s = (guint16*)src;
+
+       if (mask_area)
+	 m = (guint16*)mask;
+
+       while (w--)
+	 {
+	   if (histogram_p->color)
+	     {
+	       value = MAX (s[RED_PIX], s[GREEN_PIX]);
+	       value = MAX (value, s[BLUE_PIX]);
+	       red   = s[RED_PIX];
+	       green = s[GREEN_PIX];
+	       blue  = s[BLUE_PIX];
+
+	       /* figure out which bin to place it in */ 
+  
+	       value_bin = bins*(double)value*scale;
+	       red_bin = bins*(double)red*scale;   	
+	       green_bin = bins*(double)green*scale;
+	       blue_bin = bins*(double)blue*scale;
+
+	       if (mask_area)
+		 {
+		   values[HISTOGRAM_VALUE][value_bin] += (double) *m * scale;
+		   values[HISTOGRAM_RED][red_bin]     += (double) *m * scale;
+		   values[HISTOGRAM_GREEN][green_bin] += (double) *m * scale;
+		   values[HISTOGRAM_BLUE][blue_bin]   += (double) *m * scale;
+		 }
+	       else
+		 {
+		   values[HISTOGRAM_VALUE][value_bin] += 1.0;
+		   values[HISTOGRAM_RED][red_bin]     += 1.0;
+		   values[HISTOGRAM_GREEN][green_bin] += 1.0;
+		   values[HISTOGRAM_BLUE][blue_bin]   += 1.0;
+		 }
+	     }
+	   else
+	     {
+	       value = s[GRAY_PIX];
+	       value_bin = bins*(double)value*scale;
+	       if (mask_area)
+		 values[HISTOGRAM_VALUE][value_bin] += (double) *m * scale;
+	       else
+		 values[HISTOGRAM_VALUE][value_bin] += 1.0;
+	     }
+
+	   s += s_num_channels;
+
+	   if (mask_area)
+	     m += m_num_channels;
+	 }
+
+	 src += pixelarea_rowstride (src_area);
+
+	 if (mask_area)
+	   mask += pixelarea_rowstride (mask_area);
+     }
+}
+
 static void
 histogram_histogram_range_float16 (int          start,
 				int              end,
@@ -1096,6 +1194,71 @@ histogram_histogram_range_u8 (int              start,
     histogram_dialog_update (htd, start, end);
 }
 
+static void
+histogram_histogram_range_bfp (int          start,
+				int              end,
+				int		 bins,
+				HistogramValues  values,
+				void            *user_data)
+{
+  Histogram	*htd;
+  double		 pixels;
+  double 		 mean;
+  double 		 std_dev;
+  int 			 median;
+  double 		 count;
+  double 		 percentile;
+  double 		 tmp;
+  int 			 i;
+  HistogramPrivate *histogram_p;
+
+  htd = (Histogram *) user_data;
+  histogram_p = (HistogramPrivate *) htd->private_part;
+
+  count = 0.0;
+  pixels = 0.0;
+  for (i = 0; i < bins; i++)
+    {
+      pixels += values[histogram_p->channel][i];
+
+      if (i >= start && i <= end)
+	count += values[histogram_p->channel][i];
+    }
+
+  mean = 0.0;
+  tmp = 0.0;
+  median = -1;
+  for (i = start; i <= end; i++)
+    {
+      mean += i * 256 * values[histogram_p->channel][i];
+      tmp += values[histogram_p->channel][i];
+      if (median == -1 && tmp > count / 2)
+	median = i * 256;
+    }
+
+  if (count)
+    mean /= count;
+
+  std_dev = 0.0;
+  for (i = start; i <= end; i++)
+    std_dev += values[histogram_p->channel][i] * (256 * i - mean) * (256 * i - mean);
+
+  if (count)
+    std_dev = sqrt (std_dev / count);
+
+  percentile = count / pixels;
+
+  histogram_p->mean = mean;
+  histogram_p->std_dev = std_dev;
+  histogram_p->median = median;
+  histogram_p->pixels = pixels;
+  histogram_p->count = count;
+  histogram_p->percentile = percentile;
+
+  if (htd->shell)
+    histogram_dialog_update (htd, start * 256, end * 256 + 255);
+}
+
 
 static void
 histogram_dialog_update_float16 (Histogram *htd,
@@ -1191,3 +1354,11 @@ histogram_dialog_update_u16 (Histogram *htd,
 {
   histogram_dialog_update_u8 (htd, start, end);
 }
+
+static void
+histogram_dialog_update_bfp (Histogram *htd,
+			      int                  start,
+			      int                  end)
+{
+  histogram_dialog_update_u8 (htd, start, end);
+}
