@@ -26,6 +26,7 @@
 #include "../colormaps.h"
 #include "../errors.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../rc.h"
 #include "../gximage.h"
 #include "../image_render.h"
@@ -34,6 +35,7 @@
 #include "../pixelrow.h"
 #include "../scale.h"
 #include "../tag.h"
+#include "../tilebuf.h"
 #include "../trace.h"
 #include "displaylut.h"
 
@@ -92,8 +94,6 @@ guchar *check_buf = NULL;
 guchar *empty_buf = NULL;
 guchar *temp_buf = NULL;
 
-Canvas *comp_canvas = NULL;
-
 static guint   check_mod;
 static guint   check_shift;
 static guchar  check_combos[6][2] =
@@ -260,6 +260,11 @@ static void    render_image_gray_a_float16    (RenderInfo *info);
 static void    render_image_rgb_float16       (RenderInfo *info);
 static void    render_image_rgb_a_float16     (RenderInfo *info);
 
+static void    render_image_gray_bfp      (RenderInfo *info);
+static void    render_image_gray_a_bfp    (RenderInfo *info);
+static void    render_image_rgb_bfp       (RenderInfo *info);
+static void    render_image_rgb_a_bfp     (RenderInfo *info);
+
 static void    render_image_init_info          (RenderInfo   *info,
 						GDisplay     *gdisp,
 						int           x,
@@ -273,7 +278,6 @@ static guchar* render_image_accelerate_scaling (int           width,
 						int           scalesrc,
 						int           scaledest);
 static guchar* render_image_tile_fault         (RenderInfo   *info);
-static guchar* render_image_tile_fault1        (RenderInfo   *info);
 
 
 static RenderFunc render_funcs[6] =
@@ -310,6 +314,14 @@ static RenderFunc render_funcs_float16[4] =
   render_image_gray_a_float16,
 };
 
+static RenderFunc render_funcs_bfp[4] =
+{
+  render_image_rgb_bfp,
+  render_image_rgb_a_bfp,
+  render_image_gray_bfp,
+  render_image_gray_a_bfp,
+};
+
 /*****************************************************************/
 /*  This function is the core of the display--it offsets and     */
 /*  scales the image according to the current parameters in the  */
@@ -331,17 +343,13 @@ render_image (GDisplay *gdisp,
 
   t = canvas_tag (info.src_canvas);
 
-  /*allocate a row for aux channel compositing and intensity visibility */
-
-/*  comp_canvas = canvas_new (t, info.src_w, 1, STORAGE_FLAT); rsr */
-
   switch (tag_format (t))
     {
     case FORMAT_RGB:
     case FORMAT_GRAY:
       break;
 
-    case FORMAT_INDEXED:
+    case FORMAT_INDEXED:      
       if (tag_precision (t) != PRECISION_U8)
         {
           g_warning ("indexed images only supported in 8 bit mode");
@@ -378,11 +386,13 @@ render_image (GDisplay *gdisp,
       case PRECISION_FLOAT16:
         (* render_funcs_float16[image_type]) (&info);
   	break;
+      case PRECISION_BFP:
+         (* render_funcs_bfp[image_type]) (&info);
+  	break;	
       case PRECISION_NONE:
         break;
       }
   }
-/*  canvas_delete (comp_canvas); rsr */
 }
 
 
@@ -721,10 +731,9 @@ render_image_rgb (RenderInfo *info)
 	      dest[2] = src[2];
 
 	      src += 3;
-	      dest += 3;
+	      dest += 3;	      
 	    }
-	}
-
+	}            
       info->dest += info->dest_bpl;
 
       initial = FALSE;
@@ -849,7 +858,7 @@ render_image_init_info (RenderInfo *info,
   info->src_y = UNSCALE (gdisp, info->y);
   info->src_w = UNSCALE (gdisp, (info->x + info->w)) - UNSCALE (gdisp, info->x) + 1;
 
-  info->src_canvas = gimage_projection (gdisp->gimage);
+  info->src_canvas = gdisplay_get_projection (gdisp);
   info->channels = gimage_channels (gdisp->gimage);
   info->src_width = canvas_width (info->src_canvas);
   src_canvas_tag = canvas_tag (info->src_canvas);
@@ -968,23 +977,24 @@ render_image_accelerate_scaling (int width,
 
 void
 set_visible_channels_row_u8(
-             PixelRow * row,
-	     gint *visible
+             guchar * buffer,
+	     int num_pixels,
+	     int *visible,
+	     Tag tag
              )
 {
   gint    b;
-  guint8 *data         = (guint8*) pixelrow_data (row);
-  gint    num_channels = tag_num_channels (pixelrow_tag (row));
+  guint8 *data         = (guint8*) buffer;
+  gint    num_channels = tag_num_channels (tag);
   gint    n;
-  Alpha   alpha = tag_alpha (pixelrow_tag (row));
-  gint    width        = pixelrow_width (row);  
+  Alpha   alpha = tag_alpha (tag);
 
   if (alpha == ALPHA_YES) 
     n = num_channels - 1; 
   else
     n = num_channels;
 
-  while (width--)
+  while (num_pixels--)
     {
       for (b = 0; b < n; b++)
         data[b] =  visible[b] ? data[b] : 0;
@@ -994,49 +1004,24 @@ set_visible_channels_row_u8(
 
 void
 set_visible_channels_row_u16(
-             PixelRow * row,
-	     gint *visible
+             guchar * buffer,
+	     int num_pixels,
+	     int *visible,
+	     Tag tag
              )
 {
   gint    b;
-  guint16 *data         = (guint16*) pixelrow_data (row);
-  gint    num_channels = tag_num_channels (pixelrow_tag (row));
+  guint16 *data         = (guint16*) buffer;
+  gint    num_channels = tag_num_channels (tag);
   gint    n;
-  Alpha   alpha = tag_alpha (pixelrow_tag (row));
-  gint    width        = pixelrow_width (row);  
+  Alpha   alpha = tag_alpha (tag);
 
   if (alpha == ALPHA_YES) 
     n = num_channels - 1; 
   else
     n = num_channels;
 
-  while (width--)
-    {
-      for (b = 0; b < n; b++)
-        data[b] =  visible[b] ? data[b] : 0;
-      data += num_channels;
-    }
-}
-
-void
-set_visible_channels_row_float(
-             PixelRow * row,
-	     gint *visible
-             )
-{
-  gint    b;
-  gfloat *data         = (gfloat*) pixelrow_data (row);
-  gint    num_channels = tag_num_channels (pixelrow_tag (row));
-  gint    n;
-  Alpha   alpha = tag_alpha (pixelrow_tag (row));
-  gint    width        = pixelrow_width (row);  
-
-  if (alpha == ALPHA_YES) 
-    n = num_channels - 1; 
-  else
-    n = num_channels;
-
-  while (width--)
+  while (num_pixels--)
     {
       for (b = 0; b < n; b++)
         data[b] =  visible[b] ? data[b] : 0.0;
@@ -1045,24 +1030,25 @@ set_visible_channels_row_float(
 }
 
 void
-set_visible_channels_row_float16(
-             PixelRow * row,
-	     gint *visible
+set_visible_channels_row_float(
+             guchar * buffer,
+	     int num_pixels,
+	     int *visible,
+	     Tag tag
              )
 {
   gint    b;
-  guint16 *data         = (guint16*) pixelrow_data (row);
-  gint    num_channels = tag_num_channels (pixelrow_tag (row));
+  gfloat *data         = (gfloat*) buffer;
+  gint    num_channels = tag_num_channels (tag);
   gint    n;
-  Alpha   alpha = tag_alpha (pixelrow_tag (row));
-  gint    width        = pixelrow_width (row);  
+  Alpha   alpha = tag_alpha (tag);
 
   if (alpha == ALPHA_YES) 
     n = num_channels - 1; 
   else
     n = num_channels;
 
-  while (width--)
+  while (num_pixels--)
     {
       for (b = 0; b < n; b++)
         data[b] =  visible[b] ? data[b] : ZERO_FLOAT16;
@@ -1070,7 +1056,62 @@ set_visible_channels_row_float16(
     }
 }
 
-typedef void (*SetVisibleChannelsFunc) (PixelRow*, gint*);
+void
+set_visible_channels_row_float16(
+             guchar * buffer,
+	     int num_pixels,
+	     int *visible,
+	     Tag tag
+             )
+{
+  gint    b;
+  guint16 *data         = (guint16*) buffer;
+  gint    num_channels = tag_num_channels (tag);
+  gint    n;
+  Alpha   alpha = tag_alpha (tag);
+
+  if (alpha == ALPHA_YES) 
+    n = num_channels - 1; 
+  else
+    n = num_channels;
+
+  while (num_pixels--)
+    {
+      for (b = 0; b < n; b++)
+        data[b] =  visible[b] ? data[b] : ZERO_FLOAT16;
+      data += num_channels;
+    }
+}
+
+void
+set_visible_channels_row_bfp(
+             guchar * buffer,
+	     int num_pixels,
+	     int *visible,
+	     Tag tag
+             )
+{
+  gint    b;
+  guint16 *data         = (guint16*) buffer;
+  gint    num_channels = tag_num_channels (tag);
+  gint    n;
+  Alpha   alpha = tag_alpha (tag);
+
+  if (alpha == ALPHA_YES) 
+    n = num_channels - 1; 
+  else
+    n = num_channels;
+
+  while (num_pixels--)
+    {
+      for (b = 0; b < n; b++)
+        data[b] =  visible[b] ? data[b] : 0;
+      data += num_channels;
+    }
+}
+ 
+
+typedef void (*SetVisibleChannelsFunc) (guchar*, int, int*, Tag);
 static SetVisibleChannelsFunc 
 set_visible_channels_func (
                    Tag tag
@@ -1086,6 +1127,8 @@ set_visible_channels_func (
       return set_visible_channels_row_float16;
     case PRECISION_FLOAT:
       return set_visible_channels_row_float;
+    case PRECISION_BFP:
+      return set_visible_channels_row_bfp;
     case PRECISION_NONE:
     default:
       g_warning ("set_visible_channels_func: bad precision");
@@ -1093,6 +1136,47 @@ set_visible_channels_func (
   return NULL;
 }
 
+void 
+image_render_set_visible_channels(GImage *gimage,
+				  int       x,
+				  int       y,
+				  int       w,
+				  int       h)
+{   int i;
+    void * pag;
+    Canvas *projection = gimage_projection(gimage);
+    PixelArea projectionPA;
+    Tag projection_tag = canvas_tag(projection);
+    int num_inten = tag_num_channels(projection_tag);
+    int visible[MAX_CHANNELS];
+    SetVisibleChannelsFunc func=set_visible_channels_func(projection_tag);
+
+    if (tag_alpha (projection_tag) == ALPHA_YES )
+    {   num_inten--;
+    }
+
+    for(i = 0; i < num_inten; i++)
+    {   visible[i] = gimage_get_component_visible (gimage, i); 
+    }	
+   
+    pixelarea_init (&projectionPA, projection, x, y, w, h, TRUE);
+    for (pag = pixelarea_register (1, &projectionPA);
+	 pag != NULL;
+	 pag = pixelarea_process (pag))
+    {   guint h = pixelarea_height(&projectionPA);	  
+        PixelRow row_buffer;
+        guint8 *data;
+	guint num_pixels;
+	
+	while (h--)
+        {   pixelarea_getdata(&projectionPA, &row_buffer, h);
+   	    data = pixelrow_data(&row_buffer);
+	    num_pixels = pixelrow_width(&row_buffer);
+	    (*func)(data,num_pixels,visible,projection_tag);	    
+	}
+    }
+}
+
 #if 0
       pixelarea_init (&aux_channel_area, aux_channel_canvas, src_x, src_y, src_w, 1, FALSE);
       pixelarea_init (&comp_area, comp_canvas, 0, 0, src_w, 1, FALSE); 
@@ -1118,123 +1202,10 @@ set_visible_channels_func (
 #define ONE_OR_MORE_AUX_VISIBLE 2 
 #endif
 
+
 #if 1
 static guchar*
 render_image_tile_fault (RenderInfo *info)
-{
-  PixelArea  area, comp_area;
-  PixelRow comp_row;
-  Tag tag = canvas_tag (info->src_canvas);
-  guchar *data, *dest, *scale;
-  gint src_x, src_y, src_w;
-  gint width;
-  void * pag;
-  gint bytes = tag_bytes (tag); 
-  gint b, step;
-  GSList *list;
-  src_x = info->src_x;
-  src_y = info->src_y;
-  src_w = info->src_w;
-  width = info->w;
-
-  if (info->intensity_visible == NO_INTEN_VISIBLE && 
-	info->single_visible_channel) 
-  {
-      Canvas *aux_channel_canvas;
-      aux_channel_canvas = drawable_data ( GIMP_DRAWABLE (info->single_visible_channel) );
-      pixelarea_init (&area, aux_channel_canvas, src_x, src_y, src_w, 1, FALSE); 
-      pixelarea_init (&comp_area, comp_canvas, 0, 0, src_w, 1, TRUE); 
-      copy_area (&area, &comp_area); 
-  }
-  else 
-  {
-	/* if some RGB is visible */
-	if (info->intensity_visible != NO_INTEN_VISIBLE) 
-	{
-	  if (info->intensity_visible == ALL_INTEN_VISIBLE &&
-		info->aux_channels_visible == NO_AUX_VISIBLE)
-	    return render_image_tile_fault1 (info);
-	  
-	  pixelarea_init (&area, info->src_canvas, src_x, src_y, src_w, 1, FALSE); 
-	  pixelarea_init (&comp_area, comp_canvas, 0, 0, src_w, 1, TRUE); 
-	  copy_area (&area, &comp_area); 
-
-	  if (info->intensity_visible != ALL_INTEN_VISIBLE)
-	    {	
-	      pixelarea_init (&comp_area, comp_canvas, 0, 0, src_w, 1, TRUE); 
-	      pag = pixelarea_register (1, &comp_area);
-	      pixelarea_getdata (&comp_area, &comp_row, 0);
-	      (*set_visible_channels_func (tag)) (&comp_row, info->visible);
-	      pixelarea_process_stop (pag);
-	    }
-	}
-	else    /* NO rgb visible, 0 or 2 or more aux channels visible */
-	{
-	  /* just fill the comp_canvas with black */
-	}
-       
-       /* now if there are any aux channels visible composite them in */
-	if (info->aux_channels_visible != NO_AUX_VISIBLE)
-	{	
-	  list = info->channels;
-	  while (list)
-	    {
-	      Channel *channel = (Channel *)(list->data);
-	      gint visible = channel_visibility (channel);
-	      PixelRow *color;
-	      PixelArea aux_channel_area;
-	      gfloat opacity;
-	      Canvas *aux_channel_canvas;
-	  
-	      if (visible)
-	      {
-		aux_channel_canvas = drawable_data ( GIMP_DRAWABLE (channel) );
-		pixelarea_init (&aux_channel_area, aux_channel_canvas, src_x, src_y, src_w, 1, FALSE);
-		pixelarea_init (&comp_area, comp_canvas, 0, 0, src_w, 1, TRUE); 
-		color = channel_color (channel);
-		opacity = channel_opacity (channel);
-		
-#if 0 
-		combine_areas (&comp_area, &aux_channel_area, &comp_area, NULL, 
-		  (unsigned char*)color, opacity, 
-		   NORMAL_MODE, NULL, COMBINE_INTEN_A_CHANNEL_MASK);
-#endif
-#if 1 
-		combine_areas (&comp_area, &aux_channel_area, &comp_area, NULL, 
-		  (unsigned char*)color, opacity, 
-		   NORMAL_MODE, NULL, COMBINE_INTEN_A_CHANNEL_SELECTION);
-#endif
-	      }
-	      list = g_slist_next (list);
-    	     }
-	
-        }
-  }
-
-  dest = tile_buf;
-  scale = info->scale;
-  step = info->scalesrc * info->src_bpp;
-
-  pixelarea_init (&comp_area, comp_canvas, 0, 0, src_w, 1, FALSE); 
-  pag = pixelarea_register (1, &comp_area);
-  data = pixelarea_data (&comp_area);
-  while (width--)
-    {
-      for (b = 0; b < bytes; b++)
-	 *dest++ = data[b];
-
-      if (*scale++ != 0)
-	  data += step;
-    }
-
-  pixelarea_process_stop(pag);
-  return tile_buf;
-}
-#endif
-
-#if 1
-static guchar*
-render_image_tile_fault1 (RenderInfo *info)
 {
   guchar *data;
   guchar *dest;
@@ -1244,7 +1215,7 @@ render_image_tile_fault1 (RenderInfo *info)
   int y_portion;
   int portion_width;
   int step;
-  int x, b;
+  int x;
   int src_bpp;
   /* the first portion x and y */
   x_portion = info->src_x;
@@ -2183,3 +2154,335 @@ render_image_rgb_a_float16 (RenderInfo *info)
       error += step;
     }
 }
+
+
+/*************************/
+/* 16bit BFP channel data functions */
+/*************************/
+
+static void
+render_image_gray_bfp (RenderInfo *info)
+{
+  guint16 *src;
+  guchar *dest;
+  gulong val;
+  int y, ye;
+  int x, xe;
+  int initial;
+  float error;
+  float step;
+
+  y = info->y;
+  ye = info->y + info->h;
+  xe = info->x + info->w;
+
+  step = info->scalesrc;
+
+  error = y * step;
+  error -= ((int)error) - step;
+
+  initial = TRUE;
+  info->src = render_image_tile_fault (info);
+
+  for (; y < ye; y++)
+    {
+      if (!initial && (error < 1.0))
+	memcpy (info->dest, info->dest - info->dest_bpl, info->dest_width);
+      else
+	{
+	  src =(guint16*)info->src;
+	  dest = info->dest;
+
+	  for (x = info->x; x < xe; x++)
+	    {
+              /* check for over-white*/
+              if (src[GRAY_PIX] > ONE_BFP)
+              {
+		dest[RED_PIX] = gamut_alarm_red;
+                dest[GREEN_PIX] = gamut_alarm_blue;
+                dest[BLUE_PIX] = gamut_alarm_green;
+              }
+              else
+	      {
+	        val = BFP_TO_U8(src[GRAY_PIX]);
+	        src += 1;
+
+	        dest[RED_PIX] = val;
+	        dest[GREEN_PIX] = val;
+	        dest[BLUE_PIX] = val;
+	        dest += 3;
+              }
+	    }
+	}
+
+      info->dest += info->dest_bpl;
+
+      initial = FALSE;
+
+      if (((y + 1) % info->scaledest) == 0)
+	{
+	  info->src_y += info->scalesrc;
+	  info->src = render_image_tile_fault (info);
+	}
+
+      error += step;
+    }
+}
+
+static void
+render_image_gray_a_bfp (RenderInfo *info)
+{
+  guint16 *src;
+  guchar *dest;
+  guint *alpha;
+  gulong val;
+  guint a;
+  int dark_light;
+  int y, ye;
+  int x, xe;
+  int initial;
+  float error;
+  float step;
+
+  alpha = info->alpha;
+
+  y = info->y;
+  ye = info->y + info->h;
+  xe = info->x + info->w;
+
+  step = info->scalesrc;
+
+  error = y * step;
+  error -= ((int)error) - step;
+
+  initial = TRUE;
+  info->src = render_image_tile_fault (info);
+
+  for (; y < ye; y++)
+    {
+      if (!initial && (error < 1.0) && (y & check_mod))
+	memcpy (info->dest, info->dest - info->dest_bpl, info->dest_width);
+      else
+	{
+	  src =(guint16*) info->src;
+	  dest = info->dest;
+
+	  dark_light = (y >> check_shift) + (info->x >> check_shift);
+
+	  for (x = info->x; x < xe; x++)
+	    {
+	      a = alpha[BFP_TO_U8(src[ALPHA_G_PIX])];
+              /* check for over-white*/
+              if (src[GRAY_PIX] > ONE_BFP)
+              {
+  	        if (dark_light & 0x1)
+		{  
+		  dest[RED_PIX] = blend_dark_check[(a | gamut_alarm_red)];
+                  dest[GREEN_PIX] = blend_dark_check[(a | gamut_alarm_green)];
+                  dest[BLUE_PIX] = blend_dark_check[(a | gamut_alarm_blue)];
+                }
+                else 
+                {
+		  dest[RED_PIX] = blend_light_check[(a | gamut_alarm_red)];
+                  dest[GREEN_PIX] = blend_light_check[(a | gamut_alarm_green)];
+                  dest[BLUE_PIX] = blend_light_check[(a | gamut_alarm_blue)];       
+                }
+              }
+
+              else 
+              {
+	        if (dark_light & 0x1)
+		  val = blend_dark_check[(a | BFP_TO_U8(src[GRAY_PIX]))];
+	        else
+		  val = blend_light_check[(a | BFP_TO_U8(src[GRAY_PIX]))];
+	        src += 2;
+
+	        dest[0] = val;
+	        dest[1] = val;
+	        dest[2] = val;
+	        dest += 3;
+              }
+
+	      if (((x + 1) & check_mod) == 0)
+		dark_light += 1;
+	    }
+	}
+
+      info->dest += info->dest_bpl;
+
+      initial = FALSE;
+
+      if (((y + 1) % info->scaledest) == 0)
+	{
+	  info->src_y += info->scalesrc;
+	  info->src = render_image_tile_fault (info);
+	}
+
+      error += step;
+    }
+}
+
+static void
+render_image_rgb_bfp (RenderInfo *info)
+{
+  guint16 *src;
+  guchar *dest;
+  int y, ye;
+  int x, xe;
+  int initial;
+  float error;
+  float step;
+
+  y = info->y;
+  ye = info->y + info->h;
+  xe = info->x + info->w;
+
+  step = info->scalesrc;
+
+  error = y * step;
+  error -= ((int)error) - step;
+
+  initial = TRUE;
+  info->src = render_image_tile_fault (info);
+
+  for (; y < ye; y++)
+    {
+      if (!initial && (error < 1.0))
+	memcpy (info->dest, info->dest - info->dest_bpl, info->dest_width);
+      else
+	{
+	  src =(guint16*) info->src;
+	  dest = info->dest;
+
+	  for (x = info->x; x < xe; x++)
+          {
+            /* check for over-white */
+            if ((src[RED_PIX] > ONE_BFP) || 
+                (src[GREEN_PIX] > ONE_BFP) || 
+                (src[BLUE_PIX] > ONE_BFP))
+	    {
+	      dest[RED_PIX] = gamut_alarm_red;
+	      dest[GREEN_PIX] = gamut_alarm_green;
+	      dest[BLUE_PIX] = gamut_alarm_blue;
+	    }
+	    else 
+	    {
+	      dest[RED_PIX] = BFP_TO_U8(src[RED_PIX]);
+	      dest[GREEN_PIX] = BFP_TO_U8(src[GREEN_PIX]);
+	      dest[BLUE_PIX] = BFP_TO_U8(src[BLUE_PIX]);
+	    }
+
+	  src += 3;
+	  dest += 3;
+	  }
+	}
+
+      info->dest += info->dest_bpl;
+
+      initial = FALSE;
+
+      if (((y + 1) % info->scaledest) == 0)
+	{
+	  info->src_y += info->scalesrc;
+	  info->src = render_image_tile_fault (info);
+	}
+
+      error += step;
+    }
+}
+
+static void
+render_image_rgb_a_bfp (RenderInfo *info)
+{
+  guint16 *src;
+  guchar *dest;
+  guint *alpha;
+  gulong r, g, b;
+  guint a;
+  int dark_light;
+  int y, ye;
+  int x, xe;
+  int initial;
+  float error;
+  float step;
+
+  alpha = info->alpha;
+
+  y = info->y;
+  ye = info->y + info->h;
+  xe = info->x + info->w;
+
+  step = info->scalesrc;
+
+  error = y * step;
+  error -= ((int)error) - step;
+
+  initial = TRUE;
+  info->src = render_image_tile_fault (info);
+
+  for (; y < ye; y++)
+    {
+      if (!initial && (error < 1.0) && (y & check_mod))
+	memcpy (info->dest, info->dest - info->dest_bpl, info->dest_width);
+      else
+	{
+	  src = (guint16 *)info->src;
+	  dest = info->dest;
+
+	  dark_light = (y >> check_shift) + (info->x >> check_shift);
+
+	  for (x = info->x; x < xe; x++)
+          {
+  	      /* check for over-white */
+              if ((src[RED_PIX] > ONE_BFP) || (src[GREEN_PIX] > ONE_BFP) || (src[BLUE_PIX] > ONE_BFP)) 
+              {
+                r = gamut_alarm_red;
+                b = gamut_alarm_blue;
+                g = gamut_alarm_green;
+              }
+              else
+	      {
+	        r = BFP_TO_U8(src[RED_PIX]);
+	        g = BFP_TO_U8(src[GREEN_PIX]);
+	        b = BFP_TO_U8(src[BLUE_PIX]);
+              }
+
+	      a = alpha[BFP_TO_U8(src[ALPHA_PIX])];
+	      if (dark_light & 0x1)
+		{
+		  r = blend_dark_check[(a | r)];
+		  g = blend_dark_check[(a | g)];
+		  b = blend_dark_check[(a | b)];
+		}
+	      else
+		{
+		  r = blend_light_check[(a | r)];
+		  g = blend_light_check[(a | g)];
+		  b = blend_light_check[(a | b)];
+		}
+
+	      src += 4;
+
+	      dest[0] = r;
+	      dest[1] = g;
+	      dest[2] = b;
+	      dest += 3;
+
+	      if (((x + 1) & check_mod) == 0)
+		dark_light += 1;
+	    }
+	}
+
+      info->dest += info->dest_bpl;
+
+      initial = FALSE;
+
+      if (((y + 1) % info->scaledest) == 0)
+	{
+	  info->src_y += info->scalesrc;
+	  info->src = render_image_tile_fault (info);
+	}
+
+      error += step;
+    }
+}
