@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#include "../../lib/wire/dl_list.h"
 #include "../appenv.h"
 #include "../canvas.h"
 #include "../drawable.h"
@@ -38,8 +39,8 @@
 #include "../pixelarea.h"
 #include "../pixelrow.h"
 #include "../undo.h"
-
 #include "../layer_pvt.h"
+#include "../spline.h"
 
 enum {
   LAST_SIGNAL
@@ -231,6 +232,16 @@ layer_new  (
   layer->fs.segs = NULL;
   layer->fs.num_segs = 0;
 
+  /* spline layer variables */
+  SplineTool *st;
+  layer->sl.splines=(DL_list*)malloc(sizeof(DL_list));
+  DL_init((DL_node*)layer->sl.splines);
+  st=(SplineTool*)malloc(sizeof(SplineTool));
+  st->motion_active=0;
+  st->click_on_point=0;
+  st->active_spline=(Spline*)layer->sl.splines->head;
+  st->active_point=NULL;
+  layer->sl.spline_tool=st;
   return layer;
 }
 
@@ -1093,6 +1104,7 @@ layer_pick_correlate (layer, x, y)
           }
           break;
           
+        case PRECISION_BFP:
         case PRECISION_U16:
           {
             guint16 * d = (guint16*) c_data;
