@@ -21,6 +21,7 @@
 
 #include "../appenv.h"
 #include "float16.h"
+#include "bfp.h"
 #include "../paint_funcs_area.h"
 #include "paint_funcs_row_u8.h"
 #include "../pixelrow.h"
@@ -455,8 +456,8 @@ hsv_only_row_u8 (
 		    gint       mode
 		    )
 {
-  gint    r1, g1, b1;
-  gint    r2, g2, b2;
+  gfloat    r1, g1, b1;
+  gfloat    r2, g2, b2;
   Tag     src1_tag      = pixelrow_tag (src1_row); 
   Tag     src2_tag      = pixelrow_tag (src2_row); 
   gint    ha1           = (tag_alpha (src1_tag)==ALPHA_YES)? TRUE: FALSE;
@@ -1100,6 +1101,7 @@ apply_mask_to_alpha_channel_row_u8  (
   alpha = num_channels - 1;
   while (width --)
     {
+
       src[alpha] = (src[alpha] * *mask++ * opacity) / 255;
       src += num_channels;
     }
@@ -2913,6 +2915,117 @@ copy_row_rgb_to_float16_gray  (
 }
 
 
+static void 
+copy_row_rgb_to_bfp_rgb  (
+                          PixelRow * src_row,
+                          PixelRow * dest_row
+                          )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint8 * s = (guint8*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] << 7;
+            d[1] = s[1] << 7;
+            d[2] = s[2] << 7;
+            d[3] = s[3] << 7;
+            s += 4;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] << 7;
+            d[1] = s[1] << 7;
+            d[2] = s[2] << 7;
+            s += 4;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] * 127;
+            d[1] = s[1] * 127;
+            d[2] = s[2] * 127;
+            d[3] = ONE_BFP;
+            s += 3;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] * 127;
+            d[1] = s[1] * 127;
+            d[2] = s[2] * 127;
+            s += 3;
+            d += 3;
+          }
+    }
+}
+
+
+static void 
+copy_row_rgb_to_bfp_gray  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint8 * s = (guint8*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] << 7, s[1] << 7, s[2] << 7);
+            d[1] = s[3] << 7;
+            s += 4;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] << 7, s[1] << 7, s[2] << 7);
+            s += 4;
+            d += 1;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] << 7, s[1] << 7, s[2] << 7);
+            d[1] = ONE_BFP;
+            s += 3;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = INTENSITY (s[0] << 7, s[1] << 7, s[2] << 7);
+            s += 3;
+            d += 1;
+          }
+    }
+}
+
+
+
 static void 
 copy_row_gray_to_u8_rgb  (
                           PixelRow * src_row,
@@ -3305,10 +3418,111 @@ copy_row_gray_to_float16_gray  (
 }
 
 
+static void 
+copy_row_gray_to_bfp_rgb  (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint8 * s = (guint8*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] << 7;
+            d[3] = s[1] << 7;
+            s += 2;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] << 7;
+            s += 2;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] << 7;
+            d[3] = 65535;
+            s += 1;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = d[1] = d[2] = s[0] << 7;
+            s += 1;
+            d += 3;
+          }
+    }
+}
+
+
+static void
+copy_row_gray_to_bfp_gray (
+                           PixelRow * src_row,
+                           PixelRow * dest_row
+                           )
+{
+  int w = MIN (pixelrow_width (src_row), pixelrow_width (dest_row));
+  Tag stag = pixelrow_tag (src_row);
+  Tag dtag = pixelrow_tag (dest_row);
+  guint8 * s = (guint8*) pixelrow_data (src_row);
+  guint16 * d = (guint16*) pixelrow_data (dest_row);
+
+  if (tag_alpha (stag) == ALPHA_YES)
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] << 7;
+            d[3] = s[1] << 7;
+            s += 2;
+            d += 4;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] << 7;
+            s += 2;
+            d += 3;
+          }
+    }
+  else
+    {
+      if (tag_alpha (dtag) == ALPHA_YES)
+        while (w--)
+          {
+            d[0] = s[0] << 7;
+            d[1] = 65535;
+            s += 1;
+            d += 2;
+          }
+      else
+        while (w--)
+          {
+            d[0] = s[0] << 7;
+            s += 1;
+            d += 1;
+          }
+    }
+}
+
+
 typedef void (*CopyRowFunc) (PixelRow *, PixelRow *);
 
-static CopyRowFunc funcs[2][4][2] =
-{
+static CopyRowFunc funcs[2][5][2] = {
   {
     {
       copy_row_rgb_to_u8_rgb,
@@ -3325,6 +3539,10 @@ static CopyRowFunc funcs[2][4][2] =
     {
       copy_row_rgb_to_float16_rgb,
       copy_row_rgb_to_float16_gray
+    },
+    {
+      copy_row_rgb_to_bfp_rgb,
+      copy_row_rgb_to_bfp_gray
     }
   },
   
@@ -3344,6 +3562,10 @@ static CopyRowFunc funcs[2][4][2] =
     {
       copy_row_gray_to_float16_rgb,
       copy_row_gray_to_float16_gray
+    },
+    {
+      copy_row_gray_to_bfp_rgb,
+      copy_row_gray_to_bfp_gray
     }
   }
 };
@@ -3374,6 +3596,7 @@ copy_row_u8 (
     case PRECISION_U16:   y = 1; break;
     case PRECISION_FLOAT: y = 2; break;
     case PRECISION_FLOAT16: y = 3; break;
+    case PRECISION_BFP: y = 4; break;
     case PRECISION_NONE:
     default:
       g_warning ("unsupported dest precision in copy_row_u8()");
