@@ -20,6 +20,7 @@
 #include <errno.h>
 #include <string.h>
 #include <sys/types.h>
+#include <dirent.h>
 
 #include <unistd.h>
 #ifdef WIN32
@@ -35,6 +36,8 @@
 #include "../lib/version.h"
 #include "appenv.h"
 #include "actionarea.h"
+#include "cms.h"
+#include "convert.h"
 #include "general.h"
 #include "gimage.h"
 #include "fileops.h"
@@ -966,11 +969,32 @@ file_load (char *filename, char* raw_filename, GDisplay *gdisplay)
       /*  set the image to clean  */
       gimage_clean_all (gimage);
 
+      /* assign profile according to cms_open_action */
+      if (!gimage_get_cms_profile(gimage) &&
+	   (cms_open_action == CMS_ASSIGN_DEFAULT) &&
+	   (cms_default_image_profile_name != NULL)) 
+      {   gimage_set_cms_profile(gimage, cms_get_profile_from_file(cms_default_image_profile_name));
+      }
+      else if (cms_open_action == CMS_ASSIGN_PROMPT)
+      {   cms_open_assign_dialog(gimage);
+      }
+ 		          
+      /* if a workspace profile is given and it's not equal the 
+	 image profile, check whether to convert */
+      if ((cms_workspace_profile_name != NULL) && (gimage_get_cms_profile(gimage) != NULL))
+      {   CMSProfile *workspace_profile = cms_get_profile_from_file(cms_workspace_profile_name);
+          if ((gimage_get_cms_profile(gimage) != workspace_profile) &&
+	      ((cms_mismatch_action == CMS_MISMATCH_AUTO_CONVERT)  || 
+	       ((cms_mismatch_action == CMS_MISMATCH_PROMPT) && (cms_convert_on_open_prompt(gimage)==TRUE))))
+	  {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
+	  }
+      } 
+
       /*  display the image */
       gdisplay->gimage = gimage;
       gimage_set_filename (gimage, filename);
       
-      gdisplays_update_full (gdisplay->gimage->ID); 
+      gdisplays_update_gimage (gdisplay->gimage->ID); 
       gdisplays_update_title (gimage->ID); 
       
    }
@@ -1027,7 +1051,26 @@ file_load_without_display (char *filename, char* raw_filename, GDisplay *gdispla
 
       /*  set the image to clean  */
       gimage_clean_all (gimage);
-
+      /* assign profile according to cms_open_action */
+      if (!gimage_get_cms_profile(gimage) &&
+	   (cms_open_action == CMS_ASSIGN_DEFAULT) &&
+	   (cms_default_image_profile_name != NULL)) 
+      {   gimage_set_cms_profile(gimage, cms_get_profile_from_file(cms_default_image_profile_name));
+      }
+      else if (cms_open_action == CMS_ASSIGN_PROMPT)
+      {   cms_open_assign_dialog(gimage);
+      }
+ 		          
+      /* if a workspace profile is given and it's not equal the 
+	 image profile, check whether to convert */
+      if ((cms_workspace_profile_name != NULL) && (gimage_get_cms_profile(gimage) != NULL))
+      {   CMSProfile *workspace_profile = cms_get_profile_from_file(cms_workspace_profile_name);
+          if ((gimage_get_cms_profile(gimage) != workspace_profile) &&
+	      ((cms_mismatch_action == CMS_MISMATCH_AUTO_CONVERT)  || 
+	       ((cms_mismatch_action == CMS_MISMATCH_PROMPT) && (cms_convert_on_open_prompt(gimage)==TRUE))))
+	  {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
+	  }
+      } 
    }
 
   if (return_val)
@@ -1088,6 +1131,27 @@ file_open (char *filename, char* raw_filename)
       /*  set the image to clean  */
       gimage_clean_all (gimage);
 
+      /* assign profile according to cms_open_action */
+      if (!gimage_get_cms_profile(gimage) &&
+	   (cms_open_action == CMS_ASSIGN_DEFAULT) &&
+	   (cms_default_image_profile_name != NULL)) 
+      {   gimage_set_cms_profile(gimage, cms_get_profile_from_file(cms_default_image_profile_name));
+      }
+      else if (cms_open_action == CMS_ASSIGN_PROMPT)
+      {   cms_open_assign_dialog(gimage);
+      }
+ 		          
+      /* if a workspace profile is given and it's not equal the 
+	 image profile, check whether to convert */
+      if ((cms_workspace_profile_name != NULL) && (gimage_get_cms_profile(gimage) != NULL))
+      {   CMSProfile *workspace_profile = cms_get_profile_from_file(cms_workspace_profile_name);
+          if ((gimage_get_cms_profile(gimage) != workspace_profile) &&
+	      ((cms_mismatch_action == CMS_MISMATCH_AUTO_CONVERT)  || 
+	       ((cms_mismatch_action == CMS_MISMATCH_PROMPT) && (cms_convert_on_open_prompt(gimage)==TRUE))))
+	  {   convert_image_colorspace(gimage, workspace_profile, cms_default_intent);
+	  }
+      } 
+
       /*  display the image */
       gdisplay_new (gimage, 0x0101);
   
@@ -1948,6 +2012,54 @@ file_update_menus (GSList *procs,
     }
 }
 
+/* creates a unique name for a temporary file
+ * parameter is extension without '.' 
+*/
+char *
+file_temp_name(char *extension) 
+{   static gint id = 0;
+    static gint pid;
+
+    GString *s = g_string_new (NULL);
+
+    if (id == 0)
+    {   pid = getpid();
+    }
+
+    g_string_sprintf (s, "%s/cinepaint_temp.%d%d.%s", temp_path, pid, 
+		      id++, extension);
+
+
+    char *file_name = strdup(s->str);  
+    g_string_free (s, FALSE);
+    return file_name;
+}
+
+void
+file_temp_clear() 
+{   DIR *dir;
+    struct dirent *entry;
+
+    dir = opendir (temp_path);
+    if (!dir)
+    {   g_warning ("Can not open temp directory %s to clear temp files", temp_path);
+	return;
+    }
+
+    while ((entry = readdir (dir)))
+    {   char full_file_path[1024];
+	char *file_name = entry->d_name;
+
+	/* sort out . or .. directory and files that aren't cinepaint temp */
+	if ((strcmp (file_name, "..") == 0) || (strcmp (file_name, ".") == 0)
+	    || (strncmp(file_name, "cinepaint_temp.",15) != 0)) 
+	  continue;
+
+	sprintf (full_file_path, "%s/%s", temp_path, file_name);
+	remove(full_file_path);
+    }   
+}
+
 static Argument*
 file_load_invoker (Argument *args)
 {
@@ -2023,25 +2135,15 @@ file_save_invoker (Argument *args)
 static Argument*
 file_temp_name_invoker (Argument *args)
 {
-  static gint id = 0;
-  static gint pid;
-  Argument *return_args;
+    Argument *return_args;
+    char *file_name = file_temp_name((char*)args[0].value.pdb_pointer);
 
-  GString *s = g_string_new (NULL);
-
-
-  if (id == 0)
-    pid = getpid();
-
-  g_string_sprintf (s, "%s/gimp_temp.%d%d.%s", temp_path, pid, 
-      id++, (char*)args[0].value.pdb_pointer);
-
-
-  return_args = procedural_db_return_args (&file_temp_name_proc, TRUE);
-
-  return_args[1].value.pdb_pointer = s->str;
-
-  g_string_free (s, FALSE);
+    if (file_name != NULL) 
+    {   return_args = procedural_db_return_args (&file_temp_name_proc, TRUE);
+        return_args[1].value.pdb_pointer = file_name;
+    } 
+    else 
+        return_args = procedural_db_return_args (&file_temp_name_proc, FALSE);
 
   return return_args;
 }
