@@ -37,9 +37,12 @@
 #include "layer.h"
 #include "layers_dialog.h"
 #include "menus.h"
+#include "paint_funcs_area.h"
+#include "pixelarea.h"
 #include "plug_in.h"
 #include "scale.h"
 #include "scroll.h"
+#include "tilebuf.h"
 #include "tools.h"
 #include "undo.h"
 #include "base_frame_manager.h"
@@ -54,6 +57,7 @@ GSList *               display_list = NULL;
 static int             display_num  = 1;
 static GdkCursorType   default_gdisplay_cursor = GDK_TOP_LEFT_ARROW;
 extern char 	       BRUSH_CHANGED;
+CMSTransform *         transform_buffer=NULL;   /* the transform used for the current display flush */
 
 #define ROUND(x) ((int) (x + 0.5))
 
@@ -71,6 +75,8 @@ static void       gdisplay_paint_area       (GDisplay *, int, int, int, int);
 static void	  gdisplay_draw_cursor	    (GDisplay *);
 static void       gdisplay_display_area     (GDisplay *, int, int, int, int);
 static guint      gdisplay_hash             (GDisplay *);
+/* updates the projection */
+static void       gdisplay_project          (GDisplay *, int, int, int, int);
 
 static GHashTable *display_ht = NULL;
 
@@ -119,9 +125,10 @@ gdisplay_new (GImage       *gimage,
   gdisp->menubar_fac = NULL;
 
   gdisp->framemanager = 0;
-
   gdisp->frame_manager = NULL;
   gdisp->bfm = NULL;
+  gdisp->slide_show = 0;
+  gdisp->look_profile_manager = NULL;
 
   /* format the title */
   gdisplay_format_title (gimage, title, gdisp);
@@ -130,6 +137,10 @@ gdisplay_new (GImage       *gimage,
   gdisp->proximity = FALSE;
   gdisp->have_cursor = FALSE;
 
+  gdisp->cms_intent = cms_default_intent;
+  gdisp->cms_flags = cms_default_flags;
+  gdisp->look_profile_pipe = NULL;
+
   /*  add the new display to the list so that it isn't lost  */
   display_list = g_slist_append (display_list, (void *) gdisp);
 
@@ -169,6 +180,27 @@ gdisplay_new (GImage       *gimage,
 
   gdisplays_update_title (gimage->ID); 
 
+  /* initialize the projection */
+  Tag t = gimage_tag (gdisp->gimage);
+  t = tag_set_alpha (t, ALPHA_YES);
+  if (tag_format (t) == FORMAT_INDEXED)
+  {   t = tag_set_format (t, FORMAT_RGB);  
+  }
+  gdisp->projection = canvas_new (t, 
+				    gdisp->gimage->width, gdisp->gimage->height,
+#ifdef NO_TILES						  
+				    STORAGE_FLAT);
+#else
+                                    STORAGE_TILED);
+#endif
+
+  /* calculate the projection */
+  gdisplay_project(gdisp, 0, 0, gimage->width, gimage->height);
+
+  if (gdisplay_set_colormanaged(gdisp, cms_manage_by_default) == FALSE)	 
+  {   g_message("Cannot colormanage this display.\nEither image or display profile are undefined.\n. To avoid this message switch off automatic colormanagement in File/Preferences/Color Management");
+  }
+
   return gdisp;
 }
 
@@ -183,6 +215,7 @@ gdisplay_fm (GImage *gimage,
 
  return display;
 }
+        
 
 static void
 gdisplay_format_title (GImage *gimage,
@@ -196,12 +229,21 @@ gdisplay_format_title (GImage *gimage,
     alpha_msg = "A"; /*"alpha";*/
   }
 
-  sprintf (title, "%s-%d.%d (%s - %s%s)%s %d%%",
+  const char *profile;
+  if (gimage_get_cms_profile(gimage) != NULL)
+  {   profile = cms_get_profile_info(gimage_get_cms_profile(gimage))->description;
+  }
+  else 
+  {   profile = "[no icc profile assigned]";
+  }
+
+  sprintf (title, "%s-%d.%d (%s - %s%s - %s)%s %d%%",
 	   prune_filename (gimage_filename (gimage)),
 	   gimage->ID, gimage->instance_count,
 	   tag_string_precision (tag_precision (t)),
 	   tag_string_format (tag_format (t)),
 	   alpha_msg,
+	   profile,
            empty ? " (empty)" : "",
 	   100 * SCALEDEST (gdisp) / SCALESRC (gdisp));
 
@@ -234,6 +276,12 @@ gdisplay_delete (GDisplay *gdisp)
   /*  free the selection structure  */
   selection_free (gdisp->select);
 
+  /* free the projection */
+  canvas_delete(gdisp->projection); 
+
+  /* free the profile pipe */
+  g_slist_free(gdisp->look_profile_pipe); 
+
   if (gdisp->scroll_gc)
     gdk_gc_destroy (gdisp->scroll_gc);
 
@@ -283,7 +331,7 @@ gdisplay_free_area_list (GSList *list)
 
 static GSList *
 gdisplay_process_area_list (GSList *list,
-			    GArea  *ga1)
+                            GArea  *ga1)
 {
   GSList *new_list;
   GSList *l = list;
@@ -299,19 +347,19 @@ gdisplay_process_area_list (GSList *list,
       area1 = (ga1->x2 - ga1->x1) * (ga1->y2 - ga1->y1) + OVERHEAD;
       area2 = (ga2->x2 - ga2->x1) * (ga2->y2 - ga2->y1) + OVERHEAD;
       area3 = (MAXIMUM (ga2->x2, ga1->x2) - MINIMUM (ga2->x1, ga1->x1)) *
-	(MAXIMUM (ga2->y2, ga1->y2) - MINIMUM (ga2->y1, ga1->y1)) + OVERHEAD;
+        (MAXIMUM (ga2->y2, ga1->y2) - MINIMUM (ga2->y1, ga1->y1)) + OVERHEAD;
 
       if ((area1 + area2) < area3)
-	new_list = g_slist_prepend (new_list, ga2);
+        new_list = g_slist_prepend (new_list, ga2);
       else
-	{
-	  ga1->x1 = MINIMUM (ga1->x1, ga2->x1);
-	  ga1->y1 = MINIMUM (ga1->y1, ga2->y1);
-	  ga1->x2 = MAXIMUM (ga1->x2, ga2->x2);
-	  ga1->y2 = MAXIMUM (ga1->y2, ga2->y2);
+        {
+          ga1->x1 = MINIMUM (ga1->x1, ga2->x1);
+          ga1->y1 = MINIMUM (ga1->y1, ga2->y1);
+          ga1->x2 = MAXIMUM (ga1->x2, ga2->x2);
+          ga1->y2 = MAXIMUM (ga1->y2, ga2->y2);
 
-	  g_free (ga2);
-	}
+          g_free (ga2);
+        }
 
       l = g_slist_next (l);
     }
@@ -322,6 +370,54 @@ gdisplay_process_area_list (GSList *list,
   return new_list;
 }
 
+static void 
+gdisplay_project (GDisplay *display, int x, int y, int w, int h) 
+{   /* realloc projection if  necessary */ 
+    if((display->gimage->width != canvas_width(display->projection)) ||
+	    (display->gimage->height != canvas_height(display->projection)) ||
+       (canvas_tag(display->projection) != canvas_tag(gimage_projection(display->gimage))))
+    {   Canvas *new_canvas = canvas_new (canvas_tag(gimage_projection(display->gimage)), 
+					display->gimage->width, display->gimage->height,
+#ifdef NO_TILES						  
+						  STORAGE_FLAT);
+#else
+						  STORAGE_TILED);
+#endif
+     
+        /* if only tag changed, we won't get a full refresh, so copy old data */
+        if (canvas_tag(display->projection) != canvas_tag(gimage_projection(display->gimage)))
+        {   PixelArea src_area;
+            PixelArea dest_area;
+            pixelarea_init (&src_area, display->projection, 0,0,display->gimage->width,
+	  		     display->gimage->height, FALSE);
+	    pixelarea_init (&dest_area, new_canvas, 0,0,display->gimage->width,
+			    display->gimage->height, FALSE);
+	    copy_area(&src_area, &dest_area);
+	}
+        canvas_delete(display->projection);  
+	display->projection = new_canvas;
+    }
+
+
+    /* Stefan Klein: temporary solution, just copy projection calculated in gimage */
+    PixelArea image_projection_area;
+    PixelArea display_projection_area;
+
+    gimage_construct(display->gimage, x,y,w,h);
+    pixelarea_init (&image_projection_area, gimage_projection(display->gimage), x,y,w,h, FALSE);
+    pixelarea_init (&display_projection_area, display->projection, x,y,w,h, TRUE);
+     
+    copy_area (&image_projection_area, &display_projection_area);   
+
+    pixelarea_init (&display_projection_area, display->projection, x,y,w,h, TRUE);
+    /* apply display cms to projection */
+    if (transform_buffer != NULL)
+    {   cms_transform_area(transform_buffer, &display_projection_area, &display_projection_area); 
+    }  
+   
+}
+
+
 
 void
 gdisplay_flush (GDisplay *gdisp)
@@ -337,18 +433,68 @@ gdisplay_flush (GDisplay *gdisp)
 
   /*  First the updates...  */
   list = gdisp->update_areas;
+
+  /* if colormanaged, get the required transform */
+
+  if ((list !=NULL) && gdisp->colormanaged)
+  {   /* combine profiles */
+      GSList *profiles = NULL;
+      CMSProfile *image_profile = gimage_get_cms_profile(gdisp->gimage);
+      CMSProfile *display_profile = cms_get_display_profile();
+      if (image_profile == NULL)
+      {   g_warning("gdisplay_project: cannot colourmanage, image has no profile assigned");
+          transform_buffer = NULL;
+      }
+      else if (display_profile == NULL)
+      {   g_warning("gdisplay_project: cannot colourmanage, no display profile set");
+          transform_buffer = NULL;
+      }
+      else 
+      {   profiles = g_slist_append(profiles, (gpointer)image_profile);
+          profiles = g_slist_concat(profiles, g_slist_copy(gdisp->look_profile_pipe));
+          profiles = g_slist_append(profiles, (gpointer)display_profile);
+    
+          transform_buffer = cms_get_canvas_transform(profiles,
+                                           gimage_projection(gdisp->gimage),
+	                                   gdisp->cms_intent, gdisp->cms_flags);
+          g_slist_free(profiles);
+      }
+  } 
+  /* if we have got look profiles ... */
+  else if (!gdisp->colormanaged && (gdisp->look_profile_pipe != NULL))
+  {   GSList *profiles = NULL;
+      CMSProfile *image_profile = gimage_get_cms_profile(gdisp->gimage);
+      if (image_profile == NULL)
+      {   image_profile = cms_get_srgb_profile();
+      }
+      profiles = g_slist_append(profiles, (gpointer)image_profile);
+      profiles = g_slist_concat(profiles, g_slist_copy(gdisp->look_profile_pipe));
+      profiles = g_slist_append(profiles, (gpointer)image_profile);
+      transform_buffer = cms_get_canvas_transform(profiles,
+                                           gimage_projection(gdisp->gimage), 
+	                                   gdisp->cms_intent, gdisp->cms_flags);
+      g_slist_free(profiles);
+  }
+
   while (list)
     {
       /*  Paint the area specified by the GArea  */
       ga = (GArea *) list->data;
       gdisplay_paint_area (gdisp, ga->x1, ga->y1,
-			   (ga->x2 - ga->x1), (ga->y2 - ga->y1));
+                           (ga->x2 - ga->x1), (ga->y2 - ga->y1));
 
       list = g_slist_next (list);
     }
   /*  Free the update lists  */
   gdisp->update_areas = gdisplay_free_area_list (gdisp->update_areas);
 
+  /* delete the transform */
+  if (transform_buffer != NULL)
+  {   cms_return_transform(transform_buffer);
+      transform_buffer = NULL;
+  }
+
+
   /*  Next the displays...  */
   list = gdisp->display_areas;
 
@@ -358,14 +504,14 @@ gdisplay_flush (GDisplay *gdisp)
       active_tool_control (PAUSE, (void *) gdisp);
 
       while (list)
-	{
-	  /*  Paint the area specified by the GArea  */
-	  ga = (GArea *) list->data;
-	  gdisplay_display_area (gdisp, ga->x1, ga->y1,
-				 (ga->x2 - ga->x1), (ga->y2 - ga->y1));
+        {
+          /*  Paint the area specified by the GArea  */
+          ga = (GArea *) list->data;
+          gdisplay_display_area (gdisp, ga->x1, ga->y1,
+                                 (ga->x2 - ga->x1), (ga->y2 - ga->y1));
 
-	  list = g_slist_next (list);
-	}
+          list = g_slist_next (list);
+        }
       /*  Free the update lists  */
       gdisp->display_areas = gdisplay_free_area_list (gdisp->display_areas);
 
@@ -374,7 +520,7 @@ gdisplay_flush (GDisplay *gdisp)
 
       /* and the cursor (if we have a software cursor */
       if (gdisp->have_cursor)
-	gdisplay_draw_cursor (gdisp);
+        gdisplay_draw_cursor (gdisp);
 
       /* restart (and recalculate) the selection boundaries */
       selection_start (gdisp->select, TRUE);
@@ -386,7 +532,7 @@ gdisplay_flush (GDisplay *gdisp)
   /*  update the gdisplay's info dialog  */
   if (gdisp->window_info_dialog)
     info_window_update (gdisp->window_info_dialog,
-			(void *) gdisp);
+                        (void *) gdisp);
 }
 
 
@@ -400,19 +546,19 @@ gdisplay_draw_guides (GDisplay *gdisp)
     {
       tmp_list = gdisp->gimage->guides;
       while (tmp_list)
-	{
-	  guide = tmp_list->data;
-	  tmp_list = tmp_list->next;
+        {
+          guide = tmp_list->data;
+          tmp_list = tmp_list->next;
 
-	  gdisplay_draw_guide (gdisp, guide, FALSE);
-	}
+          gdisplay_draw_guide (gdisp, guide, FALSE);
+        }
     }
 }
 
 void
 gdisplay_draw_guide (GDisplay *gdisp,
-		     Guide    *guide,
-		     int       active)
+                     Guide    *guide,
+                     int       active)
 {
   static GdkGC *normal_hgc = NULL;
   static GdkGC *active_hgc = NULL;
@@ -432,14 +578,14 @@ gdisplay_draw_guide (GDisplay *gdisp,
       GdkGCValues values;
       guchar stipple[8] =
       {
-	0xF0,    /*  ####----  */
-	0xE1,    /*  ###----#  */
-	0xC3,    /*  ##----##  */
-	0x87,    /*  #----###  */
-	0x0F,    /*  ----####  */
-	0x1E,    /*  ---####-  */
-	0x3C,    /*  --####--  */
-	0x78,    /*  -####---  */
+        0xF0,    /*  ####----  */
+        0xE1,    /*  ###----#  */
+        0xC3,    /*  ##----##  */
+        0x87,    /*  #----###  */
+        0x0F,    /*  ----####  */
+        0x1E,    /*  ---####-  */
+        0x3C,    /*  --####--  */
+        0x78,    /*  -####---  */
       };
 
       initialize = FALSE;
@@ -449,34 +595,34 @@ gdisplay_draw_guide (GDisplay *gdisp,
       values.fill = GDK_OPAQUE_STIPPLED;
       values.stipple = gdk_bitmap_create_from_data (gdisp->canvas->window, (char*) stipple, 8, 1);
       normal_hgc = gdk_gc_new_with_values (gdisp->canvas->window, &values,
-					  GDK_GC_FOREGROUND |
-					  GDK_GC_BACKGROUND |
-					  GDK_GC_FILL |
-					  GDK_GC_STIPPLE);
+                                          GDK_GC_FOREGROUND |
+                                          GDK_GC_BACKGROUND |
+                                          GDK_GC_FILL |
+                                          GDK_GC_STIPPLE);
 
       values.background.pixel = g_active_guide_pixel;
       active_hgc = gdk_gc_new_with_values (gdisp->canvas->window, &values,
-					   GDK_GC_FOREGROUND |
-					   GDK_GC_BACKGROUND |
-					   GDK_GC_FILL |
-					   GDK_GC_STIPPLE);
+                                           GDK_GC_FOREGROUND |
+                                           GDK_GC_BACKGROUND |
+                                           GDK_GC_FILL |
+                                           GDK_GC_STIPPLE);
 
       values.foreground.pixel = gdisplay_black_pixel (gdisp);
       values.background.pixel = g_normal_guide_pixel;
       values.fill = GDK_OPAQUE_STIPPLED;
       values.stipple = gdk_bitmap_create_from_data (gdisp->canvas->window, (char*) stipple, 1, 8);
       normal_vgc = gdk_gc_new_with_values (gdisp->canvas->window, &values,
-					  GDK_GC_FOREGROUND |
-					  GDK_GC_BACKGROUND |
-					  GDK_GC_FILL |
-					  GDK_GC_STIPPLE);
+                                          GDK_GC_FOREGROUND |
+                                          GDK_GC_BACKGROUND |
+                                          GDK_GC_FILL |
+                                          GDK_GC_STIPPLE);
 
       values.background.pixel = g_active_guide_pixel;
       active_vgc = gdk_gc_new_with_values (gdisp->canvas->window, &values,
-					   GDK_GC_FOREGROUND |
-					   GDK_GC_BACKGROUND |
-					   GDK_GC_FILL |
-					   GDK_GC_STIPPLE);
+                                           GDK_GC_FOREGROUND |
+                                           GDK_GC_BACKGROUND |
+                                           GDK_GC_FILL |
+                                           GDK_GC_STIPPLE);
     }
 
   gdisplay_transform_coords (gdisp, 0, 0, &x1, &y1, FALSE);
@@ -493,25 +639,25 @@ gdisplay_draw_guide (GDisplay *gdisp,
       gdisplay_transform_coords (gdisp, 0, guide->position, &x, &y, FALSE);
 
       if (active)
-	gdk_draw_line (gdisp->canvas->window, active_hgc, x1, y, x2, y);
+        gdk_draw_line (gdisp->canvas->window, active_hgc, x1, y, x2, y);
       else
-	gdk_draw_line (gdisp->canvas->window, normal_hgc, x1, y, x2, y);
+        gdk_draw_line (gdisp->canvas->window, normal_hgc, x1, y, x2, y);
     }
   else if (guide->orientation == VERTICAL_GUIDE)
     {
       gdisplay_transform_coords (gdisp, guide->position, 0, &x, &y, FALSE);
 
       if (active)
-	gdk_draw_line (gdisp->canvas->window, active_vgc, x, y1, x, y2);
+        gdk_draw_line (gdisp->canvas->window, active_vgc, x, y1, x, y2);
       else
-	gdk_draw_line (gdisp->canvas->window, normal_vgc, x, y1, x, y2);
+        gdk_draw_line (gdisp->canvas->window, normal_vgc, x, y1, x, y2);
     }
 }
 
 Guide*
 gdisplay_find_guide (GDisplay *gdisp,
-		     int       x,
-		     int       y)
+                     int       x,
+                     int       y)
 {
   GList *tmp_list;
   Guide *guide;
@@ -527,28 +673,28 @@ gdisplay_find_guide (GDisplay *gdisp,
 
       tmp_list = gdisp->gimage->guides;
       while (tmp_list)
-	{
-	  guide = tmp_list->data;
-	  tmp_list = tmp_list->next;
+        {
+          guide = tmp_list->data;
+          tmp_list = tmp_list->next;
 
-	  switch (guide->orientation)
-	    {
-	    case HORIZONTAL_GUIDE:
-	      pos = (int) (scale * guide->position - offset_y);
-	      if ((guide->position != -1) &&
-		  (pos > (y - EPSILON)) &&
-		  (pos < (y + EPSILON)))
-		return guide;
-	      break;
-	    case VERTICAL_GUIDE:
-	      pos = (int) (scale * guide->position - offset_x);
-	      if ((guide->position != -1) &&
-		  (pos > (x - EPSILON)) &&
-		  (pos < (x + EPSILON)))
-		return guide;
-	      break;
-	    }
-	}
+          switch (guide->orientation)
+            {
+            case HORIZONTAL_GUIDE:
+              pos = (int) (scale * guide->position - offset_y);
+              if ((guide->position != -1) &&
+                  (pos > (y - EPSILON)) &&
+                  (pos < (y + EPSILON)))
+                return guide;
+              break;
+            case VERTICAL_GUIDE:
+              pos = (int) (scale * guide->position - offset_x);
+              if ((guide->position != -1) &&
+                  (pos > (x - EPSILON)) &&
+                  (pos < (x + EPSILON)))
+                return guide;
+              break;
+            }
+        }
     }
 
   return NULL;
@@ -556,10 +702,10 @@ gdisplay_find_guide (GDisplay *gdisp,
 
 void
 gdisplay_snap_point (GDisplay *gdisp,
-		     gdouble   x ,
-		     gdouble   y,
-		     gdouble   *tx,
-		     gdouble   *ty)
+                     gdouble   x ,
+                     gdouble   y,
+                     gdouble   *tx,
+                     gdouble   *ty)
 {
   GList *tmp_list;
   Guide *guide;
@@ -584,55 +730,55 @@ gdisplay_snap_point (GDisplay *gdisp,
 
       tmp_list = gdisp->gimage->guides;
       while (tmp_list)
-	{
-	  guide = tmp_list->data;
-	  tmp_list = tmp_list->next;
+        {
+          guide = tmp_list->data;
+          tmp_list = tmp_list->next;
 
-	  switch (guide->orientation)
-	    {
-	    case HORIZONTAL_GUIDE:
-	      pos = (int) (scale * guide->position - offset_y);
-	      if ((pos > (y - EPSILON)) &&
-		  (pos < (y + EPSILON)))
-		{
-		  dist = pos - y;
-		  dist = ABS (dist);
+          switch (guide->orientation)
+            {
+            case HORIZONTAL_GUIDE:
+              pos = (int) (scale * guide->position - offset_y);
+              if ((pos > (y - EPSILON)) &&
+                  (pos < (y + EPSILON)))
+                {
+                  dist = pos - y;
+                  dist = ABS (dist);
 
-		  if (dist < minhdist)
-		    {
-		      minhdist = dist;
-		      *ty = pos;
-		    }
-		}
-	      break;
-	    case VERTICAL_GUIDE:
-	      pos = (int) (scale * guide->position - offset_x);
-	      if ((pos > (x - EPSILON)) &&
-		  (pos < (x + EPSILON)))
-		{
-		  dist = pos - x;
-		  dist = ABS (dist);
+                  if (dist < minhdist)
+                    {
+                      minhdist = dist;
+                      *ty = pos;
+                    }
+                }
+              break;
+            case VERTICAL_GUIDE:
+              pos = (int) (scale * guide->position - offset_x);
+              if ((pos > (x - EPSILON)) &&
+                  (pos < (x + EPSILON)))
+                {
+                  dist = pos - x;
+                  dist = ABS (dist);
 
-		  if (dist < minvdist)
-		    {
-		      minvdist = dist;
-		      *tx = pos;
-		    }
-		}
-	      break;
-	    }
-	}
+                  if (dist < minvdist)
+                    {
+                      minvdist = dist;
+                      *tx = pos;
+                    }
+                }
+              break;
+            }
+        }
     }
 }
 
 void
 gdisplay_snap_rectangle (GDisplay *gdisp,
-			 int       x1,
-			 int       y1,
-			 int       x2,
-			 int       y2,
-			 int      *tx1,
-			 int      *ty1)
+                         int       x1,
+                         int       y1,
+                         int       x2,
+                         int       y2,
+                         int      *tx1,
+                         int      *ty1)
 {
   double nx1, ny1;
   double nx2, ny2;
@@ -648,14 +794,14 @@ gdisplay_snap_rectangle (GDisplay *gdisp,
       gdisplay_snap_point (gdisp, x2, y2, &nx2, &ny2);
 
       if (x1 != (int)nx1)
-	*tx1 = nx1;
+        *tx1 = nx1;
       else if (x2 != (int)nx2)
-	*tx1 = x1 + (nx2 - x2);
+        *tx1 = x1 + (nx2 - x2);
 
       if (y1 != (int)ny1)
-	*ty1 = ny1;
+        *ty1 = ny1;
       else if (y2 != (int)ny2)
-	*ty1 = y1 + (ny2 - y2);
+        *ty1 = y1 + (ny2 - y2);
     }
 }
 
@@ -666,23 +812,23 @@ gdisplay_draw_cursor (GDisplay *gdisp)
   int y = gdisp->cursor_y;
   
   gdk_draw_line (gdisp->canvas->window,
-		 gdisp->canvas->style->white_gc,
-		 x - 7, y-1, x + 7, y-1);
+                 gdisp->canvas->style->white_gc,
+                 x - 7, y-1, x + 7, y-1);
   gdk_draw_line (gdisp->canvas->window,
-		 gdisp->canvas->style->black_gc,
-		 x - 7, y, x + 7, y);
+                 gdisp->canvas->style->black_gc,
+                 x - 7, y, x + 7, y);
   gdk_draw_line (gdisp->canvas->window,
-		 gdisp->canvas->style->white_gc,
-		 x - 7, y+1, x + 7, y+1);
+                 gdisp->canvas->style->white_gc,
+                 x - 7, y+1, x + 7, y+1);
   gdk_draw_line (gdisp->canvas->window,
-		 gdisp->canvas->style->white_gc,
-		 x-1, y - 7, x-1, y + 7);
+                 gdisp->canvas->style->white_gc,
+                 x-1, y - 7, x-1, y + 7);
   gdk_draw_line (gdisp->canvas->window,
-		 gdisp->canvas->style->black_gc,
-		 x, y - 7, x, y + 7);
+                 gdisp->canvas->style->black_gc,
+                 x, y - 7, x, y + 7);
   gdk_draw_line (gdisp->canvas->window,
-		 gdisp->canvas->style->white_gc,
-		 x+1, y - 7, x+1, y + 7);
+                 gdisp->canvas->style->white_gc,
+                 x+1, y - 7, x+1, y + 7);
 }
 
 void
@@ -695,16 +841,16 @@ gdisplay_update_cursor (GDisplay *gdisp, int x, int y)
   /* Erase old cursor, if necessary */
 
   if (gdisp->have_cursor && (!new_cursor || x != gdisp->cursor_x ||
-			     y != gdisp->cursor_y))
+                             y != gdisp->cursor_y))
     {
       gdisplay_expose_area (gdisp, gdisp->cursor_x - 7,
-			    gdisp->cursor_y - 7,
-			    15, 15);
+                            gdisp->cursor_y - 7,
+                            15, 15);
       if (!new_cursor)
-	{
-	  gdisp->have_cursor = FALSE;
-	  gdisplay_flush (gdisp);
-	}
+        {
+          gdisp->have_cursor = FALSE;
+          gdisplay_flush (gdisp);
+        }
     }
   
   gdisp->have_cursor = new_cursor;
@@ -726,10 +872,10 @@ gdisplay_remove_and_delete (GDisplay *gdisp)
 
 void
 gdisplay_add_update_area (GDisplay *gdisp,
-			  int       x,
-			  int       y,
-			  int       w,
-			  int       h)
+                          int       x,
+                          int       y,
+                          int       w,
+                          int       h)
 {
   GArea * ga;
 
@@ -745,10 +891,10 @@ gdisplay_add_update_area (GDisplay *gdisp,
 
 static void
 gdisplay_add_display_area (GDisplay *gdisp,
-			   int       x,
-			   int       y,
-			   int       w,
-			   int       h)
+                           int       x,
+                           int       y,
+                           int       w,
+                           int       h)
 {
   GArea * ga;
 
@@ -765,10 +911,10 @@ gdisplay_add_display_area (GDisplay *gdisp,
 
 static void
 gdisplay_paint_area (GDisplay *gdisp,
-		     int       x,
-		     int       y,
-		     int       w,
-		     int       h)
+                     int       x,
+                     int       y,
+                     int       w,
+                     int       h)
 {
   int x1, y1, x2, y2;
 
@@ -781,9 +927,10 @@ gdisplay_paint_area (GDisplay *gdisp,
   y = y1;
   w = (x2 - x1);
   h = (y2 - y1);
-  /* composite the area */
-  gimage_construct (gdisp->gimage, x, y, w, h);
-  
+
+  /* re-project the area */
+  gdisplay_project (gdisp, x, y, w, h);
+      
   /*  display the area  */
   gdisplay_transform_coords (gdisp, x, y, &x1, &y1, FALSE);
   gdisplay_transform_coords (gdisp, x + w, y + h, &x2, &y2, FALSE);
@@ -1144,6 +1291,8 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
   menus_set_sensitive (buff, (precision != PRECISION_FLOAT16));
   g_snprintf(buff, 1024, "<Image%d>/Image/" PRECISION_FLOAT_STRING, gdisp->unique_id);
   menus_set_sensitive (buff, (precision != PRECISION_FLOAT));
+  g_snprintf(buff, 1024, "<Image%d>/Image/" PRECISION_BFP_STRING, gdisp->unique_id);
+  menus_set_sensitive (buff, (precision != PRECISION_BFP));
 
   g_snprintf(buff, 1024, "<Image%d>/Image/Colors/Color Balance", gdisp->unique_id);
   menus_set_sensitive (buff, (format == FORMAT_RGB));
@@ -1214,6 +1363,7 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
   menus_set_sensitive ("<Image>/Image/" PRECISION_U16_STRING, (precision != PRECISION_U16));
   menus_set_sensitive ("<Image>/Image/" PRECISION_FLOAT16_STRING, (precision != PRECISION_FLOAT16));
   menus_set_sensitive ("<Image>/Image/" PRECISION_FLOAT_STRING, (precision != PRECISION_FLOAT));
+  menus_set_sensitive ("<Image>/Image/" PRECISION_BFP_STRING, (precision != PRECISION_BFP));  
 #if 0
   menus_set_sensitive ("<Image>/Image/Indexed", (format != FORMAT_INDEXED));
 #endif
@@ -1253,6 +1403,34 @@ gdisplay_set_menu_sensitivity (GDisplay *gdisp)
   /* save selection to channel */
   menus_set_sensitive ("<Image>/Select/Save To Channel", !fs);
 
+  if(gdisp->cms_intent == INTENT_PERCEPTUAL)
+  {
+      g_snprintf(buff, 1024, "<Image%d>/View/Rendering Intent/Perceptual", gdisp->unique_id); 
+      menus_set_state (buff, TRUE);
+      menus_set_state ("<Image>/View/Rendering Intent/Perceptual", TRUE);
+  }
+  if(gdisp->cms_intent == INTENT_RELATIVE_COLORIMETRIC)
+  {
+      g_snprintf(buff, 1024, "<Image%d>/View/Rendering Intent/Relative Colorimetric", gdisp->unique_id); 
+      menus_set_state (buff, TRUE);
+      menus_set_state ("<Image>/View/Rendering Intent/Relative Colorimetric", TRUE);
+  }
+  if(gdisp->cms_intent == INTENT_SATURATION)
+  {
+      g_snprintf(buff, 1024, "<Image%d>/View/Rendering Intent/Saturation", gdisp->unique_id); 
+      menus_set_state (buff, TRUE);
+      menus_set_state ("<Image>/View/Rendering Intent/Saturation", TRUE);
+  }
+  if(gdisp->cms_intent == INTENT_ABSOLUTE_COLORIMETRIC)
+  {
+      g_snprintf(buff, 1024, "<Image%d>/View/Rendering Intent/Absolute Colorimetric", gdisp->unique_id); 
+      menus_set_state (buff, TRUE);
+      menus_set_state ("<Image>/View/Rendering Intent/Absolute Colorimetric", TRUE);
+  }
+  g_snprintf(buff, 1024, "<Image%d>/View/Colormanage Display", gdisp->unique_id);
+  menus_set_state (buff, gdisp->colormanaged);
+  menus_set_state ("<Image>/View/Colormanage Display", gdisp->colormanaged);
+
   menus_set_state ("<Image>/View/Toggle Rulers", GTK_WIDGET_VISIBLE (gdisp->origin) ? 1 : 0);
   menus_set_state ("<Image>/View/Toggle Guides", gdisp->draw_guides);
   menus_set_state ("<Image>/View/Snap To Guides", gdisp->snap_to_guides);
@@ -1301,6 +1479,15 @@ gdisplay_expose_full (GDisplay *gdisp)
 			     gdisp->disp_height);
 }
 
+void
+gdisplay_update_full (GDisplay *gdisp)
+{
+  gdisplay_add_update_area (gdisp, 0, 0,
+			     gdisp->gimage->width,
+			     gdisp->gimage->height);
+}
+
+
 /**************************************************/
 /*  Functions independent of a specific gdisplay  */
 /**************************************************/
@@ -1383,6 +1570,11 @@ gdisplay_get_unique_id (int ID)
   return NULL;
 }
 
+Canvas * 
+gdisplay_get_projection (GDisplay *display)
+{   return display->projection;
+}
+
 GDisplay *
 gdisplay_get_ID (int ID)
 {
@@ -1499,7 +1691,7 @@ gdisplays_expose_guide (int    ID,
 
 
 void
-gdisplays_update_full (int ID)
+gdisplays_update_gimage (int ID)
 {
   GDisplay *gdisp;
   GSList *list = display_list;
@@ -1556,6 +1748,21 @@ gdisplays_shrink_wrap (int ID)
 }
 
 
+void
+gdisplays_update_full ()
+{
+  GDisplay *gdisp;
+  GSList *list = display_list;
+
+  /*  traverse the linked list of displays, handling each one  */
+  while (list)
+    {
+      gdisp = (GDisplay *) list->data;
+      gdisplay_update_full (gdisp);
+      list = g_slist_next (list);
+    }
+}
+
 void
 gdisplays_expose_full ()
 {
@@ -1705,3 +1912,170 @@ gdisplay_find_display (int ID)
 
   return NULL;
 }
+
+void       
+gdisplay_set_cms_intent(GDisplay *display, int intent)
+{
+    if (display->colormanaged && display->cms_intent != intent)
+    {
+        display->cms_intent = intent;
+        gdisplay_update_full(display);
+        gdisplay_flush(display);
+    }
+}
+
+void       
+gdisplay_set_cms_flags(GDisplay *display, DWORD flags)
+{   display->cms_flags = flags;
+}
+
+gboolean       
+gdisplay_set_colormanaged(GDisplay *display, gboolean colormanaged)
+{   if (colormanaged != display->colormanaged)
+    {   if ((colormanaged == TRUE) && (gimage_get_cms_profile(display->gimage) == NULL))
+        {   g_warning("gdisplay_set_colormanaged: cannot colormanage. no display profile defined");  
+   	    return FALSE;
+	}
+        if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
+	{   g_warning("gdisplay_set_colormanaged: cannot colormanage. no image profile defined");
+            return FALSE;
+	}
+
+	display->colormanaged = colormanaged;
+	/* set the toggle menu */
+	GString *buffer = g_string_new(NULL);
+	g_string_sprintf(buffer, "<Image%d>/View/Colormanage Display", display->unique_id);
+	menus_set_state (buffer->str, colormanaged);
+	g_string_free(buffer, TRUE);
+
+	/* and update */
+	gdisplay_update_full (display);
+	gdisplay_flush (display);
+    }
+
+    return TRUE;
+}
+
+gboolean 
+gdisplay_all_set_colormanaged(gboolean colormanaged)
+{   if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
+    {   g_warning("gdisplay_all_set_colormanaged: cannot colormanage. no display profile defined");    
+        return FALSE;
+    }
+    
+    gboolean some_without_profile = FALSE; 
+    GSList *list = display_list;
+    GDisplay *current_display;
+    GString *buffer = g_string_new(NULL);
+
+    /*  traverse the linked list of displays  */
+    while (list)
+    {   current_display = (GDisplay *) list->data;
+        list = g_slist_next (list);
+        if ((colormanaged == TRUE) && (gimage_get_cms_profile(current_display->gimage) == NULL))
+        {   some_without_profile = TRUE;
+   	    continue;
+	}
+        
+	current_display->colormanaged = colormanaged;
+	/* set the toggle menu */
+	g_string_sprintf(buffer, "<Image%d>/View/Colormanage Display", current_display->unique_id);
+	menus_set_state (buffer->str, colormanaged);
+
+	/* and update */
+	gdisplay_update_full (current_display);
+	gdisplay_flush (current_display);
+
+    }
+    g_string_free(buffer, TRUE);
+    
+    if (some_without_profile) 
+    {   g_warning("gdisplay_all_set_colormanaged: cannot colormanage some displays, because the image doesn't have a profile assigned.");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+gboolean
+gdisplay_image_set_colormanaged(int image_id, gboolean colormanaged)
+{   if ((colormanaged == TRUE) && (cms_display_profile_name == NULL))
+    {   g_warning("gdisplay_image_set_colormanaged: cannot colormanage. no display profile defined");    
+        return FALSE;
+    }
+    
+    gboolean some_without_profile = FALSE; 
+    GSList *list = display_list;
+    GDisplay *current_display;
+    GString *buffer = g_string_new(NULL);
+
+    /*  traverse the linked list of displays  */
+    while (list)
+    {   current_display = (GDisplay *) list->data;
+        list = g_slist_next (list);
+        if (current_display->gimage->ID == image_id)
+        {   if ((colormanaged == TRUE) && (gimage_get_cms_profile(current_display->gimage) == NULL))
+	    {   some_without_profile = TRUE;
+   	        continue;
+	    }
+        
+	    current_display->colormanaged = colormanaged;
+	    /* set the toggle menu */
+	    g_string_sprintf(buffer, "<Image%d>/View/Colormanage Display", current_display->unique_id);
+	    menus_set_state (buffer->str, colormanaged);
+
+	    /* and update */
+	    gdisplay_update_full (current_display);
+	    gdisplay_flush (current_display);
+	}
+    }
+    g_string_free(buffer, TRUE);
+    
+    if (some_without_profile) 
+    {   g_warning("gdisplay_image_set_colormanaged: cannot colormanage some displays, because the image doesn't have a profile assigned.");
+        return FALSE;
+    }
+
+    return TRUE;
+}
+
+
+/* look profiles */
+void
+gdisplay_set_look_profile_pipe(GDisplay *display, GSList* pipe)
+{   /* return all profiles and free the old list */
+    GSList *iterator = display->look_profile_pipe;
+    while (iterator != NULL)
+    {   cms_return_profile((CMSProfile *)iterator->data);
+        iterator = g_slist_next(iterator);
+    }
+    g_slist_free(display->look_profile_pipe);
+    display->look_profile_pipe = pipe;
+
+    gdisplay_update_full(display);
+    gdisplay_flush(display);
+}
+
+/* gets its own reference to the profile,
+   the caller has to return his reference himself */
+void
+gdisplay_add_look_profile(GDisplay *display, CMSProfile *profile, gint index)
+{   /* there is no need to free the profiles themselves, 
+       this is taken care of by cms */
+    display->look_profile_pipe = g_slist_insert(display->look_profile_pipe, 
+						(gpointer)cms_duplicate_profile(profile), 
+						index);
+
+    gdisplay_update_full(display);
+    gdisplay_flush(display);
+}
+
+void
+gdisplay_remove_look_profile(GDisplay *display, CMSProfile *profile)
+{   /* return our reference to the profile and remove it from the list */
+    display->look_profile_pipe = g_slist_remove(display->look_profile_pipe, (gpointer)profile);
+    cms_return_profile(profile);
+
+    gdisplay_update_full(display);
+    gdisplay_flush(display);
+}
