@@ -23,6 +23,7 @@
 #include "appenv.h"
 #include "canvas.h"
 #include "channels_dialog.h"
+#include "cms.h"
 #include "drawable.h"
 #include "errors.h"
 #include "floating_sel.h"
@@ -32,6 +33,7 @@
 #include "gimage_mask.h"
 #include "indexed_palette.h"
 #include "interface.h"
+#include "image_render.h"
 #include "layers_dialog.h"
 #include "paint_funcs_area.h"
 #include "palette.h"
@@ -165,6 +167,8 @@ gimage_create (void)
   gimage->comp_preview_valid[2] = FALSE;
   gimage->comp_preview = NULL;
 
+  gimage->cms_profile = NULL;  
+
   image_list = g_slist_append (image_list, (void *) gimage);
 
   return gimage;
@@ -182,7 +186,7 @@ gimage_tag  (
 Format 
 gimage_format  (
                 GImage * gimage
-                )
+                ) 
 {
   return tag_format (gimage_tag (gimage));
 }
@@ -606,7 +610,7 @@ gimage_resize (GImage *gimage, int new_width, int new_height,
   channel_invalidate_previews (gimage->ID);
   layer_invalidate_previews (gimage->ID);
   gimage_invalidate_preview (gimage);
-  gdisplays_update_full (gimage->ID);
+  gdisplays_update_gimage (gimage->ID);
   gdisplays_shrink_wrap (gimage->ID);
 
   zoom_view_changed(gdisplay_get_from_gimage(gimage));
@@ -679,7 +683,7 @@ gimage_scale (GImage *gimage, int new_width, int new_height)
   channel_invalidate_previews (gimage->ID);
   layer_invalidate_previews (gimage->ID);
   gimage_invalidate_preview (gimage);
-  gdisplays_update_full (gimage->ID);
+  gdisplays_update_gimage (gimage->ID);
   gdisplays_shrink_wrap (gimage->ID);
 
   zoom_view_changed(gdisplay_get_from_gimage(gimage));
@@ -778,7 +782,7 @@ gimage_delete (GImage *gimage)
 {
   gimage->ref_count--;
 
-  if (gimage->ref_count < 0)
+  if (gimage->ref_count <= 0)
     {
       /*  free the undo list  */
       undo_free (gimage);
@@ -798,6 +802,9 @@ gimage_delete (GImage *gimage)
       gimage_free_layers (gimage);
       gimage_free_channels (gimage);
       channel_delete (gimage->selection_mask);
+      if (gimage->cms_profile != NULL) 
+      {   cms_return_profile(gimage->cms_profile);
+      }
 
       g_free (gimage);
 
@@ -1243,6 +1250,7 @@ gimage_construct_layers (GImage *gimage, int x, int y, int w, int h)
 {
   Layer * layer;
   int x1, y1, x2, y2;
+  /* src1PR is the projection, src2PR the layer that we are currently applying */
   PixelArea src1PR, src2PR, maskPR;
   PixelArea * mask;
   GSList *list = gimage->layers;
@@ -1494,6 +1502,8 @@ gimage_construct (GImage *gimage, int x, int y, int w, int h)
        gimage->construct_flag = 0;
        gimage_initialize_projection (gimage, x, y, w, h);
        gimage_construct_layers (gimage, x, y, w, h);
+       image_render_set_visible_channels(gimage, x, y, w, h);
+       gimage_construct_channels (gimage, x, y, w, h);
     }
 }
 
@@ -3092,6 +3102,7 @@ gimage_is_flat (GImage *gimage)
    *  2) no layer mask
    *  3) opacity == OPAQUE_OPACITY
    *  4) single layer has no alpha
+   *  5) no auxiliary channels
    */
   
   if ( (gimage->layers) &&
@@ -3113,10 +3124,9 @@ gimage_is_flat (GImage *gimage)
               (drawable_height (d) == gimage->height))
             {
               int alpha = layer_has_alpha (l);
-		if (!alpha) 
-		  {
-		    flat = TRUE;
-		  }
+	      if ((!alpha) && (gimage->channels == NULL))
+	      {   flat = TRUE;
+	      }
             }
         }
     }
@@ -3313,7 +3323,7 @@ gimage_projection_realloc (GImage *gimage)
 /*  Composition access functions                            */
 /************************************************************/
 
-static Canvas *
+Canvas *
 gimage_construct_composite_preview (GImage *gimage, int width, int height)
 {
   Layer * layer;
@@ -3603,3 +3613,67 @@ gimage_linked_drawable (GImage *gimage)
   return NULL;
 }
 
+void
+gimage_set_cms_profile(GImage *image, CMSProfile *profile)
+{   image->cms_profile = profile;    
+    if (profile == NULL) 
+    {   g_warning("gimage_set_cms_profile: profile is NULL, Colormanagement switched off for all displays of this image");
+        gdisplay_image_set_colormanaged(image->ID, FALSE);
+    } 
+    gdisplays_update_gimage(image->ID);
+    gdisplays_update_title(image->ID);
+}
+
+void gimage_transform_colors(GImage *image, CMSTransform *transform, gboolean allow_undo)
+{   if (allow_undo)
+    {   undo_push_group_start (image, GIMAGE_MOD_UNDO);
+    }
+
+    /*  Relax the floating selection  */
+    Layer* floating_layer = gimage_floating_sel (image);
+    if (floating_layer)
+    {   floating_sel_relax (floating_layer, TRUE);
+    }
+  
+    GSList *list = NULL;
+    /*  Convert all layers  */
+    for (list = image->layers; list; list = g_slist_next (list))
+    {   /* get the original layer */
+        Layer * layer = (Layer *) list->data;
+
+        /* get the data */
+        Canvas *tiles = drawable_data (GIMP_DRAWABLE (layer));
+        
+        /*  Push the layer on the undo stack  */
+	if (allow_undo) 
+	{   undo_push_image(image, GIMP_DRAWABLE(layer),0,0,
+			    drawable_width(GIMP_DRAWABLE(layer)),
+			    drawable_height(GIMP_DRAWABLE(layer)),
+			    0,0);
+	}
+
+        /* get the transform and convert */
+        PixelArea transform_area;        
+        pixelarea_init(&transform_area, tiles,0, 0, 
+	               drawable_width(GIMP_DRAWABLE(layer)), 
+ 	               drawable_height(GIMP_DRAWABLE(layer)), TRUE);
+        cms_transform_area(transform, &transform_area, &transform_area);
+    }
+
+    /*  Make sure the projection is up to date  */
+    gimage_projection_realloc (image);
+
+    /*  Rigor the floating selection  */
+    if (floating_layer)
+    {   floating_sel_rigor (floating_layer, TRUE);
+    }
+
+    if (allow_undo)
+    {   undo_push_group_end (image);
+    }
+
+    /*  shrink wrap and update all views  */
+    layer_invalidate_previews (image->ID);
+    gimage_invalidate_preview (image);
+    gdisplays_update_gimage (image->ID);
+}
