@@ -26,6 +26,7 @@
 #include "commands.h"
 #include "devices.h"
 #include "disp_callbacks.h"
+#include "dnd.h"
 #include "errors.h"
 #include "gdisplay.h"
 #include "gdisplay_ops.h"
@@ -42,8 +43,6 @@
 
 
 /*  local functions  */
-static void  tools_select_update   (GtkWidget *widget,
-				    gpointer   data);
 static gint  tools_button_press    (GtkWidget *widget,
 				    GdkEventButton *bevent,
 				    gpointer   data);
@@ -68,6 +67,22 @@ static GdkPixmap *create_pixmap    (GdkWindow  *parent,
 				    int         width,
 				    int         height);
 
+static void  tools_select_update   (GtkWidget *widget,
+				    gpointer   data);
+
+/* DnD */
+static void     toolbox_set_drag_dest      (GtkWidget        *widget);
+static gboolean toolbox_drag_drop          (GtkWidget        *widget,
+                        GdkDragContext   *context,
+                        gint              x,
+                        gint              y,
+                        guint             time);
+static ToolType toolbox_drag_tool          (GtkWidget        *widget,
+                        gpointer          data);
+static void     toolbox_drop_tool          (GtkWidget        *widget,
+                        ToolType          tool,
+                        gpointer          data);
+
 
 typedef struct ToolButton ToolButton;
 
@@ -153,6 +168,9 @@ static ToolButton tool_data[] =
   { (char **) text_bits,
     "Add text to the image",
     (gpointer) TEXT },
+  { (char **) spline_bits,
+    "Spline",
+    (gpointer) SPLINE },
 
   { (char **) colorpicker_bits,
     "Pick colors from the image",
@@ -193,7 +211,14 @@ static ToolButton tool_data[] =
   { (char **) measure_bits,
     "Measure",
     (gpointer) MEASURE },
-#if 0
+  { (char **) spline_edit_bits,
+    "Spline Edit",
+    (gpointer) SPLINE_EDIT },
+};
+
+/* non-visible tool buttons */
+static ToolButton tool_nonvis_data[] =
+{
   { NULL,
     NULL,
     (gpointer) BY_COLOR_SELECT },
@@ -224,9 +249,10 @@ static ToolButton tool_data[] =
   { NULL,
     NULL,
     (gpointer) GAMMA_EXPOSE }
-#endif
 };
 
+  GSList *toolbox_table_group=NULL;
+
 static int pixmap_colors[8][3] =
 {
   { 0x00, 0x00, 0x00 }, /* a - 0   */
@@ -237,17 +263,25 @@ static int pixmap_colors[8][3] =
   { 0xB6, 0xB6, 0xB6 }, /* f - 182 */
   { 0xDB, 0xDB, 0xDB }, /* g - 219 */
   { 0xFF, 0xFF, 0xFF }, /* h - 255 */
+#define NUM_TOOLS_NONVIS (sizeof (tool_nonvis_data) / sizeof (tool_nonvis_data[0]))
 };
 
 #define NUM_TOOLS (sizeof (tool_data) / sizeof (tool_data[0]))
+#ifdef VERTICAL_TOOLBAR
 #define COLUMNS   3
 #define ROWS      7 
+#else
+#define COLUMNS   13
+#define ROWS      2 
+#endif
 #define MARGIN    2
 
+gint num_tools = NUM_TOOLS;//sizeof (tool_info) / sizeof (tool_info[0]);
+
 /*  Widgets for each tool button--these are used from command.c to activate on
  *  tool selection via both menus and keyboard accelerators.
  */
-GtkWidget *tool_widgets[NUM_TOOLS];
+GtkWidget *tool_widgets[NUM_TOOLS+NUM_TOOLS_NONVIS];
 GtkWidget *tool_label;
 GtkTooltips *tool_tips;
 
@@ -261,7 +295,30 @@ GtkWidget *popup_shell = NULL;
 static GtkWidget *tool_label_area = NULL;
 static GtkWidget *progress_area = NULL;
 static GdkColor colors[12];
-static GtkWidget *toolbox_shell = NULL;
+GtkWidget *toolbox_shell = NULL;
+
+//static GtkWidget * toolbox_shell = NULL;
+
+static GtkTargetEntry toolbox_target_table[] =
+{
+  GIMP_TARGET_URI_LIST,
+  GIMP_TARGET_TEXT_PLAIN,
+  GIMP_TARGET_NETSCAPE_URL,
+  GIMP_TARGET_LAYER,
+  GIMP_TARGET_CHANNEL,
+  GIMP_TARGET_LAYER_MASK,
+  GIMP_TARGET_TOOL
+};
+static guint toolbox_n_targets = (sizeof (toolbox_target_table) /
+                  sizeof (toolbox_target_table[0]));
+
+static GtkTargetEntry tool_target_table[] =
+{
+  GIMP_TARGET_TOOL
+};
+static guint tool_n_targets = (sizeof (tool_target_table) /
+                   sizeof (tool_target_table[0]));
+
 
 static void
 tools_select_update (GtkWidget *w,
@@ -275,12 +332,15 @@ tools_select_update (GtkWidget *w,
     tools_select (tool_type);
 }
 
-static gint tools_button_press (GtkWidget *w,GdkEventButton *event,gpointer data)
-{	if( ((event->type == GDK_2BUTTON_PRESS) && (event->button == 1))
-	 || ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) )
-	{	tools_options_dialog_show ();
-	}
-	return FALSE;
+static gint 
+tools_button_press (GtkWidget *w,GdkEventButton *event,gpointer data)
+{	
+  if(((event->type == GDK_2BUTTON_PRESS) && (event->button == 1))
+     || ((event->type == GDK_BUTTON_PRESS) && (event->button == 3)) )
+    {
+      tools_options_dialog_show();
+    }
+  return FALSE;
 }
 
 static gint
@@ -377,7 +437,6 @@ create_color_area (GtkWidget *parent)
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
   gtk_box_pack_start (GTK_BOX (parent), frame, FALSE, FALSE, 0);
   gtk_widget_realize (frame);
-
   alignment = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
   gtk_container_border_width (GTK_CONTAINER (alignment), 3);
   gtk_container_add (GTK_CONTAINER (frame), alignment);
@@ -416,6 +475,7 @@ create_tool_pixmap (GtkWidget *parent, ToolType type)
   return NULL;	/* not reached */
 }
 
+
 static void
 create_tools (GtkWidget *parent)
 {
@@ -431,19 +491,17 @@ create_tools (GtkWidget *parent)
   gtk_box_pack_start (GTK_BOX (parent), table, TRUE, TRUE, 0);
   gtk_widget_realize (table);
 
-  group = NULL;
-
   for (i = 0; i < NUM_TOOLS; i++)
     {
-      tool_widgets[i] = button = gtk_radio_button_new (group);
+      tool_widgets[i] = button = gtk_radio_button_new (toolbox_table_group);
       gtk_container_border_width (GTK_CONTAINER (button), 0);
-      group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
+      toolbox_table_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
 
       gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (button), FALSE);
 
       gtk_table_attach (GTK_TABLE (table), button,
-			(i % 3), (i % 3) + 1,
-			(i / 3), (i / 3) + 1,
+			(i % COLUMNS), (i % COLUMNS) + 1,
+			(i / COLUMNS), (i / COLUMNS) + 1,
 			GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			0, 0);
@@ -469,15 +527,24 @@ create_tools (GtkWidget *parent)
       gtk_widget_show (alignment);
       gtk_widget_show (button);
     }
-  /*  The non-visible tool buttons  */
-  for (i = NUM_TOOLS+1; i < NUM_TOOLS; i++)
+    /*  The non-visible tool buttons  */
+    for (; i < num_tools; i++)
     {
-      tool_widgets[i] = button = gtk_radio_button_new (group);
-      group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
+      tool_widgets[i] = button = gtk_radio_button_new (toolbox_table_group);
+      toolbox_table_group = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
+
+      /*  dnd stuff  */
+      gtk_drag_source_set (tool_widgets[i],
+                   GDK_BUTTON2_MASK,
+                   tool_target_table, tool_n_targets,
+                   GDK_ACTION_COPY);
+      gimp_dnd_tool_source_set (tool_widgets[i],
+                    toolbox_drag_tool,
+                    GINT_TO_POINTER (i));
 
       gtk_signal_connect (GTK_OBJECT (button), "clicked",
 			  (GtkSignalFunc) tools_select_update,
-			  tool_data[i].callback_data);
+			  tool_nonvis_data[i].callback_data);
     }
 
   gtk_widget_show (table);
@@ -584,15 +651,15 @@ void
 create_toolbox ()
 {
   GtkWidget *window;
-  GtkWidget *main_vbox;
-  GtkWidget *vbox;
+  GtkWidget *main_box;
+  GtkWidget *box;
   GtkWidget *menubar;
   GList     *device_list;
   GtkAccelGroup *table;
   
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_window_set_wmclass (GTK_WINDOW (window), "toolbox", PROGRAM_NAME);
-  gtk_window_set_title (GTK_WINDOW (window),"  " /*"CP"*/ /*PROGRAM_NAME*/);
+  gtk_window_set_title (GTK_WINDOW (window),/*"CP"*/ PROGRAM_NAME " " PROGRAM_VERSION);
   gtk_widget_set_uposition (window, toolbox_x, toolbox_y);
   layout_connect_window_position(window, &toolbox_x, &toolbox_y, TRUE);
   minimize_track_toolbox(window);
@@ -628,14 +695,17 @@ create_toolbox ()
       gtk_widget_set_events (window, GDK_POINTER_MOTION_MASK);
       gtk_widget_set_extension_events (window, GDK_EXTENSION_EVENTS_CURSOR);
     }
-
-  main_vbox = gtk_vbox_new (FALSE, 1);
-  gtk_container_border_width (GTK_CONTAINER (main_vbox), 1);
-  gtk_container_add (GTK_CONTAINER (window), main_vbox);
-  gtk_widget_show (main_vbox);
+#ifndef VERTICAL_TOOLBAR
+  main_box = gtk_vbox_new (FALSE, 1);
+#else
+  main_box = gtk_hbox_new (FALSE, 1);
+#endif
+  gtk_container_border_width (GTK_CONTAINER (main_box), 1);
+  gtk_container_add (GTK_CONTAINER (window), main_box);
+  gtk_widget_show (main_box);
 
   /*  allocate the colors for creating pixmaps  */
-  allocate_colors (main_vbox);
+  allocate_colors (main_box);
 
   /*  tooltips  */
   tool_tips = gtk_tooltips_new ();
@@ -644,30 +714,35 @@ create_toolbox ()
 #else
   gtk_tooltips_set_colors (tool_tips,
 			   &colors[11],
-			   &main_vbox->style->fg[GTK_STATE_NORMAL]);
+			   &main_box->style->fg[GTK_STATE_NORMAL]);
 #endif
   if (!show_tool_tips)
     gtk_tooltips_disable (tool_tips);
 
   /*  Build the menu bar with menus  */
   menus_get_toolbox_menubar (&menubar, &table);
-  gtk_box_pack_start (GTK_BOX (main_vbox), menubar, FALSE, TRUE, 0);
+  gtk_box_pack_start (GTK_BOX (main_box), menubar, FALSE, TRUE, 0);
   gtk_widget_show (menubar);
 
   /*  Install the accelerator table in the main window  */
   gtk_window_add_accel_group (GTK_WINDOW (window), table);
+#ifdef VERTICAL_TOOLBAR
+  box = gtk_vbox_new (FALSE, 1);
+#else
+  box = gtk_hbox_new (FALSE, 1);
+#endif
+  gtk_box_pack_start (GTK_BOX (main_box), box, TRUE, TRUE, 0);
+  gtk_container_border_width (GTK_CONTAINER (box), 0);
+  gtk_widget_show (box);
 
-  vbox = gtk_vbox_new (FALSE, 1);
-  gtk_box_pack_start (GTK_BOX (main_vbox), vbox, TRUE, TRUE, 0);
-  gtk_container_border_width (GTK_CONTAINER (vbox), 0);
-  gtk_widget_show (vbox);
-
-  create_tools (vbox);
-  /*create_tool_label (vbox);*/
-  /*create_progress_area (vbox);*/
-  create_color_area (vbox);
+  create_tools (box);
+  /*create_tool_label (box);*/
+  /*create_progress_area (box);*/
+  create_color_area (box);
 
   gtk_widget_show (window);
+  toolbox_set_drag_dest (window);
+
   toolbox_shell = window;
 }
 
@@ -882,7 +957,8 @@ create_display_shell (int   gdisp_unique_id,
   gtk_widget_show (gdisp->hsb);
   gtk_widget_show (gdisp->vsb);
 
-  if (show_rulers)
+  //  if (show_rulers)
+  if(enable_rulers_on_start)
     {
       gtk_widget_show (gdisp->origin);
       gtk_widget_show (gdisp->hrule);
@@ -903,6 +979,194 @@ create_display_shell (int   gdisp_unique_id,
   gtk_widget_grab_focus (gdisp->canvas);
 }
 
+/* DnD functions */
+static void
+toolbox_set_drag_dest (GtkWidget *object)
+{
+  gtk_drag_dest_set (object,
+             GTK_DEST_DEFAULT_ALL,
+             toolbox_target_table, toolbox_n_targets,
+             GDK_ACTION_COPY);
+
+  gimp_dnd_file_dest_set (object);
+
+  gtk_signal_connect (GTK_OBJECT (object), "drag_drop",
+              GTK_SIGNAL_FUNC (toolbox_drag_drop),
+              NULL);
+
+  gimp_dnd_tool_dest_set (object, toolbox_drop_tool, NULL);
+}
+
+static gboolean
+toolbox_drag_drop (GtkWidget      *widget,
+           GdkDragContext *context,
+           gint            x,
+           gint            y,
+           guint           time)
+{
+  GtkWidget *src_widget;
+  gboolean return_val = FALSE;
+/*
+  if ((src_widget = gtk_drag_get_source_widget (context)))
+    {
+      GDrawable *drawable       = NULL;
+      Layer        *layer          = NULL;
+      Channel      *channel        = NULL;
+      LayerMask    *layer_mask     = NULL;
+      GImage       *component      = NULL;
+      ChannelType   component_type = -1;
+
+      layer = (Layer *) gtk_object_get_data (GTK_OBJECT (src_widget),
+                         "gimp_layer");
+      channel = (Channel *) gtk_object_get_data (GTK_OBJECT (src_widget),
+                         "gimp_channel");
+      layer_mask = (LayerMask *) gtk_object_get_data (GTK_OBJECT (src_widget),
+                              "gimp_layer_mask");
+      component = (GImage *) gtk_object_get_data (GTK_OBJECT (src_widget),
+                          "gimp_component");
+
+      if (layer)
+    {
+      drawable = GIMP_DRAWABLE (layer);
+    }
+      else if (channel)
+    {
+      drawable = GIMP_DRAWABLE (channel);
+    }
+      else if (layer_mask)
+    {
+      drawable = GIMP_DRAWABLE (layer_mask);
+    }
+      else if (component)
+    {
+      component_type =
+        (ChannelType) gtk_object_get_data (GTK_OBJECT (src_widget),
+                           "gimp_component_type");
+    }
+
+      if (drawable)
+    {
+          GImage *gimage;
+      GImage *new_gimage;
+      Layer  *new_layer;
+          gint    width, height;
+      gint    off_x, off_y;
+      gint    bytes;
+
+      GimpImageBaseType type;
+
+      gimage = gimp_drawable_gimage (drawable);
+          width  = gimp_drawable_width  (drawable);
+          height = gimp_drawable_height (drawable);
+      bytes  = gimp_drawable_bytes  (drawable);
+
+      switch (gimp_drawable_type (drawable))
+        {
+        case RGB_GIMAGE: case RGBA_GIMAGE:
+          type = RGB; break;
+        case GRAY_GIMAGE: case GRAYA_GIMAGE:
+          type = GRAY; break;
+        case INDEXED_GIMAGE: case INDEXEDA_GIMAGE:
+          type = INDEXED; break;
+        default:
+          type = RGB; break;
+        }
+
+      new_gimage = gimage_new (width, height, type);
+      gimage_disable_undo (new_gimage);
+
+      if (type == INDEXED)*/ /* copy the colormap */
+/*        {
+          new_gimage->num_cols = gimage->num_cols;
+          memcpy (new_gimage->cmap, gimage->cmap, COLORMAP_SIZE);
+        }
+
+      gimage_set_resolution (new_gimage, gimage->xresolution, gimage->yresolution);
+      gimage_set_unit (new_gimage, gimage->unit);
+
+      if (layer)
+        {
+          new_layer = layer_copy (layer, FALSE);
+        }
+      else
+        {
+*/          /*  a non-layer drawable can't have an alpha channel,
+           *  so add one
+           */
+/*          PixelRegion  srcPR, destPR;
+          TileManager *tiles;
+
+          tiles = tile_manager_new (width, height, bytes + 1);
+
+          pixel_region_init (&srcPR, gimp_drawable_data (drawable),
+                 0, 0, width, height, FALSE);
+          pixel_region_init (&destPR, tiles,
+                 0, 0, width, height, TRUE);
+
+          add_alpha_region (&srcPR, &destPR);
+
+          new_layer = layer_new_from_tiles (new_gimage,
+                                                gimp_image_base_type_with_alpha(new_gimage),
+                                                tiles,
+                        "", OPAQUE_OPACITY, NORMAL_MODE);
+
+          tile_manager_destroy (tiles);
+        }
+
+      gimp_drawable_set_gimage (GIMP_DRAWABLE (new_layer), new_gimage);
+
+      layer_set_name (GIMP_LAYER (new_layer),
+              gimp_drawable_get_name (drawable));
+
+      if (layer)
+        {
+          LayerMask *mask;
+          LayerMask *new_mask;
+
+          mask     = layer_get_mask (layer);
+          new_mask = layer_get_mask (new_layer);
+
+          if (new_mask)
+        {
+          gimp_drawable_set_name (GIMP_DRAWABLE (new_mask),
+                      gimp_drawable_get_name (GIMP_DRAWABLE (mask)));
+        }
+        }
+
+      gimp_drawable_offsets (GIMP_DRAWABLE (new_layer), &off_x, &off_y);
+      layer_translate (new_layer, -off_x, -off_y);
+
+      gimage_add_layer (new_gimage, new_layer, 0);
+
+      gimp_context_set_display (gimp_context_get_user (),
+                    gdisplay_new (new_gimage, 0x0101));
+
+      gimage_enable_undo (new_gimage);
+
+      return_val = TRUE;
+    }
+    }
+*/
+  gtk_drag_finish (context, return_val, FALSE, time);
+
+  return return_val;
+}
+
+static ToolType
+toolbox_drag_tool (GtkWidget *widget,
+           gpointer   data)
+{
+  return (ToolType) data;
+}
+
+static void
+toolbox_drop_tool (GtkWidget *widget,
+           ToolType   tool,
+           gpointer   data)
+{
+  //gimp_context_set_tool (gimp_context_get_user (), tool);
+}
+
 
 /*
  *  A text string query box
