@@ -33,8 +33,16 @@
 #include "gdisplay.h"
 #include "brushgenerated.h"
 #include "clone.h"
+#include "spline.h"
 #include "store_frame_manager.h"
 
+/* rsr: ld error hack, this is a bug: */
+#define sfm_slide_show 0
+
+#if 0
+#define TEAR_OFF
+#endif
+
 #if 1 
 #define R_AND_H 1
 #endif
@@ -54,11 +62,24 @@ menus_strip_extra_entries(const char *tmp_filename, const char *filename);
 
 static GtkItemFactoryEntry toolbox_entries[] =
 {
+#ifdef TEAR_OFF
   { "/File/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/File/New...", "<alt>N", file_new_cmd_callback, 0 },
+  { "/File/New From", NULL, NULL, 0, "<Branch>" },
+#ifdef TEAR_OFF
+  { "/File/New From/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/File/Open...", "<alt>O", file_open_cmd_callback, 0 },
-  { "/File/Preferences...", NULL, file_pref_cmd_callback, 0 },
+  { "/File/Acquire", NULL, NULL, 0, "<Branch>" },
+#ifdef TEAR_OFF
+  { "/File/Acquire/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/File/---", NULL, NULL, 0, "<Separator>" },
+  { "/File/Preferences...", NULL, file_pref_cmd_callback, 0 },
+#ifdef TEAR_OFF
+  { "/File/Dialogs/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/File/Dialogs/Brushes...", "<control>B", dialogs_brushes_cmd_callback, 0 },
 #if 0
   { "/File/Dialogs/Patterns...", "<control><shift>P", dialogs_patterns_cmd_callback, 0 },
@@ -66,10 +87,13 @@ static GtkItemFactoryEntry toolbox_entries[] =
   { "/File/Dialogs/Palette...", "<control>P", dialogs_palette_cmd_callback, 0 },
   { "/File/Dialogs/Gradient Editor...", "<control>G", dialogs_gradient_editor_cmd_callback, 0 },
   { "/File/Dialogs/Tool Options...", "<control>T", dialogs_tools_options_cmd_callback, 0 },
-  { "/File/Device Dialog", NULL, dialogs_input_devices_cmd_callback, 0}, 
   { "/File/Dialogs/Device Status...", NULL, dialogs_device_status_cmd_callback, 0 },
+  { "/File/Device Dialog", NULL, dialogs_input_devices_cmd_callback, 0}, 
   { "/File/---", NULL, NULL, 0, "<Separator>" },
   { "/File/Quit", "<alt>Q", file_quit_cmd_callback, 0 },
+#ifdef TEAR_OFF
+  { "/Help/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Help/About...", NULL, about_dialog_cmd_callback, 0 },
   { "/Help/Bugs & Kudos...", NULL, bugs_dialog_cmd_callback, 0 },
   { "/Help/Tips...", NULL, tips_dialog_cmd_callback, 0 },
@@ -80,20 +104,24 @@ static GtkItemFactory *toolbox_factory = NULL;
 
 static GtkItemFactoryEntry image_entries[] =
 {
+#ifdef TEAR_OFF
   { "/File/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/File/New...", "<alt>N", file_new_cmd_callback, 1 },
   { "/File/Revert to Saved", "<alt>R", file_reload_cmd_callback, 0 },
   { "/File/Open...", "<alt>O", file_open_cmd_callback, 0 },
 #ifndef RnH_STYLE
-  { "/File/Frame Manager/tearoff1", NULL, NULL, 0, "<Tearoff>" },
-  { "/File/Frame Manager/create", "<control>F", dialogs_store_frame_manager_cmd_callback, 0 },
-  { "/File/Frame Manager/flip forward", "W", dialogs_store_frame_manager_flip_forward_cmd_callback, 0 },
-  { "/File/Frame Manager/flip backward", "Q", dialogs_store_frame_manager_flip_backwards_cmd_callback, 0 },
-  { "/File/Frame Manager/frame forward", "S", dialogs_store_frame_manager_adv_forward_cmd_callback, 0 },
-  { "/File/Frame Manager/frame backward", "A", dialogs_store_frame_manager_adv_backwards_cmd_callback, 0 },
-  { "/File/Frame Manager/play forward", "Q", dialogs_store_frame_manager_play_forward_cmd_callback, 0 },
-  { "/File/Frame Manager/play backward", "Q", dialogs_store_frame_manager_play_backwards_cmd_callback, 0 },
-  { "/File/Frame Manager/stop", "Q", dialogs_store_frame_manager_stop_cmd_callback, 0 },
+#ifdef TEAR_OFF
+  { "/File/Flipbook/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
+  { "/File/Flipbook/create", "<control>F", dialogs_store_frame_manager_cmd_callback, 0 },
+  { "/File/Flipbook/flip forward", "W", dialogs_store_frame_manager_flip_forward_cmd_callback, 0 },
+  { "/File/Flipbook/flip backward", "Q", dialogs_store_frame_manager_flip_backwards_cmd_callback, 0 },
+  { "/File/Flipbook/frame forward", "S", dialogs_store_frame_manager_adv_forward_cmd_callback, 0 },
+  { "/File/Flipbook/frame backward", "A", dialogs_store_frame_manager_adv_backwards_cmd_callback, 0 },
+  { "/File/Flipbook/play forward", "Q", dialogs_store_frame_manager_play_forward_cmd_callback, 0 },
+  { "/File/Flipbook/play backward", "Q", dialogs_store_frame_manager_play_backwards_cmd_callback, 0 },
+  { "/File/Flipbook/stop", "Q", dialogs_store_frame_manager_stop_cmd_callback, 0 },
 #endif
   { "/File/Save", "<alt>S", file_save_cmd_callback, 0 },
   { "/File/Save as...", "<alt><shift>s", file_save_as_cmd_callback, 0 },
@@ -108,7 +136,9 @@ static GtkItemFactoryEntry image_entries[] =
   { "/File/Close", "<alt>W", file_close_cmd_callback, 0 },
   { "/File/Quit", "<alt>Q", file_quit_cmd_callback, 0 },
   { "/File/---", NULL, NULL, 0, "<Separator>" },
+#ifdef TEAR_OFF
   { "/Edit/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Edit/Undo", "<alt>Z", edit_undo_cmd_callback, 0 },
   { "/Edit/Redo", "<alt><shift>z", edit_redo_cmd_callback, 0 },
   { "/Edit/---", NULL, NULL, 0, "<Separator>" },
@@ -124,7 +154,9 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Edit/Copy Named", "<control><alt>C", edit_named_copy_cmd_callback, 0 },
   { "/Edit/Paste Named", "<control><alt>V", edit_named_paste_cmd_callback, 0 },
   { "/Edit/---", NULL, NULL, 0, "<Separator>" },
+#ifdef TEAR_OFF
   { "/Select/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Select/Toggle", "<control>T", select_toggle_cmd_callback, 0 },
   { "/Select/Invert", "<control>I", select_invert_cmd_callback, 0 },
   { "/Select/All", "<control>A", select_all_cmd_callback, 0 },
@@ -139,9 +171,22 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Select/Shrink", NULL, select_shrink_cmd_callback, 0 },
   { "/Select/Save To Channel", NULL, select_save_cmd_callback, 0 },
   { "/Select/By Color...", NULL, select_by_color_cmd_callback, 0 },
+  { "/View/", "plus", view_zoomin_cmd_callback, 0 },
+#ifdef TEAR_OFF
   { "/View/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
+  { "/View/Colormanage Display", NULL, view_colormanage_display_cmd_callback, 0, "<ToggleItem>" },
+  { "/View/Rendering Intent/Perceptual", NULL, view_rendering_intent_cmd_callback, INTENT_PERCEPTUAL, "<RadioItem>"},
+  { "/View/Rendering Intent/Relative Colorimetric", NULL, view_rendering_intent_cmd_callback, INTENT_RELATIVE_COLORIMETRIC, "/View/Rendering Intent/Perceptual" },
+  { "/View/Rendering Intent/Saturation", NULL, view_rendering_intent_cmd_callback, INTENT_SATURATION, "/View/Rendering Intent/Relative Colorimetric"},
+  { "/View/Rendering Intent/Absolute Colorimetric", NULL, view_rendering_intent_cmd_callback, INTENT_ABSOLUTE_COLORIMETRIC, "/View/Rendering Intent/Saturation"},
+  { "/View/Look Profiles...", NULL, view_look_profiles_cmd_callback, 0 },
+  { "/View/---", NULL, NULL, 0, "<Separator>" },
+
   { "/View/Pan Zoom Window", NULL, view_pan_zoom_window_cb, 0 },
+#ifdef TEAR_OFF
   { "/View/Zoom Bookmarks/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/View/Zoom Bookmarks/1", "<control>1", view_zoom_bookmark0_cb, 0},
   { "/View/Zoom Bookmarks/2", "<control>2", view_zoom_bookmark1_cb, 0},
   { "/View/Zoom Bookmarks/3", "<control>3", view_zoom_bookmark2_cb, 0},
@@ -151,7 +196,9 @@ static GtkItemFactoryEntry image_entries[] =
   { "/View/Zoom Bookmarks/Save", NULL, view_zoom_bookmark_save_cb, 0},
   { "/View/Zoom In", "equal", view_zoomin_cmd_callback, 0 },
   { "/View/Zoom Out", "minus", view_zoomout_cmd_callback, 0 },
+#ifdef TEAR_OFF
   { "/View/Zoom/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/View/Zoom/16:1", NULL, view_zoom_16_1_callback, 0 },
   { "/View/Zoom/8:1", NULL, view_zoom_8_1_callback, 0 },
   { "/View/Zoom/4:1", NULL, view_zoom_4_1_callback, 0 },
@@ -166,16 +213,21 @@ static GtkItemFactoryEntry image_entries[] =
   { "/View/Toggle Guides", "<control><shift>T", view_toggle_guides_cmd_callback, 0, "<ToggleItem>" },
   { "/View/Snap To Guides", NULL, view_snap_to_guides_cmd_callback, 0, "<ToggleItem>" },
   { "/View/---", NULL, NULL, 0, "<Separator>" },
+  { "/View/Slide Show", NULL, sfm_slide_show, 0, NULL },
+  { "/View/---", NULL, NULL, 0, "<Separator>" },
   { "/View/New View", NULL, view_new_view_cmd_callback, 0 }, 
   { "/View/Shrink Wrap", "<control>E", view_shrink_wrap_cmd_callback, 0 },
+#ifdef TEAR_OFF
   { "/Image/tearoff1", NULL, NULL, 0, "<Tearoff>" },
   { "/Image/Colors/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Image/Colors/Equalize", NULL, image_equalize_cmd_callback, 0 },
   { "/Image/Colors/Invert", NULL, image_invert_cmd_callback, 0 },
   { "/Image/Colors/Posterize", NULL, image_posterize_cmd_callback, 0 },
   { "/Image/Colors/Threshold", NULL, image_threshold_cmd_callback, 0 },
   { "/Image/Colors/---", NULL, NULL, 0, "<Separator>" },
   { "/Image/Colors/Color Balance", NULL, image_color_balance_cmd_callback, 0 },
+  { "/Image/Colors/Color Correction", "<control>C", image_color_correction_cmd_callback, 0 },
   { "/Image/Colors/Brightness-Contrast", NULL, image_brightness_contrast_cmd_callback, 0 },
   { "/Image/Colors/Hue-Saturation", NULL, image_hue_saturation_cmd_callback, 0 },
   { "/Image/Colors/Gamma-Expose", NULL, image_gamma_expose_cmd_callback, 0 },
@@ -192,6 +244,7 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Image/Channel Ops/Offset", "<control><shift>O", channel_ops_offset_cmd_callback, 0 },
 #if 1
   { "/Image/Alpha/Add Alpha Channel", NULL, layers_add_alpha_channel_cmd_callback, 0 },
+  { "/Layers/Stroke Splines", NULL, spline_stroke_cmd_callback, 0 },
 #endif
   { "/Image/---", NULL, NULL, 0, "<Separator>" },
   { "/Image/RGB", NULL, image_convert_rgb_cmd_callback, 0 },
@@ -203,13 +256,18 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Image/" PRECISION_U8_STRING, NULL, image_convert_depth_cmd_callback, PRECISION_U8 },
   { "/Image/" PRECISION_U16_STRING, NULL, image_convert_depth_cmd_callback, PRECISION_U16 },
   { "/Image/" PRECISION_FLOAT16_STRING, NULL, image_convert_depth_cmd_callback, PRECISION_FLOAT16 },
+  { "/Image/" PRECISION_BFP_STRING, NULL, image_convert_depth_cmd_callback, PRECISION_BFP },  
   { "/Image/" PRECISION_FLOAT_STRING, NULL, image_convert_depth_cmd_callback, PRECISION_FLOAT },
   { "/Image/---", NULL, NULL, 0, "<Separator>" },
+  { "/Image/Convert using ICC Profile...", NULL, image_convert_colorspace_cmd_callback, 0 },
+  { "/Image/Assign ICC Profile...", NULL, image_assign_cms_profile_cmd_callback, 0 },
   { "/Image/Resize", NULL, image_resize_cmd_callback, 0 },
   { "/Image/Scale", NULL, image_scale_cmd_callback, 0 },
   { "/Image/---", NULL, NULL, 0, "<Separator>" },
   { "/Image/Histogram", NULL, image_histogram_cmd_callback, 0 },
+#ifdef TEAR_OFF
   { "/Layers/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Layers/Layers & Channels...", "<control>L", dialogs_lc_cmd_callback, 0 },
   { "/Layers/Raise Layer", "<shift>F", layers_raise_cmd_callback, 0 },
   { "/Layers/Lower Layer", "<shift>B", layers_lower_cmd_callback, 0 },
@@ -225,8 +283,10 @@ static GtkItemFactoryEntry image_entries[] =
 #endif
   { "/Layers/Make Layer", NULL, layers_add_alpha_channel_cmd_callback, 0 },
 #if 1
+#ifdef TEAR_OFF
   { "/Tools/tearoff1", NULL, NULL, 0, "<Tearoff>" },
   { "/Tools/Select Tools/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Tools/Select Tools/Rect Select", NULL, tools_select_cmd_callback, RECT_SELECT },
   { "/Tools/Select Tools/Ellipse Select", NULL, tools_select_cmd_callback, ELLIPSE_SELECT },
   { "/Tools/Select Tools/Free Select", NULL, tools_select_cmd_callback, FREE_SELECT },
@@ -235,7 +295,9 @@ static GtkItemFactoryEntry image_entries[] =
 #if 1
   { "/Tools/Select Tools/Intelligent Scissors", "I", tools_select_cmd_callback, ISCISSORS },
 #endif
+#ifdef TEAR_OFF
   { "/Tools/Transform Tools/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Tools/Transform Tools/Move", NULL, tools_select_cmd_callback, MOVE },
   { "/Tools/Transform Tools/Magnify", NULL, tools_select_cmd_callback, MAGNIFY },
   { "/Tools/Transform Tools/Crop", NULL, tools_select_cmd_callback, CROP },
@@ -243,10 +305,13 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Tools/Transform Tools/Flip", NULL, tools_select_cmd_callback, FLIP_HORZ },
   { "/Tools/Text", "T", tools_select_cmd_callback, TEXT },
   { "/Tools/Color Picker", NULL, tools_select_cmd_callback, COLOR_PICKER },
+  { "/Tools/Splines", "S", tools_select_cmd_callback, SPLINE },
 #if 0
   { "/Tools/Color Picker Down", "I", select_color_picker, NULL },
 #endif
+#ifdef TEAR_OFF
   { "/Tools/Paint Tools/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Tools/Paint Tools/Bucket Fill", NULL, tools_select_cmd_callback, BUCKET_FILL },
   { "/Tools/Paint Tools/Blend", NULL, tools_select_cmd_callback, BLEND },
   { "/Tools/Paint Tools/Paintbrush", NULL, tools_select_cmd_callback, PAINTBRUSH },
@@ -262,7 +327,9 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Tools/---", NULL, NULL, 0, "<Separator>" },
   { "/Tools/Toolbox", NULL, toolbox_raise_callback, 0 },
 #endif
+#ifdef TEAR_OFF
   { "/Short cuts/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Short cuts/Increase Brush Radius (+1)", ".", brush_increase_radius, 0 },
   { "/Short cuts/Decrease Brush Radius (-1)", ",", brush_decrease_radius, 0 },
   { "/Short cuts/Increase clone x offset ", ",", clone_x_offset_increase, 0 },
@@ -270,14 +337,18 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Short cuts/Increase clone y offset ", ",", clone_y_offset_increase, 0 },
   { "/Short cuts/Decrease clone y offset ", ",", clone_y_offset_decrease, 0 },
   { "/Short cuts/Reset clone offset ", ",", clone_reset_offset, 0 },
+#ifdef TEAR_OFF
   { "/Filters/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Filters/Repeat last", "<alt>F", filters_repeat_cmd_callback, 0x0 },
   { "/Filters/Re-show last", "<alt><shift>F", filters_repeat_cmd_callback, 0x1 },
   { "/Filters/---", NULL, NULL, 0, "<Separator>" },
 #if 0
   { "/Parsley/", NULL, NULL, 0 },
 #endif
+#ifdef TEAR_OFF
   { "/Dialogs/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
   { "/Dialogs/Brushes...", "<control><shift>B", dialogs_brushes_cmd_callback, 0 },
 #if 0
   { "/Dialogs/Patterns...", "<control><shift>P", dialogs_patterns_cmd_callback, 0 },
@@ -286,15 +357,17 @@ static GtkItemFactoryEntry image_entries[] =
   { "/Dialogs/Gradient Editor...", "<control>G", dialogs_gradient_editor_cmd_callback, 0 },
   { "/Dialogs/Layers & Channels...", "<control>L", dialogs_lc_cmd_callback, 0 },
 #ifdef RnH_STYLE
-  { "/Dialogs/Frame Manager/tearoff1", NULL, NULL, 0, "<Tearoff>" },
-  { "/Dialogs/Frame Manager/create", "<control>F", dialogs_store_frame_manager_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/flip forward", "W", dialogs_store_frame_manager_flip_forward_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/flip backward", "Q", dialogs_store_frame_manager_flip_backwards_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/frame forward", "S", dialogs_store_frame_manager_adv_forward_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/frame backward", "A", dialogs_store_frame_manager_adv_backwards_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/play forward", "Q", dialogs_store_frame_manager_play_forward_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/play backward", "Q", dialogs_store_frame_manager_play_backwards_cmd_callback, 0 },
-  { "/Dialogs/Frame Manager/stop", "Q", dialogs_store_frame_manager_stop_cmd_callback, 0 },
+#ifdef TEAR_OFF
+  { "/Dialogs/Flipbook/tearoff1", NULL, NULL, 0, "<Tearoff>" },
+#endif
+  { "/Dialogs/Flipbook/create", "<control>F", dialogs_store_frame_manager_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/flip forward", "W", dialogs_store_frame_manager_flip_forward_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/flip backward", "Q", dialogs_store_frame_manager_flip_backwards_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/frame forward", "S", dialogs_store_frame_manager_adv_forward_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/frame backward", "A", dialogs_store_frame_manager_adv_backwards_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/play forward", "Q", dialogs_store_frame_manager_play_forward_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/play backward", "Q", dialogs_store_frame_manager_play_backwards_cmd_callback, 0 },
+  { "/Dialogs/Flipbook/stop", "Q", dialogs_store_frame_manager_stop_cmd_callback, 0 },
 #endif
 #if 0
   { "/Dialogs/Indexed Palette...", NULL, dialogs_indexed_palette_cmd_callback, 0 },
@@ -331,10 +404,12 @@ static GtkItemFactoryEntry sfm_store_entries[] =
     {"/Store/Save All", NULL, sfm_store_save_all, 0, NULL},
     {"/Store/Revert", NULL, sfm_store_revert, 0, NULL},
     {"/Store/Change Frame", NULL, sfm_store_change_frame, 0, NULL},
+    {"/View/Slide Show", NULL, sfm_slide_show, 0, NULL},
     {"/Dir/Recent Src", NULL, sfm_store_recent_src, 0, NULL},
     {"/Dir/Recent Dest", NULL, sfm_store_recent_dest, 0, NULL},
-    {"/Dir/Load Smart", NULL, sfm_store_load_smart, 0, "<ToggleItem>"},
-};
+    {"/Dir/Load Smart", NULL, sfm_store_load_smart, 0, "<ToggleItem>"}
+/*    {"/Commands/Filters/Apply Last", NULL, sfm_filter_apply, 0, NULL}
+*/};
 
 static guint n_sfm_store_entries = sizeof (sfm_store_entries) / sizeof (sfm_store_entries[0]);
 static GtkItemFactory *sfm_store_factory = NULL;
