@@ -100,6 +100,8 @@ char *    default_frame_manager = NULL;
 char *    gradient_path = NULL;
 char *    default_gradient = NULL;
 char *    pluginrc_path = NULL;
+char *    cms_profile_path = NULL;
+char *    look_profile_path = NULL;
 int       tile_cache_size = 4194304;  /* 4 MB */
 int       marching_speed = 150;   /* 150 ms */
 double    gamma_val = 1.0;
@@ -131,6 +133,9 @@ Precision default_precision = PRECISION_FLOAT16;
 int       show_tips = TRUE;
 int       last_tip = -1;
 int       show_tool_tips = TRUE;
+int       gamut_alarm_red = 255;
+int       gamut_alarm_green = 0;
+int       gamut_alarm_blue = 255;
 float     monitor_xres = 72.0;
 float     monitor_yres = 72.0;
 int       using_xserver_resolution = FALSE;
@@ -159,6 +164,15 @@ int       layer_channel_visible = FALSE;
 int       color_visible = FALSE;
 int       palette_visible = FALSE;
 int       gradient_visible = FALSE;
+char *    cms_default_image_profile_name = NULL;
+char *    cms_workspace_profile_name = NULL;
+char *    cms_display_profile_name = NULL;
+int       cms_default_intent = INTENT_PERCEPTUAL;
+int       cms_default_flags = 0;
+int       cms_open_action = 0;
+int       cms_mismatch_action = 0;
+int       cms_manage_by_default = 0;
+int       enable_rulers_on_start=TRUE;
 
 static int get_next_token (void);
 static int peek_next_token (void);
@@ -224,7 +238,17 @@ static ParseFunc funcs[] =
   { "palette-path",          TT_PATH,       &palette_path, NULL },
   { "gradient-path",         TT_PATH,       &gradient_path, NULL },
   { "pluginrc-path",         TT_PATH,       &pluginrc_path, NULL },
+  { "cms-profile-path",      TT_PATH,       &cms_profile_path, NULL },
   { "default-brush",         TT_STRING,     &default_brush, NULL },
+  { "cms-default-image-profile-name",TT_STRING,     &cms_default_image_profile_name, NULL },
+  { "cms-workspace-profile-name",TT_STRING,     &cms_workspace_profile_name, NULL },
+  { "cms-display-profile-name",TT_STRING,     &cms_display_profile_name, NULL },
+  { "cms-default-intent",    TT_INT,        &cms_default_intent, NULL },
+  { "cms-default-flags",     TT_INT,        &cms_default_flags, NULL },
+  { "cms-open-action",       TT_INT,        &cms_open_action, NULL },
+  { "cms-mismatch-action",   TT_INT,        &cms_mismatch_action, NULL },
+  { "cms-manage-by-default", TT_BOOLEAN,    &cms_manage_by_default, NULL },
+  { "look-profile-path",     TT_STRING,     &look_profile_path, NULL },
   { "default-pattern",       TT_STRING,     &default_pattern, NULL },
   { "default-palette",       TT_STRING,     &default_palette, NULL },
   { "default-gradient",      TT_STRING,     &default_gradient, NULL },
@@ -281,6 +305,7 @@ static ParseFunc funcs[] =
   { "show-tips",             TT_BOOLEAN,    &show_tips, NULL },
   { "dont-show-tips",        TT_BOOLEAN,    NULL, &show_tips },
   { "enable-rgbm-painting",             TT_BOOLEAN,    &enable_rgbm_painting, NULL },
+  { "enable-rulers-on-start",             TT_BOOLEAN,    &enable_rulers_on_start, NULL },
   { "dont-enable-rgbm-painting",        TT_BOOLEAN,    NULL, &enable_rgbm_painting },
   { "enable-brush-dialog",             TT_BOOLEAN,    &enable_brush_dialog, NULL },
   { "dont-enable-brush-dialog",        TT_BOOLEAN,    NULL, &enable_brush_dialog },
@@ -302,6 +327,9 @@ static ParseFunc funcs[] =
   { "device",                TT_XDEVICE,    NULL, NULL },
   { "show-tool-tips",        TT_BOOLEAN,    &show_tool_tips, NULL },
   { "dont-show-tool-tips",   TT_BOOLEAN,    NULL, &show_tool_tips },
+  { "gamut-alarm-red",   TT_INT,    &gamut_alarm_red, NULL },  
+  { "gamut-alarm-green",   TT_INT,    &gamut_alarm_green, NULL },  
+  { "gamut-alarm-blue",   TT_INT,    &gamut_alarm_blue, NULL }    
 };
 static int nfuncs = sizeof (funcs) / sizeof (funcs[0]);
 
@@ -1905,8 +1933,17 @@ string_to_str (gpointer val1p,
 {
   char *str;
 
-  str = g_malloc (strlen (*((char **)val1p)) + 3);
-  sprintf (str, "%c%s%c", '"', *((char **)val1p), '"');
+  if (*((char **)val1p) != NULL)
+  {   str = g_malloc (strlen (*((char **)val1p)) + 3);
+      sprintf (str, "%c%s%c", '"', *((char **)val1p), '"');
+  } 
+  else
+  {  str = g_malloc (3);
+     str[0]='"';
+     str[1]='"';
+     str[2]='\0';
+  }
+
   return str;
 }
 
