@@ -19,6 +19,15 @@
 #include "rect_selectP.h"
 #include "menus.h"
 #include "../lib/wire/iodebug.h"
+#include "../lib/wire/wirebuffer.h"
+#include "../lib/wire/enums.h"
+#include "plug_in.h"
+#include "undo.h"
+#include "../lib/wire/wirebuffer.h"
+#include "../lib/wire/enums.h"
+#include "plug_in.h"
+#include "spline.h"
+#include "layer_pvt.h"
 
 #include "buttons/play_forward.xpm"
 #include "buttons/play_forward_is.xpm"
@@ -31,6 +40,12 @@
 #include "buttons/stop.xpm"
 #include "buttons/stop_is.xpm"
 
+extern GSList *display_list;
+extern int initial_frames_loaded;
+extern Tool *active_tool;
+extern GSList *display_list;
+extern int initial_frames_loaded;
+
 typedef gint (*CallbackAction) (GtkWidget *, gpointer);
 #define STORE_LIST_WIDTH        200
 #define STORE_LIST_HEIGHT       150
@@ -147,12 +162,10 @@ sfm_create_gui (GDisplay *disp)
     };
 
   gchar *store_col_name[6] = {"RO", "Adv", "F", "Bg", "*", "Filename"};
-
   if (disp->gimage->filename == NULL){
      g_message ("You need to have saved the filename out first, also it should probably be of the form FILE.0001.EXT");
      return;
    }
-
  
   disp->bfm->sfm->accel_group = gtk_accel_group_new ();
 
@@ -160,11 +173,13 @@ sfm_create_gui (GDisplay *disp)
   disp->bfm->sfm->shell = gtk_dialog_new ();
   gtk_window_set_wmclass (GTK_WINDOW (disp->bfm->sfm->shell), "sfm", PROGRAM_NAME);
   gtk_window_set_policy (GTK_WINDOW (disp->bfm->sfm->shell), FALSE, TRUE, FALSE);
-  sprintf (tmp, "Store Frame Manager for %s", disp->gimage->filename);
+  sprintf (tmp, "Flipbook for %s", disp->gimage->filename);
   gtk_window_set_title (GTK_WINDOW (disp->bfm->sfm->shell), tmp);
 
+  /*
   gtk_window_set_transient_for (GTK_WINDOW (disp->bfm->sfm->shell),
       GTK_WINDOW (disp->shell));
+  */
 
   /* handle the wm close signal */
   gtk_signal_connect (GTK_OBJECT (disp->bfm->sfm->shell), "delete_event",
@@ -181,7 +196,7 @@ sfm_create_gui (GDisplay *disp)
   gtk_widget_reparent (menubar, GTK_WIDGET (GTK_DIALOG (disp->bfm->sfm->shell)->vbox));
   gtk_widget_show (menubar);
 
-  gtk_window_add_accel_group (GTK_WINDOW (disp->shell), disp->bfm->sfm->accel_group);
+  // gtk_window_add_accel_group (GTK_DIALOG (disp->shell), disp->bfm->sfm->accel_group);
   
   /* check buttons */
   hbox = gtk_hbox_new (FALSE, 1);
@@ -384,11 +399,11 @@ sfm_create_gui (GDisplay *disp)
   gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
   gtk_widget_show (vbox);
   
-  label = gtk_label_new ("DEST DIR");
+  label = gtk_label_new ("SRC DIR");
   gtk_box_pack_start(GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
   
-  label = gtk_label_new ("SRC DIR");
+  label = gtk_label_new ("DEST DIR");
   gtk_box_pack_start(GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
   
@@ -541,8 +556,10 @@ sfm_flip_backwards (GDisplay* gdisplay)
 {
   store_frame_manager *fm;
   gint row=0;
-  if (sfm_check (gdisplay)) return;
-  
+  if (sfm_check (gdisplay))
+    {
+      return;
+    }
   bfm_onionskin_rm (gdisplay);
 
   if (!gdisplay || row == -1)
@@ -570,8 +587,10 @@ sfm_flip_forward (GDisplay* gdisplay)
 {
   store_frame_manager *fm;
   gint row=0;
-  if (sfm_check (gdisplay)) return;
-  
+  if (sfm_check (gdisplay))
+    {
+      return;
+    }
   bfm_onionskin_rm (gdisplay);
 
   if (!gdisplay || row == -1)
@@ -634,7 +653,10 @@ sfm_adv_backwards (GDisplay *disp)
   GSList *list=NULL, *l=NULL;
   store *item, *i;
 
-  if (sfm_check (disp)) return 1;
+  if (sfm_check (disp)) 
+    { 
+      return 1;
+    }
   fm = disp->bfm->sfm; 
   num_to_adv = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(fm->num_to_adv));
 
@@ -847,7 +869,10 @@ sfm_adv_forward (GDisplay* disp)
 
   GSList *list=NULL, *l=NULL;
   store *item, *i;
-  if (sfm_check (disp)) return 1;
+  if (sfm_check (disp)) 
+    { 
+      return 1;
+    }
   fm = disp->bfm->sfm; 
   num_to_adv = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(fm->num_to_adv));
 
@@ -1149,13 +1174,33 @@ sfm_onionskin_rm (GDisplay* disp)
 static void 
 sfm_store_clear (GDisplay* disp)
 {
+  store_frame_manager *fm;
+  store *item;
+  gint l;
+  
+  fm = disp->bfm->sfm;
+  l = g_slist_length(fm->stores);
+
+  while (l > 0)
+    {
+      item = (store*)g_slist_nth(fm->stores, 0)->data;  
+      if (strcmp(item->gimage->filename, disp->gimage->filename) != 0)
+	gimage_delete(item->gimage);
+      fm->stores = g_slist_remove (fm->stores, 
+				   (g_slist_nth (fm->stores, 0))->data);  
+      l = g_slist_length(fm->stores); 
+   }
+
+  g_free(disp->bfm->sfm);
+  g_free(disp->bfm);
+  disp->bfm=NULL;
 }
 
 void 
 sfm_store_add (GtkWidget *w, gpointer data)
 {
-  bfm_onionskin_rm ((GDisplay*)data);
- sfm_store_add_create_dialog ((GDisplay *)data); 
+  bfm_onionskin_rm((GDisplay *)data);
+  sfm_store_add_create_dialog ((GDisplay *)data); 
 }
 
 static void
@@ -1392,9 +1437,12 @@ sfm_store_recent_dest (GtkWidget *w, gpointer data)
 void 
 sfm_store_load_smart (GtkWidget *w, gpointer data)
 {
+  GDisplay *display;
+
+  display=(GDisplay *)data;
+
   ((GDisplay *)data)->bfm->sfm->load_smart = ((GDisplay *)data)->bfm->sfm->load_smart ? 0 : 1; 
   bfm_onionskin_rm ((GDisplay*)data);
-  
 }
 
 static store* 
@@ -1446,6 +1494,7 @@ sfm_store_make_cur (GDisplay *gdisplay, int row)
   store *item;
   store_frame_manager *fm;
   char tmp[256];
+  SplineTool *spline_tool=NULL;
 
   if (row == -1)
     return;
@@ -1460,6 +1509,18 @@ sfm_store_make_cur (GDisplay *gdisplay, int row)
 
   item->fg = 1;
 
+  /* shut down any spline tools associated with the current display's gimage. */
+  /* added 21December2004 - twh */
+  if(active_tool->type==SPLINE)
+    {
+      if(active_tool->private)
+	{
+	  spline_tool=(SplineTool*)active_tool->private;
+	  if(spline_tool->active_spline && spline_tool->active_spline->drawn)
+	      spline_control(active_tool, PAUSE, gdisplay);
+	}
+    }
+
   /* display the new store */
   gdisplay->gimage = item->gimage;
   gdisplay->ID = item->gimage->ID;
@@ -1472,9 +1533,21 @@ sfm_store_make_cur (GDisplay *gdisplay, int row)
   bfm_set_fg (gdisplay, 
       ((store*)(g_slist_nth (fm->stores, fm->fg)->data))->gimage); 
 
-  sprintf (tmp, "Store Frame Manager for %s", gdisplay->gimage->filename);
+  sprintf (tmp, "Flipbook for %s", gdisplay->gimage->filename);
   gtk_window_set_title (GTK_WINDOW (gdisplay->bfm->sfm->shell), tmp);
-  
+  /* restart any spline tools associated with the current display's new gimage. */
+  /* added 21December2004 - twh */
+  if(active_tool->type==SPLINE)
+    {
+      if(item->gimage->active_layer->sl.spline_tool)
+	{
+	  spline_tool=(SplineTool*)item->gimage->active_layer->sl.spline_tool;
+	  active_tool->private=spline_tool;
+	  if(spline_tool->active_spline && !spline_tool->active_spline->drawn)
+	      spline_control(active_tool, RESUME, gdisplay);
+	}
+    }
+
   gdisplays_flush ();
 }
 
@@ -1824,7 +1897,11 @@ sfm_store_add_stores (GDisplay* disp)
 
 
 	  /* find the current */
-	  if ((gimage=file_load_without_display (temp_path, prune_filename(temp_path), disp)))
+	  if (temp_path ==NULL)
+	    {
+	      g_message("Frame numbers can not be negative.");
+	    }
+	  if ((temp_path!=NULL)&&(gimage=file_load_without_display (temp_path, prune_filename(temp_path), disp)))
 	    {
 	      sfm_store_add_image (gimage, disp, row+(i+1), 0, fm->readonly);
 	    }
@@ -1866,7 +1943,11 @@ sfm_store_add_stores (GDisplay* disp)
 	    }
 
 	  /* find the current */
-	  if ((gimage=file_load_without_display (temp_path, prune_filename(temp_path), disp)))
+	  if (temp_path ==NULL)
+	    {
+	      g_message("Frame numbers can not be negative.");
+	    }
+	  if ((temp_path!=NULL)&&(gimage=file_load_without_display (temp_path, prune_filename(temp_path), disp)))
 	    {
 	      sfm_store_add_image (gimage, disp, row+(i+1), 0, fm->readonly);
 	    }
@@ -2072,7 +2153,51 @@ sfm_store_change_frame_to (GDisplay* disp)
       disp->ID = disp->gimage->ID;
       item->gimage = disp->gimage;
       sfm_store_make_cur (disp, fm->fg);
+void 
+sfm_slide_show (GtkWidget *w, gpointer data)
+{
+  GDisplay *display=NULL;
+  GtkWidget *window;
+  static gint win_x, win_y, win_w, win_h, win_d;
 
+  display = gdisplay_active();
+  window = display->shell;
+
+  if(display)
+    {
+      display->slide_show = display->slide_show ? 0 : 1;
+      if(display->slide_show)
+	{ 
+	  gdk_window_get_geometry(window->window, &win_x, &win_y, &win_w, &win_h, &win_d);
+
+	  gtk_widget_hide(display->menubar);
+	  gtk_widget_hide(display->hsb);
+	  gtk_widget_hide(display->vsb);
+	  gtk_widget_hide(display->hrule);
+	  gtk_widget_hide(display->vrule);
+
+	  gdk_window_move_resize (window->window, -50, -75, gdk_screen_width()+92, 
+				  gdk_screen_height()+105);
+	  gtk_window_set_default_size (GTK_WINDOW (window), gdk_screen_width()+92,
+				       gdk_screen_height()+105);
+	  gtk_widget_set_usize (GTK_WIDGET(window), gdk_screen_width()+92, 
+				gdk_screen_height()+105);
+	}
+      else
+	{
+	  gdk_window_move_resize (window->window, win_x, win_y, win_w, win_h);
+	  gtk_window_set_default_size (GTK_WINDOW (window), win_w, win_h);
+	  gtk_widget_set_usize (GTK_WIDGET(window), win_w, win_h);
+
+	  gtk_widget_show(display->menubar);
+	  gtk_widget_show(display->hsb);
+	  gtk_widget_show(display->vsb);
+	  gtk_widget_show(display->hrule);
+	  gtk_widget_show(display->vrule);
+	}
+      gdisplays_flush();
+    }
+}
       gtk_clist_set_text (GTK_CLIST (disp->bfm->sfm->store_list), disp->bfm->sfm->fg, 
 	  5, temp_name);
 
@@ -2122,6 +2247,45 @@ sfm_store_change_frame_dialog_delete (GtkWidget *w, GdkEvent *e, gpointer data)
 {
   store_frame_manager *fm = ((GDisplay*) data)->bfm->sfm;
 
+void 
+sfm_filter_apply (GtkWidget *w, gpointer data)
+{
+  Argument *args;
+  GDisplay *display=(GDisplay*)data;
+  int i;
+  gint start_frame;
+
+  bfm_onionskin_rm ((GDisplay*)data);
+
+  start_frame=display->bfm->sfm->fg;
+
+  if (wire_buffer->last_plug_in)
+    {
+      /* construct the procedures arguments */
+      args = g_new (Argument, wire_buffer->last_plug_in->num_args);
+      memset (args, 0, (sizeof (Argument) * wire_buffer->last_plug_in->num_args));
+
+      /* initialize the argument types */
+      for (i = 0; i < wire_buffer->last_plug_in->num_args; i++)
+	args[i].arg_type = wire_buffer->last_plug_in->args[i].arg_type;
+
+      sfm_flip_forward(display);
+      while(start_frame != display->bfm->sfm->fg)
+	{
+	  /* initialize the first 3 plug-in arguments  */
+	  args[0].value.pdb_int = RUN_WITH_LAST_VALS;
+	  args[1].value.pdb_int = display->gimage->ID;
+	  args[2].value.pdb_int = drawable_ID (gimage_active_drawable (display->gimage));
+
+	  /* run the plug-in procedure */
+	  plug_in_run (wire_buffer->last_plug_in, args, FALSE, TRUE);
+
+	  /* advance to the next store */
+	  sfm_flip_forward(display);
+	}
+      g_free (args);
+    }
+} 
 
   if (fm)
     {
@@ -2212,4 +2376,46 @@ sfm_store_change_frame_create_dialog (GDisplay *disp)
     }
 }
 
+void 
+sfm_dialog_create ()
+{
+  GDisplay *display;
+  GImage *gimage; 
+  char* temp_path = 0;
+  gint row, f, i;
+  store_frame_manager *fm;
+  store *item;
 
+  if(display_list != NULL)
+    {
+      display=(GDisplay *)display_list->data;
+      bfm_create_sfm(display);
+      fm = display->bfm->sfm;
+      row = fm->fg;
+      item = (store*) g_slist_nth (fm->stores, row)->data;
+      bfm_onionskin_rm(display);
+      f = filename_get_frame_number(item->gimage->filename);
+      for (i=0; i<(initial_frames_loaded-1); i++)
+	{
+	  if (temp_path) 
+	    {
+	      free(temp_path);
+	    }
+	  temp_path = filename_build_path(display->bfm->src_dir,
+					  display->bfm->src_name,
+					  f+(i+1));
+
+	  /* find the current */
+	  if ((gimage=file_load_without_display (temp_path, prune_filename(temp_path), display)))
+	    {
+	      sfm_store_add_image (gimage, display, row+(i+1), 0, fm->readonly);
+	    }
+	}       
+    }
+  else
+    {
+    printf("No gdisplays available.\n");
+    printf("Starting CinePaint without Frame Manger.\n");
+    printf("You may manually create a Frame Manager once you have loaded an image.\n");
+    }  
+}
