@@ -7,7 +7,10 @@
 
 #include "gdisplay.h"
 
+void sfm_slide_show (GtkWidget *w, gpointer data);
+
 void sfm_create_gui (GDisplay *);
+void sfm_dialog_create ();
 void sfm_onionskin_set_offset (GDisplay*, int, int);
 void sfm_onionskin_rm (GDisplay*);
 void sfm_store_add_image (GImage *, GDisplay *, int, int, int);
@@ -24,6 +27,7 @@ void sfm_store_save (GtkWidget *, gpointer);
 void sfm_store_save_all (GtkWidget *, gpointer);
 void sfm_store_revert (GtkWidget *, gpointer);
 void sfm_store_change_frame (GtkWidget *, gpointer);
+void sfm_filter_apply (GtkWidget *, gpointer);
 
 void sfm_set_dir_src (GDisplay *);
 void sfm_set_dir_dest (GDisplay *);
