@@ -56,6 +56,7 @@
 #include "posterize.h"
 #include "rect_select.h"
 #include "smudge.h"
+#include "spline.h"
 #include "text_tool.h"
 #include "threshold.h"
 #include "tools.h"
@@ -95,31 +96,34 @@ ToolInfo tool_info[] =
   { NULL, "Flip", 10 }, /* horizontal */
   { NULL, "Flip", 10 }, /* vertical */
   { NULL, "Text", 11 },
-  { NULL, "Color Picker", 12 },
-  { NULL, "Bucket Fill", 13 },
-  { NULL, "Blend", 14 },
-  { NULL, "Pencil", 15 },
-  { NULL, "Paintbrush", 16 },
-  { NULL, "Eraser", 17 },
-  { NULL, "Airbrush", 18 },
-  { NULL, "Clone", 19 },
-  { NULL, "Convolve", 20 },
-  { NULL, "Dodge Burn", 21 },
-  { NULL, "Smudge", 22 },
-  { NULL, "Measure", 23 },
+  { NULL, "Spline", 12 },
+  { NULL, "Color Picker", 13 },
+  { NULL, "Bucket Fill", 14 },
+  { NULL, "Blend", 15 },
+  { NULL, "Pencil", 16 },
+  { NULL, "Paintbrush", 17 },
+  { NULL, "Eraser", 18 },
+  { NULL, "Airbrush", 19 },
+  { NULL, "Clone", 20 },
+  { NULL, "Convolve", 21 },
+  { NULL, "Dodge Burn", 22 },
+  { NULL, "Smudge", 23 },
+  { NULL, "Measure", 24 },
+  { NULL, "Spline Edit", 25 },
 
   /*  Non-toolbox tools  */
-  { NULL, "By Color Select", 24 },
-  { NULL, "Color Balance", 25 },
-  { NULL, "Brightness-Contrast", 26 },
-  { NULL, "Hue-Saturation", 27 },
-  { NULL, "Posterize", 28 },
-  { NULL, "Threshold", 29 },
-  { NULL, "Curves", 30 },
-  { NULL, "Levels", 31 },
-  { NULL, "Histogram", 32 },
-  { NULL, "Gamma-Expose", 33 },
-  { NULL, "Expose-Image", 34 }
+  { NULL, "By Color Select", 26 },
+  { NULL, "Color Balance", 27 },
+  { NULL, "Brightness-Contrast", 28 },
+  { NULL, "Hue-Saturation", 29 },
+  { NULL, "Posterize", 30 },
+  { NULL, "Threshold", 31 },
+  { NULL, "Curves", 32 },
+  { NULL, "Levels", 33 },
+  { NULL, "Histogram", 34 },
+  { NULL, "Gamma-Expose", 35 },
+  { NULL, "Expose-Image", 36 },
+  { NULL, "Splines", 37 }
 };
 #endif
 
@@ -309,6 +313,11 @@ active_tool_free (void)
     case EXPOSE_IMAGE:
       tools_free_expose_image (active_tool);
       break;
+    case SPLINE:
+      tools_free_spline(active_tool);
+    case SPLINE_EDIT:
+      tools_free_spline(active_tool);
+      break;
     default:
       return;
     }
@@ -448,6 +457,12 @@ tools_select (ToolType type)
     case EXPOSE_IMAGE:
       active_tool = tools_new_expose_image ();
       break;
+    case SPLINE:
+      active_tool=tools_new_spline();
+      break;
+    case SPLINE_EDIT:
+      active_tool=tools_new_spline_edit();
+      break;
     default:
 	active_tool = tools_new_measure_tool();
       return;
@@ -569,6 +584,12 @@ tools_initialize (ToolType type, GDisplay *gdisp_ptr)
     case SMUDGE:
       active_tool = tools_new_smudge();
       break;
+    case SPLINE:
+      active_tool=tools_new_spline();
+      break;
+    case SPLINE_EDIT:
+      active_tool=tools_new_spline_edit();
+      break;
 
     case BY_COLOR_SELECT:
       if (gdisp) {
