@@ -143,7 +143,11 @@ export_convert_indexed (gint32  image_ID,
   gint32 nlayers;
   
   /* check alpha */
+#ifdef WIN32
+  g2_free (gimp_image_get_layers (image_ID, &nlayers));
+#else
   g_free (gimp_image_get_layers (image_ID, &nlayers));
+#endif
   if (nlayers > 1 || gimp_drawable_has_alpha (*drawable_ID))
     gimp_image_convert_indexed (image_ID, GIMP_FS_DITHER, GIMP_MAKE_PALETTE, 255, FALSE, FALSE, "");
   else
