@@ -834,3 +834,256 @@ gimp_bfm_set_dir_dest (gint32  disp_ID, char   *name)
 }
 
 
+void
+gimp_image_set_icc_profile_by_name (gint32  image_ID,
+                                    gchar  *data)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  int size = strlen(data)+1;
+
+  return_vals = gimp_run_procedure ("gimp_image_set_icc_profile_by_name",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, size,
+                                    PARAM_INT8ARRAY, data,
+                                    PARAM_END);
+
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+}
+
+void
+gimp_image_set_icc_profile_by_mem  (gint32  image_ID,
+		                    gint    size,
+                                    gchar  *data)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+
+
+  return_vals = gimp_run_procedure ("gimp_image_set_icc_profile_by_mem",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_INT32, size,
+                                    PARAM_INT8ARRAY, data,
+                                    PARAM_END);
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+}
+
+void
+gimp_image_set_lab_profile  (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+
+
+  return_vals = gimp_run_procedure ("gimp_image_set_lab_profile",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+}
+
+void
+gimp_image_set_xyz_profile  (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+
+
+  return_vals = gimp_run_procedure ("gimp_image_set_xyz_profile",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+}
+
+void
+gimp_image_set_srgb_profile (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+
+
+  return_vals = gimp_run_procedure ("gimp_image_set_srgb_profile",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+}
+
+gboolean
+gimp_image_has_icc_profile         (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  int has_profile = FALSE;
+
+  return_vals = gimp_run_procedure ("gimp_image_has_icc_profile",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      has_profile = return_vals[1].data.d_int32;
+    }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+
+  return has_profile;
+}
+
+char*
+gimp_image_get_icc_profile_by_mem  (gint32  image_ID,
+		                    gint   *size)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  char *data=NULL;
+
+  return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_by_mem",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      *size = return_vals[1].data.d_int32;
+      data = calloc (sizeof (char), *size);
+      memcpy (data, return_vals[2].data.d_int8array, *size);
+    }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return data;
+}
+
+
+char*
+gimp_image_get_icc_profile_info    (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  char *data=NULL;
+  int  size;
+
+  return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_info",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      size = return_vals[1].data.d_int32;
+      data = calloc (sizeof (char), size);
+      memcpy (data, return_vals[2].data.d_int8array, size);
+   }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return data;
+}
+
+
+char*
+gimp_image_get_icc_profile_description    (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  char *data=NULL;
+  int  size;
+
+  return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_description",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      size = return_vals[1].data.d_int32;
+      data = calloc (sizeof (char), size);
+      memcpy (data, return_vals[2].data.d_int8array, size);
+   }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return data;
+}
+
+
+char*
+gimp_image_get_icc_profile_pcs     (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  char *data=NULL;
+  int  size;
+
+  return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_pcs",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      size = return_vals[1].data.d_int32;
+      data = calloc (sizeof (char), size);
+      memcpy (data, return_vals[2].data.d_int8array, size);
+    }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return data;
+}
+
+
+char*
+gimp_image_get_icc_profile_color_space_name (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  char *data=NULL;
+  int  size;
+
+  return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_color_space_name",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      size = return_vals[1].data.d_int32;
+      data = calloc (sizeof (char), size);
+      memcpy (data, return_vals[2].data.d_int8array, size);
+    }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return data;
+}
+
+char*
+gimp_image_get_icc_profile_device_class_name (gint32  image_ID)
+{
+  GParam *return_vals;
+  int nreturn_vals;
+  char *data=NULL;
+  int  size;
+
+  return_vals = gimp_run_procedure ("gimp_image_get_icc_profile_device_class_name",
+                                    &nreturn_vals,
+                                    PARAM_IMAGE, image_ID,
+                                    PARAM_END);
+
+  if (return_vals[0].data.d_status == STATUS_SUCCESS)
+    {
+      size = return_vals[1].data.d_int32;
+      data = calloc (sizeof (char), size);
+      memcpy (data, return_vals[2].data.d_int8array, size);
+    }
+
+  gimp_destroy_params (return_vals, nreturn_vals);
+  return data;
+}
+
