@@ -262,6 +262,10 @@ gint       gimp_image_get_component_active  (gint32     image_ID,
 					     GimpChannelType       component);
 gint       gimp_image_get_component_visible (gint32     image_ID,
 					     GimpChannelType       component);
+gboolean   gimp_image_get_resolution        (gint32     image_ID,
+                                             gdouble    *xresolution,
+                                             gdouble    *yresolution);
+
 DLL_API char*      gimp_image_get_filename          (gint32     image_ID);
 DLL_API gint32*    gimp_image_get_layers            (gint32     image_ID,
 					     gint      *nlayers);
@@ -284,6 +288,22 @@ DLL_API void       gimp_image_set_filename          (gint32     image_ID,
 void gimp_bfm_set_dir_src    (gint32     disp_ID, char *filename);
 void gimp_bfm_set_dir_dest    (gint32     disp_ID, char *filename);
 
+void       gimp_image_set_icc_profile_by_name (gint32   image_ID,
+                                               gchar   *data);
+void       gimp_image_set_icc_profile_by_mem  (gint32   image_ID,
+                                               gint     size,
+                                               gchar   *data);
+void       gimp_image_set_lab_profile         (gint32   image_ID);
+void       gimp_image_set_xyz_profile         (gint32   image_ID);
+void       gimp_image_set_srgb_profile        (gint32   image_ID);
+gboolean   gimp_image_has_icc_profile         (gint32   image_ID);
+char*      gimp_image_get_icc_profile_by_mem  (gint32   image_ID,
+                                               gint    *size);
+char*      gimp_image_get_icc_profile_description (gint32  image_ID);
+char*      gimp_image_get_icc_profile_info    (gint32   image_ID);
+char*      gimp_image_get_icc_profile_pcs     (gint32   image_ID);
+char*      gimp_image_get_icc_profile_color_space_name  (gint32   image_ID);
+char*      gimp_image_get_icc_profile_device_class_name (gint32   image_ID);
 
 /****************************************
  *             Displays                 *
@@ -295,7 +315,18 @@ gint32 gimp_display_active  ();
 void   gimp_display_delete (gint32 display_ID);
 void   gimp_displays_delete_image (gint32 image_ID);
 DLL_API void   gimp_displays_flush (void);
-
+gint32     gimp_display_get_cms_intent        (gint32   display_ID);
+void       gimp_display_set_cms_intent        (gint32   display_ID,
+                                               gint32   intent);
+gint32     gimp_display_get_cms_flags         (gint32   display_ID);
+void       gimp_display_set_cms_flags         (gint32   display_ID,
+                                               gint32   flags);
+gint32     gimp_display_is_colormanaged       (gint32   display_ID);
+void       gimp_display_set_colormanaged      (gint32   display_ID,
+                                               gboolean True_False);
+void       gimp_display_all_set_colormanaged  (gboolean True_False);
+void       gimp_display_image_set_colormanaged(gint32   display_ID,
+                                               gboolean True_False);
 
 /****************************************
  *              Layers                  *
