@@ -35,6 +35,11 @@
 
 #include "intl.h"
 
+#ifdef WIN32
+#undef g_free
+#define g_free g2_free
+#endif
+
 
 /*
  *  Widget Constructors
