@@ -18,6 +18,9 @@ DLL_API const char* GetDirDot();
 DLL_API const char* GetDirHome();
 DLL_API const char* GetDirData();
 
+/* a function for obtaining an valid path with an given rcfile name */
+const char* gimp_personal_rc_file (const char* rcfile);
+
 /*#else
 
 #define DATA_DIR "share"
