@@ -336,7 +336,7 @@ cineonGetRowBytes(CineonFile* cineon, unsigned short* row, int y) {
   /* convert longwords to pixels */
   pixelIndex = 0;
   for (longIndex = 0; longIndex < cineon->lineBufferLength; ++longIndex) {
-    unsigned long t = ntohl(cineon->lineBuffer[longIndex]);
+    unsigned int t = ntohl(cineon->lineBuffer[longIndex]);
     t = t >> 2;
     cineon->pixelBuffer[pixelIndex+2] = CAST(unsigned short) t & 0x3ff;
     t = t >> 10;
@@ -372,7 +372,7 @@ cineonSetRowBytes(CineonFile* cineon, const unsigned short* row, int y) {
   /* pack into longwords */
   pixelIndex = 0;
   for (longIndex = 0; longIndex < cineon->lineBufferLength; ++longIndex) {
-    unsigned long t =
+    unsigned int t =
         (cineon->pixelBuffer[pixelIndex] << 22) |
         (cineon->pixelBuffer[pixelIndex+1] << 12) |
         (cineon->pixelBuffer[pixelIndex+2] << 2);
@@ -433,7 +433,7 @@ cineonGetRow(CineonFile* cineon, unsigned short* row, int y) {
   /* convert longwords to pixels */
   pixelIndex = 0;
   for (longIndex = 0; longIndex < cineon->lineBufferLength; ++longIndex) {
-    unsigned long t = ntohl(cineon->lineBuffer[longIndex]);
+    unsigned int t = ntohl(cineon->lineBuffer[longIndex]);
     t = t >> 2;
     row[pixelIndex+2] = CAST(unsigned short) t & 0x3ff;
     t = t >> 10;
@@ -457,7 +457,7 @@ cineonSetRow(CineonFile* cineon, const unsigned short* row, int y) {
   /* pack into longwords */
   pixelIndex = 0;
   for (longIndex = 0; longIndex < cineon->lineBufferLength; ++longIndex) {
-    unsigned long t =
+    unsigned int t =
         (row[pixelIndex] << 22) |
         (row[pixelIndex+1] << 12) |
         (row[pixelIndex+2] << 2);
