@@ -32,6 +32,11 @@
 #include "gtk/gtk.h"
 #include "lib/plugin_main.h"
 
+#include <stdio.h>
+#include "lcms.h"
+#include "iccjpeg.h"
+
+
 #define SCALE_WIDTH 125
 
 typedef struct
@@ -319,6 +324,8 @@ load_image (char *filename)
 
   jpeg_stdio_src (&cinfo, infile);
 
+  setup_read_icc_profile(&cinfo);  
+
   /* Step 3: read file parameters with jpeg_read_header() */
 
   (void) jpeg_read_header (&cinfo, TRUE);
@@ -406,6 +413,27 @@ load_image (char *filename)
       gimp_progress_update ((double) cinfo.output_scanline / (double) cinfo.output_height);
     }
 
+  // Step 6a: read icc profile
+  {
+    LPBYTE Buffer = NULL;
+    size_t Len = 0;
+    cmsHPROFILE hProfile=NULL;
+
+    if (read_icc_profile(&cinfo, &Buffer, &Len))
+    {
+      hProfile = cmsOpenProfileFromMem(Buffer, Len);
+      printf ("%s:%d %s() profile found\n",__FILE__,__LINE__,__func__);
+      if (hProfile) {
+        gimp_image_set_icc_profile_by_mem (image_ID, Len, Buffer);
+        cmsCloseProfile (hProfile);
+        free(Buffer);
+        printf ("%s:%d %s() set embedded icc profile\n",__FILE__,__LINE__,__func__);
+      }
+    } else {
+      printf ("%d %d %d\n",(int)Buffer, (int)Len, (int)hProfile);
+    }
+  }
+
   /* Step 7: Finish decompression */
 
   jpeg_finish_decompress (&cinfo);
@@ -571,6 +599,20 @@ save_image (char   *filename,
    */
   jpeg_start_compress (&cinfo, TRUE);
 
+  // Step 4a: write icc profile
+  {
+    char *buffer = NULL;
+    int size = 0;
+
+    if (gimp_image_has_icc_profile (image_ID)) { 
+      buffer = gimp_image_get_icc_profile_by_mem (image_ID, &size);
+      if (buffer) {
+        write_icc_profile (&cinfo, buffer, size);
+        printf ("%s:%d %s() embedded icc profile\n",__FILE__,__LINE__,__func__);
+      }
+    }
+  }
+
   /* Step 5: while (scan lines remain to be written) */
   /*           jpeg_write_scanlines(...); */
 
