@@ -1,5 +1,5 @@
 /*
- * "$Id: png.c,v 1.1 2004/02/10 01:13:08 robinrowe Exp $"
+ * "$Id: png.c,v 1.5 2005/05/24 18:57:49 beku Exp $"
  *
  *   Portable Network Graphics (PNG) plug-in for The GIMP -- an image
  *   manipulation program
@@ -55,6 +55,7 @@
 #include "../../lib/ui.h"
 #include "../../lib/wire/iodebug.h"
 #include "../../lib/dialog.h"
+#include "../../lib/widgets.h"
 
 #include <png.h>		/* PNG library definitions */
 
@@ -585,6 +586,8 @@ load_image (gchar *filename)	/* I - File to load */
     gimp_quit();
   };
 
+  gimp_image_set_filename(image, filename);
+
  /*
   * Create the "background" layer to hold the image...
   */
@@ -615,8 +618,6 @@ load_image (gchar *filename)	/* I - File to load */
   }
 #endif /* PNG_LIBPNG_VER > 99 */
 
-  gimp_image_set_filename(image, filename);
-
  /*
   * Load the colormap as necessary...
   */
@@ -698,6 +699,17 @@ load_image (gchar *filename)	/* I - File to load */
 	  };
   };
 
+#if defined(PNG_iCCP_SUPPORTED)
+  /* set icc profile */
+  if (info->iccp_proflen > 0) {
+    gimp_image_set_icc_profile_by_mem (image, info->iccp_proflen,
+                                              info->iccp_profile);
+    printf ("%s:%d %s() set embedded profile \"%s\"\n",
+             __FILE__,__LINE__,__func__,
+                                              info->iccp_name);
+  }
+#endif
+
  /*
   * Done with the file...
   */
@@ -944,6 +956,21 @@ save_image (gchar  *filename,	        /* I - File to save to */
     /* otherwise the default is fine */
   }
 
+  // write icc profile
+#if defined(PNG_iCCP_SUPPORTED)
+  if (gimp_image_has_icc_profile (image_ID)) {
+    int size;
+    char *buffer;
+    
+    buffer = gimp_image_get_icc_profile_by_mem (image_ID, &size);
+    png_set_iCCP (pp, info, gimp_image_get_icc_profile_description (image_ID),
+                  0, buffer, size);
+    printf ("%s:%d %s() embedd icc profile \"%s\"\n",
+             __FILE__,__LINE__,__func__,
+                                              info->iccp_name);
+  }
+#endif
+
   /* All this stuff is optional extras, if the user is aiming for smallest
      possible file size she can turn them all off */
 
@@ -1155,6 +1182,8 @@ gimp_dialog_delete_callback (GtkWidget *widget,
 }
 
 
+#if 0
+
 void
 gimp_dialog_create_action_areav (GtkDialog *dialog,
 				 va_list    args)
@@ -1250,8 +1279,6 @@ gimp_dialog_create_action_areav (GtkDialog *dialog,
     }
 }
 
-#if 0
-
 GtkWidget *
 gimp_dialog_newv (const gchar       *title,
 		const gchar       *wmclass_name,
@@ -1335,7 +1362,7 @@ gimp_dialog_new (const gchar       *title,
  * button's inverse "active" state by attaching widgets with the
  * "inverse_sensitive" key.
  **/
-void
+/*void
 gimp_toggle_button_sensitive_update (GtkToggleButton *toggle_button)
 {
   GtkWidget *set_sensitive;
@@ -1360,7 +1387,7 @@ gimp_toggle_button_sensitive_update (GtkToggleButton *toggle_button)
       set_sensitive =
         gtk_object_get_data (GTK_OBJECT (set_sensitive), "inverse_sensitive");
     }
-}
+}*/
 
 
 /**
@@ -1371,7 +1398,7 @@ gimp_toggle_button_sensitive_update (GtkToggleButton *toggle_button)
  *
  * Note that this function calls gimp_toggle_button_sensitive_update().
  **/
-void
+/*void
 gimp_toggle_button_update (GtkWidget *widget,
 			   gpointer   data)
 {
@@ -1385,7 +1412,7 @@ gimp_toggle_button_update (GtkWidget *widget,
     *toggle_val = FALSE;
 
   gimp_toggle_button_sensitive_update (GTK_TOGGLE_BUTTON (widget));
-}
+}*/
 
 
 /**
@@ -1404,7 +1431,7 @@ gimp_toggle_button_update (GtkWidget *widget,
  * Note that the @label_text can be #NULL and that the widget will be attached
  * starting at (@column + 1) in this case, too.
  **/
-void
+/*void
 gimp_table_attach_aligned (GtkTable    *table,
 			   gint         column,
 			   gint         row,
@@ -1446,7 +1473,7 @@ gimp_table_attach_aligned (GtkTable    *table,
 		    GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0);
 
   gtk_widget_show (widget);
-}
+}*/
 
 /**
  * gimp_int_adjustment_update:
@@ -1457,7 +1484,7 @@ gimp_table_attach_aligned (GtkTable    *table,
  * Note that the #GtkAdjustment's value (which is a #gfloat) will be rounded
  * with RINT().
  **/
-void
+/*void
 gimp_int_adjustment_update (GtkAdjustment *adjustment,
 			    gpointer       data)
 {
@@ -1465,7 +1492,7 @@ gimp_int_adjustment_update (GtkAdjustment *adjustment,
 
   val = (gint *) data;
   *val = rint(adjustment->value);
-}
+}*/
 
 
 static gint
