@@ -1,4 +1,4 @@
-/* $Id: unsharp.c,v 1.2 2004/03/26 22:22:55 robinrowe Exp $
+/* $Id: unsharp.c,v 1.5 2005/05/24 18:58:30 beku Exp $
  *
  * unsharp.c 0.10 -- This is a plug-in for the GIMP 1.0
  *  http://www.stdout.org/~winston/gimp/unsharp.html
@@ -38,6 +38,7 @@
 
 #ifndef GIMP_VERSION
 #include "lib/rint.h"
+#include "lib/widgets.h"
 #endif
 
 #define PLUG_IN_VERSION "0.10"
@@ -859,6 +860,7 @@ gimp_dialog_delete_callback (GtkWidget *widget,
   return TRUE;
 }
 
+#if 0
 void
 gimp_dialog_create_action_areav (GtkDialog *dialog,
 				 va_list    args)
@@ -953,7 +955,7 @@ gimp_dialog_create_action_areav (GtkDialog *dialog,
       label = va_arg (args, gchar *);
     }
 }
-#if 0
+
 GtkWidget *
 gimp_dialog_newv (const gchar       *title,
 		const gchar       *wmclass_name,
@@ -1048,8 +1050,8 @@ gimp_scale_entry_unconstrained_adjustment_callback (GtkAdjustment *adjustment,
  *
  * Returns: A #GtkSpinbutton and it's #GtkAdjustment.
  **/
-GtkWidget *
-gimp_spin_button_new (GtkObject **adjustment,  /* return value */
+/*GtkWidget *
+gimp_spin_button_new (GtkObject **adjustment,
 		      gfloat      value,
 		      gfloat      lower,
 		      gfloat      upper,
@@ -1072,7 +1074,7 @@ gimp_spin_button_new (GtkObject **adjustment,  /* return value */
   gtk_widget_set_usize (spinbutton, 75, -1);
 
   return spinbutton;
-}
+}*/
 
 
 /**
@@ -1107,7 +1109,7 @@ gimp_spin_button_new (GtkObject **adjustment,  /* return value */
  *
  * Returns: The #GtkSpinButton's #GtkAdjustment.
  **/
-GtkObject *
+/*GtkObject *
 gimp_scale_entry_new (GtkTable    *table,
 		      gint         column,
 		      gint         row,
@@ -1207,7 +1209,7 @@ gimp_scale_entry_new (GtkTable    *table,
   gtk_object_set_data (GTK_OBJECT (return_adj), "spinbutton", spinbutton);
 
   return return_adj;
-}
+}*/
 
 /**
  * gimp_double_adjustment_update:
@@ -1215,7 +1217,7 @@ gimp_scale_entry_new (GtkTable    *table,
  * @data:       A pointer to a #gdouble variable which will store the
  *              @adjustment's value.
  **/
-void
+/*void
 gimp_double_adjustment_update (GtkAdjustment *adjustment,
 			       gpointer       data)
 {
@@ -1223,7 +1225,7 @@ gimp_double_adjustment_update (GtkAdjustment *adjustment,
 
   val = (gdouble *) data;
   *val = adjustment->value;
-}
+}*/
 
 #ifdef GIMP_VERSION
 #define RINT(x) rint(x)
@@ -1238,7 +1240,7 @@ gimp_double_adjustment_update (GtkAdjustment *adjustment,
  * Note that the #GtkAdjustment's value (which is a #gfloat) will be rounded
  * with RINT().
  **/
-void
+/*void
 gimp_int_adjustment_update (GtkAdjustment *adjustment,
 			    gpointer       data)
 {
@@ -1246,7 +1248,7 @@ gimp_int_adjustment_update (GtkAdjustment *adjustment,
 
   val = (gint *) data;
   *val = RINT (adjustment->value);
-}
+}*/
 
 
 /* ------------------------ unsharp_mask_dialog ----------------------- */
@@ -1261,7 +1263,7 @@ unsharp_mask_dialog (void)
   gimp_ui_init ("unsharp", TRUE);
 
   window = gimp_dialog_new ("Unsharp Mask", "unsharp",
-			    /*gimp_standard_help_func, "filters/unsharp.html",*/
+			    gimp_standard_help_func, "filters/unsharp.html",
 			    GTK_WIN_POS_MOUSE,
 			    FALSE, TRUE, FALSE,
 
